##############################################################################
#
# 	Copyright (c) GeoWorks 1989 -- All Rights Reserved
#
# PROJECT:	PC GEOS
# MODULE:	Swat System Library -- File module error handling
# FILE: 	file-err.tcl
# AUTHOR: 	Adam de Boor, Aug 14, 1989
#
# COMMANDS:
# 	Name			Description
#	----			-----------
#   	file-err    	    	Locates a file
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	8/14/89		Initial Revision
#
# DESCRIPTION:
#	Routines for use by the File module when searching for a patient.
#
#	$Id: file-err.tcl,v 3.145 94/01/21 16:47:30 jimmy Exp $
#
###############################################################################

#
# Variable holding the pattern against which to compare a path to see if it's
# absolute.
#
defvar file-abspat {}

#
# Set up the initial load-path variable inside a subroutine to avoid leaving
# extra variables lying around. Makes sure the lib directory appears at the
# end of the path.
#
[defsubr setup-load-path {}
{
    global load-path file-devel-dir file-syslib-dir file-root-dir file-os

    var lp [getenv SWATPATH] load-path {}
    
    if {[string c ${file-os} unix] != 0} {
    	# translate backslashes to forward slashes from DOS
    	var lp [string subst $lp \\ / global]
    }
    for {} {![null $lp]} {} {
    	if {[string c ${file-os} unix] == 0} {
    	    var colon [string first : $lp]
    	} else {
	    var colon [string first ; $lp]
    	}
	if {$colon < 0} {
	    var load-path [concat ${load-path} [file expand $lp]]
	    break
	} else {
	    var load-path [concat ${load-path} 
				  [file expand 
				    [range $lp 0 [expr $colon-1] chars]]]
	    var lp [range $lp [expr $colon+1] end chars]
	}
    }
    global file-abspat
    if {[string c ${file-os} unix] == 0} {
    	var file-abspat /*
    } else {
    	var file-abspat {?:[\\/]*}
    }
    if {[string match ${file-syslib-dir} ${file-abspat}]} {
    	#
	# If file-syslib-dir is absolute, but actually within our
	# domain (it starts with file-root-dir), see if there's one in the
	# user's development tree at which we should be looking.
	#
	if {[string first ${file-root-dir} ${file-syslib-dir}] == 0} {
    	    #
	    # Yup. Figure the subdirectory below the root and see if that
	    # subdirectory exists in the development tree.
	    #
    	    var subdir [range ${file-syslib-dir}
	    	    	    [length ${file-root-dir} chars]
			    end chars]
	    var dir ${file-devel-dir}${subdir}
	    if {[file exists $dir]} {
    	    	#
		# Yup. Tack it onto the end of the load path.
		#
	    	var load-path [concat ${load-path} ${dir}]
    	    } else {
    	    	#
		# Trim off the first component and try again. file-syslib-dir
		# may be in a branch tree.
		#
	    	var subdir [range ${subdir}
				   [expr [string first /
						 [range $subdir 1 end chars]]+1]
				   end chars]
		var dir ${file-devel-dir}${subdir}
		if {[file exists $dir]} {
		    var load-path [concat ${load-path} ${dir}]
    	    	}
    	    }
    	}
    	var load-path [concat ${load-path} ${file-syslib-dir}]
    } elif {[null ${file-devel-dir}]} {
	var load-path [concat ${load-path} ${file-root-dir}/${file-syslib-dir}]
    } else {
    	var load-path [concat ${load-path}
			      ${file-devel-dir}/${file-syslib-dir}
			      ${file-root-dir}/${file-syslib-dir}]
    }
}]
setup-load-path


##############################################################################
#				get-load-path
##############################################################################
#
# SYNOPSIS:	    Return the list of directories in which to search for
#   	    	    a .tcl file. This exists so users can dynamically
#		    alter the load-path based on the current patient.
# PASS:		    nothing
# CALLED BY:	    load
# RETURN:	    list of directories in which to search
# SIDE EFFECTS:	    none
#
# STRATEGY
#   	    The default behaviour of this command is to simply return
#   	    the load-path variable's value.
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	12/ 3/91	Initial Revision
#
##############################################################################
[defsubr get-load-path {}
{
    global load-path
    return ${load-path}
}]

##############################################################################
#				load
##############################################################################
#
# SYNOPSIS:	Load a file of Tcl commands
# PASS:		file	= name of file to load
# CALLED BY:	user
# RETURN:	whatever the source command returns.
# SIDE EFFECTS:	?
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	12/ 3/91	Initial Revision
#
##############################################################################
[defcommand load {file} {swat_prog.load swat_prog.file}
{Usage:
    load <file>

Examples:
    "load print"    	Searches for a file called "print" or "print.tcl"
			and loads it.

Synopsis:
    This loads a file of Tcl commands, evaluating each command in turn until
    the last is evaluated, or an error is detected.

Notes:
    * The directories in which this command searches are the elements of the
      list returned by the "get-load-path" command. Unless you've redefined
      this command, this list is taken from the global "load-path" variable.

    * The load-path variable is initialized from the SWATPATH environment
      variable, which is in the form <dir1>:<dir2>:...:<dirn>. The Swat
      system library directory (both in your development tree and in the
      installed source tree) is automatically appended to this path, so you
      needn't include it yourself.

    * Directories may contain ~ or ~<user-name> at their start. (UNIX only)

    * When searching, both <file> and <file>.tcl are checked for.

See also:
    autoload, source
}
{
    #
    # If the file is absolute, then pass it to source as-is. Note that we
    # use "uplevel 0" to make sure the file being loaded is operating in
    # the global context at all times.
    #
    global file-os file-abspat

    [if {[string match $file /*] ||
	 ([string c ${file-os} dos] == 0 && [string match $file ?:*])}
    {
    	return [uplevel 0 [format {source %s} $file]]
    }]
    #
    # Loop through all the directories returned by get-load-path, trying the
    # file as-is (i.e. from Swat's current directory) first.
    #
    if {[string match $file *.tcl]} {
    	var cfile [file rootname $file].tlc
    } else {
    	var cfile $file.tlc
    }

    foreach dir [concat . [get-load-path]] {
    	var dir [file expand $dir]
	var f [file-mangle-for-dos $dir/$file]
	var cf [file-mangle-for-dos $dir/$cfile]
    	[if {[file isfile $f] && [file readable $f]} {
    	    #
	    # The file is there and readable, so source it in the global
	    # context.
	    #
    	    if {[file isfile $cf] && [file newer $cf $f]} {
	    	return [uplevel 0 bc fload $cf]
	    } else {
	    	return [uplevel 0 source $f]
    	    }
    	} elif {[file isfile $f.tcl] && [file readable $f.tcl]}
    	{
    	    #
	    # The file.tcl is there and readable, so source it in the global
	    # context.
	    #
	    if {[file isfile $f.tlc] && [file newer $f.tlc $f.tcl]} {
	    	return [uplevel 0 bc fload $f.tlc]
    	    } else {
	    	return [uplevel 0 source $f.tcl]
    	    }
    	} elif {[file isfile $cf] && [file readable $cf]} {
    	    # if there is only a tlc file the use it
    	    	return [uplevel 0 bc fload $cf]
    	} elif {[file isfile $f.tlc] && [file readable $f.tlc]} {
    	    # if there is only a tlc file the use it
    	    	return [uplevel 0 bc fload $f.tlc]
    	}]
    }
    error [format {load: can't find %s} $file]
}]

##############################################################################
#				file-mangle-for-dos
##############################################################################
#
# SYNOPSIS:	Ensure the file name is 8.3, so DOS doesn't reject it
#   	    	spuriously.
# PASS:		file	= absolute path of file
# CALLED BY:	INTERNAL
# RETURN:	path with all things trimmed to be 8.3 names
# SIDE EFFECTS:	none
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	8/14/92		Initial Revision
#
##############################################################################
[defsubr file-mangle-for-dos {file}
{
    global file-os
    
    if {[string c ${file-os} dos]} {
    	# no mangling required
	return $file
    } else {
    	return [mapconcat f [explode $file /\\] {
	    if {[string match $f *:]} {
	    	var f
    	    } else {
	    	[format {/%s%s} [range [file root $f] 0 7 char]
		    [range [file ext $f] 0 3 char]]
    	    }
    	}]
    }
}]
		

##############################################################################
# Error-handling routine for File module when a patient's executable cannot
# be located. Prompts for the location, with file-name completion, and
# returns either the name of an existing file or the null string, if the
# user has requested we ignore the thing.
#
[defsubr file-err {patient maydetach mayignore}
{
    require top-level-read toplevel
    global file-init-dir file-os file-abspat

    # Figure the name of the patient w/o the extension or trailing spaces
    var pname [index [range $patient 0 7 char] 0]

    # If the patient is listed in the "SWATIGNORE" envariable, ignore it,
    # since we can't seem to locate it.
    foreach i [getenv SWATIGNORE] {
	if {[string c $i $pname] == 0} {
	    return {}
	}
    }

    echo Can't find symbol file for "$patient" (version mismatch?)
    echo Answer "quit" to exit to the shell
    if {$maydetach} {echo Answer "detach" to detach and return to top level}
    if {$mayignore} {echo Answer "ignore" to ignore this patient}
    
    for {} {1} {} {
    	var file [top-level-read {Where is it? } ${file-init-dir}/ 0]

    	#
	# Handle lazy initial-value override, looking for any double slash,
	# which indicates a switch to a different absolute path, or a /~,
    	# which indicates a switch to someone's home directory. $i is left
	# holding the index of the first character to use in $file, defaulting
	# to 0 if neither // nor /~ is present.
	#
    	var // [string last // $file] /~ [string last /~ $file] i 0
    	if {${//} >= 0} {
	    if {${//} > ${/~}} {
	    	var i ${//}
	    } else {
	    	var i ${/~}
	    }
	} elif {${/~} >= 0} {
	    var i ${/~}
	}
	
    	#
	# Deal with any home-directory specs in the file as well as trimming the
	# file as indicated by the above checks.
	#
    	var file [file expand [range $file $i end char]]

    	#
	# Deal with drive specifier under DOS
	#
    	if {[string c ${file-os} dos] == 0} {
	    var i [string last : $file]
	    if {$i > 0} {
	    	var file [range $file [expr $i-1] end char]
    	    }
    	}

    	#
	# See if the line given is one of the special commands we accept
	#
    	var check {}
    	[case [index $file 0] in
    	 q* {var check quit}
	 d* {var check detach}
	 i* {var check ignore}]

	if {![null $check]} {
    	    #
	    # Restrict the special command to the length of the first element
	    # of the input line (which should be the command).
	    #
	    var check [range $check 0
	    	    	     [expr [length [index $file 0] char]-1] char]
    	    #
	    # Make sure the first element matches the command in all its
	    # particulars.
	    #
	    if {[string c $check [index $file 0]] == 0} {
    	    	#
		# Is a special command -- perform the appropriate action.
		#   quit    	just eval the command -- we'll exit stage left
		#   detach  	eval the command and return to top level
		#   ignore  	just return empty -- caller should expect
		#   	    	that as a signal to return empty.
    	    	#
    	    	[case $check in
		 q* {eval [concat quit [range $file 1 end]]}
		 d* {
		    if {$maydetach} {
    	    	    	eval [concat detach [range $file 1 end]]
			return {}
		    } else {
		    	echo Detaching not allowed.
			continue
	    	    }
    	    	 }
		 i* {
		    if {$mayignore} {
		    	return {}
		    } else {
		    	echo Ignoring this patient is not allowed.
			continue
		    }
		 }]
    	    }
	}	 
	#
	# Check for the existence of the file -- the caller will have to
	# figure out about serial numbers and whatnot.
	#
    	var rfile [file-mangle-for-dos $file]
    	if {[string match $rfile /*] || [string match $rfile ${file-abspat}]} {
	    if {[file readable $rfile]} {
	    	return $rfile
	    }
    	}
	var rfile [file-mangle-for-dos ${file-init-dir}/${file}]
	if {[file readable ${rfile}]} {
	    #
	    # Given relative to initial directory
	    #
	    return ${rfile}
	}
	echo $file not readable. Try again.
    }
}]

##############################################################################
#				file-locate-geode
##############################################################################
#
# SYNOPSIS:	Perform a quick lookup to locate a geode given its name and
#   	    	geodeFileType. This one uses the global variable
#		file-geode-list.
# PASS:		geode	= 2-list of the form {geodeType name.ext}
# CALLED BY:	File_FindGeode
# RETURN:	Path of possible match if known.
# SIDE EFFECTS:	?
#
# STRATEGY
#
# REVISION HISTORY:
#	Name	Date		Description
#	----	----		-----------
#	ardeb	1/14/91		Initial Revision
#
##############################################################################
[defsubr file-locate-geode {geode}
{
    if {[catch {file-locate-geode-common [index $geode 0] [index $geode 1] 0} result]} {
    	echo file-locate-geode: error: $result
	return
    } else {
    	return $result
    }
}]

[defsubr file-locate-ecgeode {geode}
{
    var type [index $geode 0] name [index $geode 1]
    
    var dot [string first . $name]
    var pat [range $name 0 [expr $dot-1] chars].[range $name [expr $dot+2] end char]*
    if {[catch {file-locate-geode-common $type $pat 1} result]} {
    	echo file-locate-ecgeode: error: $result
	return
    } else {
    	return $result
    }
}]

[defsubr file-locate-geode-common {type pattern index}
{
    global file-geode-list-$type file-devel-dir file-default-dir file-os
    
    var result {}
    
    foreach i [var file-geode-list-$type] {
    	if {[string match [index $i 0] $pattern]} {
	    var files [range $i 1 end]
	    #
	    # If nothing for the index (currently ec or non-ec) in the list,
	    # use the previous index (i.e. use the non-ec filename if there's 
	    # no ec version)
	    #
	    for {} {[null [index $files $index]]} {} {
		var index [expr $index-1]
	    }
	    #
	    # Pluck out the proper file, based on the index we were given.
	    #
	    var file [index $files $index]

	    #
	    # If file has more than one element, the first one is for UNIX and
	    # the second is for DOS
	    #
	    if {[length $file] > 1} {
		[case ${file-os} in
		 unix {
		    var file [index $file 0]
		 }
		 dos {
		    var file [index $file 1]
		 }
		]
	    }
	    #
	    # If there's a development directory, tell caller to check the file
	    # there first.
	    #
    	    if {![null ${file-devel-dir}]} {
		if {![null [info command file-build-devel-path]]} {
		    #
		    # Use user-defined procedure to create development version
		    # path
		    #
		    var result [concat $result
				[file-build-devel-path $file]]
		} else {
		    var result [concat $result
				[file-mangle-for-dos ${file-devel-dir}/$file]]
		}
    	    }
	    var result [concat $result
		    	[file-mangle-for-dos ${file-default-dir}/$file]]
    	}
    }
    return $result
}]

#
# The lists for the different geodes, themselves.
#
# There exists one global list for each geode type (1-5, currently). Each
# element of the list is structured like:
#   {<permname> <non-ec-file> [<ec-file>]}
#
# each of <non-ec-file> and <ec-file> may be either a single path, or
# a list of two paths, where the first is as it appears on UNIX, and the second
# is as it appears on DOS (where Novell transmutes duplicate 8.3 names into
# a 7+digit.3 name). If no <ec-file> is given, <non-ec-file> will be used
# instead.
#

#############################################################################
#
#			     APPLICATIONS
#
defvar file-geode-list-1 {
    {debug.app
    	Appl/Tools/Debug/debug.geo
    	Appl/Tools/Debug/debugec.geo}
    {Command.app
	Appl/Test/Command/command.geo
	Appl/Test/Command/commande.geo}
    {addrbk.app
	../Palm/Installed/Appl/AddrBook/addrbk.geo
	../Palm/Installed/Appl/AddrBook/addrbkec.geo}
    {addrBook.app
	../Palm/Installed/Appl/AddrBook/addrbook.geo
	{../Palm/Installed/Appl/AddrBook/addrbookec.geo
	 ../Palm/Installed/Appl/AddrBook/addrboo0.geo}}
    {alarm.app
	../Palm/Installed/Appl/Alarm/alarm.geo
	../Palm/Installed/Appl/Alarm/alarmec.geo}
    {amateur.app
	Appl/Games/Amateur/amateur.geo
	Appl/Games/Amateur/amateurec.geo}
    {autologi.app
	Appl/Iclas/Autologin/autolog.geo
	{Appl/Iclas/Autologin/autologec.geo
	 Appl/Iclas/Autologin/autologe.geo}}
    {bigcalc.app
	Appl/BigCalc/bigcalc.geo
	Appl/BigCalc/bigcalcec.geo}
    {bjack.app
	Appl/Games/BJack/bjack.geo
	Appl/Games/BJack/bjackec.geo}
    {blank.lib
	Appl/Saver/Blank/blank.geo
	Appl/Saver/Blank/blankec.geo}
    {bmanager.app
	Appl/FileMgrs/BManager/bmanager.geo
	Appl/FileMgrs/BManager/bmanagerec.geo}
    {bobbin.lib
	Appl/Saver/Bobbin/bobbin.geo
	Appl/Saver/Bobbin/bobbinec.geo}
    {bookmark.app
	Appl/Iclas/Bookmark/bookmrk.geo
	{Appl/Iclas/Bookmark/bookmrkec.geo
	 Appl/Iclas/Bookmark/bookmrke.geo}}
    {bstartup.app
	Appl/Startup/BStartup/bstartup.geo
	Appl/Startup/BStartup/bstartupec.geo}
    {changepw.app
	Appl/Iclas/ChangePW/changepw.geo
	{Appl/Iclas/ChangePW/changepwec.geo
	 Appl/Iclas/ChangePW/changep0.geo}}
    {charmap.uti
	Appl/Util/CharMap/charmap.geo
	Appl/Util/CharMap/charmapec.geo}
    {circles.lib
	Appl/Saver/Circles/circles.geo
	Appl/Saver/Circles/circlesec.geo}
    {command.app
	Appl/Tools/Command/command.geo
	Appl/Tools/Command/commandec.geo}
    {concen.app
	Appl/Games/Concen/concen.geo
	Appl/Games/Concen/concenec.geo}
    {consum.app
	../Palm/Installed/Appl/Consum/consum.geo
	../Palm/Installed/Appl/Consum/consumec.geo}
    {csplist.app
	Appl/Iclas/CSPList/csplist.geo
	Appl/Iclas/CSPList/csplistec.geo}
    {datebook.app
	../Palm/Installed/Appl/DateBook/datebook.geo
	{../Palm/Installed/Appl/DateBook/datebookec.geo
	 ../Palm/Installed/Appl/DateBook/dateboo0.geo}}
    {debug.app
	Appl/Tools/Debug/debug.geo
	Appl/Tools/Debug/debugec.geo}
    {dict.app
	../Palm/Installed/Appl/Dict/dict.geo
	../Palm/Installed/Appl/Dict/dictec.geo}
    {dosexec.app
    	Appl/Test/DosExec/dosexec.geo
	Appl/Test/DosExec/dosexecec.geo}
    {draw.app
	Appl/GeoDraw/draw.geo
	Appl/GeoDraw/drawec.geo}
    {dribble.lib
	Appl/Saver/Dribble/dribble.geo
	Appl/Saver/Dribble/dribbleec.geo}
    {dust.lib
	Appl/Saver/Dust/dust.geo
	Appl/Saver/Dust/dustec.geo}
    {expo.app
	Appl/Email/Expo/expo.geo
	Appl/Email/Expo/expoec.geo}
    {fades.lib
	Appl/Saver/Fades/fades.geo
	Appl/Saver/Fades/fadesec.geo}
    {faxinstl.app
    	Appl/Fax/Install/install.geo
    	Appl/Fax/Install/installec.geo}
    {faxspool.app
    	Appl/Fax/Spool/faxspool.geo
    	Appl/Fax/Spool/faxspoolec.geo}
    {fcalc.app
	../Palm/Installed/Appl/FCalc/fcalc.geo
	../Palm/Installed/Appl/FCalc/fcalcec.geo}
    {filebox.app
	Appl/TestUI/FileBox/filebox.geo}
    {fileremo.app
	Appl/Iclas/FileRemove/flremo.geo
	Appl/Iclas/FileRemove/flremoec.geo}
    {finstall.app
	Appl/Tools/FInstall/finstall.geo
	Appl/Tools/FInstall/finstallec.geo}
    {gentestc.app
	Appl/CTest/GenTest/gentest.geo}
    {geocalc.app
	Appl/GeoCalc/geocalc.geo
	Appl/GeoCalc/geocalcec.geo}
    {geodex.app
	Appl/GeoDex/geodex.geo
	Appl/GeoDex/geodexec.geo}
    {geofile.app
	Appl/GeoFile/geofile.geo
	Appl/GeoFile/geofileec.geo}
    {geoplann.app
	Appl/Calendar/geoplan.geo
	Appl/Calendar/geoplanec.geo}
    {gsprint.app
    	Appl/Tools/GSPrint/gsprint.geo
	Appl/Tools/GSPrint/gsprintec.geo}
    {gwpoker.app
	Appl/Games/GWPoker/gwpoker.geo
	Appl/Games/GWPoker/gwpokerec.geo}
    {hearts.app
	Appl/Games/Hearts/hearts.geo
	Appl/Games/Hearts/heartsec.geo}
    {hibtest.app
	Appl/Test/HardIconBar/hardiconbarec.geo}
    {hmanager.app
	Appl/FileMgrs/HManager/hmanager.geo}
    {ihelp.app
	Appl/Iclas/Help/help.geo}
    {istartup.app
	Appl/Startup/IStartup/istartup.geo
	{Appl/Startup/IStartup/istartupec.geo
	 Appl/Startup/IStartup/istartu0.geo}}
    {k2shell.app
	Appl/Iclas/K2Shell/k2shell.geo
	Appl/Iclas/K2Shell/k2shellec.geo}
    {langx.app
	../Palm/Installed/Appl/LangX/langx.geo
	../Palm/Installed/Appl/LangX/langxec.geo}
    {letter.lib
	Appl/Saver/Letter/letter.geo
	Appl/Saver/Letter/letterec.geo}
    {linktest.app
	Appl/Test/Linktest/linktest.geo
	{Appl/Test/Linktest/linktest.geo
	 Appl/Test/Linktest/linktes0.geo}}
    {login.app
	Appl/Iclas/Login/login.geo
	Appl/Iclas/Login/loginec.geo}
    {lol.hack
	Appl/LOL/lol.geo}
    {makeDisk.util
	Appl/Tools/MakeDisk/makedisk.geo
	{Appl/Tools/MakeDisk/makediskec.geo
	 Appl/Tools/MakeDisk/makedis0.geo}}
    {manager.app
	Appl/FileMgrs/GeoManager/manager.geo
	Appl/FileMgrs/GeoManager/managerec.geo}
    {melt.lib
	Appl/Saver/Melt/melt.geo
	Appl/Saver/Melt/meltec.geo}
    {myopt.app
	Appl/Preferences/Myopt/myopt.geo
	Appl/Preferences/Myopt/myoptec.geo}
    {netmsg.app
	Appl/Net/Netmsg/netmsg.geo
	Appl/Net/Netmsg/netmsgec.geo}
    {note.app
	../Palm/Installed/Appl/Note/note.geo
	../Palm/Installed/Appl/Note/noteec.geo}
    {notepad.app
	Appl/NotePad/notepad.geo}
    {ntaker.app
	Appl/NTaker/ntaker.geo
	Appl/NTaker/ntakerec.geo}
    {pieces.lib
	Appl/Saver/Pieces/pieces.geo
	Appl/Saver/Pieces/piecesec.geo}
    {pitest.app
	Appl/Test/PenInputTest/peninputtest.geo
	Appl/Test/PenInputTest/peninputtestec.geo}
    {prefmgr.app
	Appl/Preferences/PrefMgr/prefmgr.geo
	Appl/Preferences/PrefMgr/prefmgrec.geo}
    {qix.lib
	Appl/Saver/Qix/qix.geo
	Appl/Saver/Qix/qixec.geo}
    {reports.app
	Appl/Iclas/Reports/report.geo
	Appl/Iclas/Reports/reportec.geo}
    {resedit.app
	Appl/Tools/Localize/localize.geo
	Appl/Tools/Localize/localizeec.geo}
    {sand.lib
	Appl/Saver/Sand/sand.geo
	Appl/Saver/Sand/sandec.geo}
    {scrapbk.app
	Appl/ScrapBk/scrapbk.geo
	Appl/ScrapBk/scrapbkec.geo}
    {setup.app
	Appl/Preferences/Setup/setup.geo
	Appl/Preferences/Setup/setupec.geo}
    {solitair.app
	Appl/Solitaire/soli.geo
	Appl/Solitaire/soliec.geo}
    {stars.lib
	Appl/Saver/Stars/stars.geo
	Appl/Saver/Stars/starsec.geo}
    {swarm.lib
	Appl/Saver/Swarm/swarm.geo
	Appl/Saver/Swarm/swarmec.geo}
    {tedit.app
	Appl/TEdit/tedit.geo
	Appl/TEdit/teditec.geo}
    {textui.app
	Appl/Test/TextUI/textuiec.geo}
    {tickerta.lib
	Appl/Saver/Tickertape/tickertape.geo
	{Appl/Saver/Tickertape/tickertapeec.geo
	 Appl/Saver/Tickertape/tickert0.geo}}
    {tiles.lib
	Appl/Saver/Tiles/tiles.geo
	Appl/Saver/Tiles/tilesec.geo}
    {useheap.app
	Appl/Test/UseHeap/useheap.geo}
    {usinfo.app
	../Palm/Installed/Appl/USinfo/usinfo.geo
	../Palm/Installed/Appl/USinfo/usinfoec.geo}
    {wclock.app
	../Palm/Installed/Appl/WorldClock/wclock.geo
	../Palm/Installed/Appl/WorldClock/wclockec.geo}
    {welcome.app
	Appl/Startup/Welcome/welcome.geo
	Appl/Startup/Welcome/welcomeec.geo}
    {winfo.app
	../Palm/Installed/Appl/Winfo/winfo.geo
	../Palm/Installed/Appl/Winfo/winfoec.geo}
    {worms.lib
	Appl/Saver/Worms/worms.geo
	Appl/Saver/Worms/wormsec.geo}
    {write.app
	Appl/GeoWrite/write.geo
	Appl/GeoWrite/writeec.geo}
    {wshell.app
	Appl/FileMgrs/WShell/wshell.geo
	Appl/FileMgrs/WShell/wshellec.geo}
    {wshellba.app
	Appl/FileMgrs/WShellBA/wshellb.geo
	Appl/FileMgrs/WShellBA/wshellbec.geo}
    {zmanager.app
	Appl/FileMgrs/ZManager/zmanager.geo
	Appl/FileMgrs/ZManager/zmanagerec.geo}
    {zprefmgr.app
	Appl/Preferences/ZPrefMgr/zprefmgr.geo
	{Appl/Preferences/ZPrefMgr/zprefmgrec.geo
	 Appl/Preferences/ZPrefMgr/zprefmg0.geo}}
    {zsetup.app
	Appl/Preferences/ZSetup/zsetup.geo
	Appl/Preferences/ZSetup/zsetupec.geo}
}

#############################################################################
#
#			      LIBRARIES
#
defvar file-geode-list-2 {
    {ansic.lib
	Library/AnsiC/ansic.geo
	Library/AnsiC/ansicec.geo}
    {bitmap.lib
	Library/Bitmap/bitmap.geo
	Library/Bitmap/bitmapec.geo}
    {blib.lib
	Library/Bullet/blib.geo
	Library/Bullet/blibec.geo}
    {book.lib
	../Palm/Installed/Library/Book/book.geo
	../Palm/Installed/Library/Book/bookec.geo}
    {borlandc.lib
	Library/Math/Compiler/BorlandC/borlandc.geo
	{Library/Math/Compiler/BorlandC/borlandcec.geo
	 Library/Math/Compiler/BorlandC/borland0.geo}}
    {bprefcn.lib
	Library/Pref/BulletCN/bprefcn.geo
	Library/Pref/BulletCN/bprefcnec.geo}
    {bprefdz.lib
	Library/Pref/BulletDZ/bprefdz.geo
	Library/Pref/BulletDZ/bprefdzec.geo}
    {bprefhi.lib
	Library/Pref/BulletHI/bprefhi.geo
	Library/Pref/BulletHI/bprefhiec.geo}
    {bprefhw.lib
	Library/Pref/BulletHW/bprefhw.geo
        Library/Pref/BulletHW/bprefhwec.geo}
    {bpreflm.lib
	Library/Pref/BulletLM/bpreflm.geo
	Library/Pref/BulletLM/bpreflmec.geo}
    {bprefmm.lib
	Library/Pref/BulletMM/bprefmm.geo
	Library/Pref/BulletMM/bprefmmec.geo}
    {bprefpm.lib
	Library/Pref/BulletPM/bprefpm.geo
        Library/Pref/BulletPM/bprefpmec.geo}
    {bprefpt.lib
	Library/Pref/BulletPT/bprefpt.geo
	Library/Pref/BulletPT/bprefptec.geo}
    {bprefpw.lib
	Library/Pref/BulletPW/bprefpw.geo
        Library/Pref/BulletPW/bprefpwec.geo}
    {bulldemo.lib
	Library/BullDemo/bulldemo.geo
	Library/BullDemo/bulldemoec.geo}
    {cardid.lib
	Library/CardID/cardid.geo
	Library/CardID/cardidec.geo}
    {cell.lib
	Library/Cell/cell.geo
	Library/Cell/cellec.geo}
    {chart.lib
	Library/Chart/chart.geo
	Library/Chart/chartec.geo}
    {color.lib
	Library/Color/color.geo
	Library/Color/colorec.geo}
    {config.lib
        Library/Config/config.geo
        Library/Config/configec.geo}
    {convert.lib
	Library/Convert/convert.geo
	Library/Convert/convertec.geo}
    {crunch.fmtl
	Library/FMTools/Crunch/crunch.geo
	Library/FMTools/Crunch/crunchec.geo}
    {csv.lib
	Library/Translation/Database/CSV/csv.geo
	Library/Translation/Database/CSV/csvec.geo}
    {cvttool.fmtl
	Library/FMTools/CvtTool/cvttool.geo
	Library/FMTools/CvtTool/cvttoolec.geo}
    {datepick.lib
	Library/DatePick/datepick.geo
	{Library/DatePick/datepickec.geo
	 Library/DatePick/datepic0.geo}}
    {dbase3.lib
	Library/Translation/Database/dbase3/dbase3.geo
	Library/Translation/Database/dbase3/dbase3ec.geo}
    {dwrite.lib
	Library/Translation/Text/DisplayWrite/dwrite.geo
	Library/Translation/Text/DisplayWrite/dwriteec.geo}
    {pabapi.lib
	Library/AddressBook/PalmAddrBookApi/pabapi.geo
	Library/AddressBook/PalmAddrBookApi/pabapiec.geo}
    {pdafs.lib
	Library/Email/PdaFileSys/pdafs.geo
	Library/Email/PdaFileSys/pdafsec.geo}
    {attels.lib
	Library/Email/ExpoAtt/attels.geo
	Library/Email/ExpoAtt/attelsec.geo}
    {eps.lib
	Library/Translation/Graphics/Vector/EPS/eps.geo
	Library/Translation/Graphics/Vector/EPS/epsec.geo}
    {faxfile.lib
    	Library/Fax/File/faxfile.geo
    	Library/Fax/File/faxfileec.geo}
    {ffile.lib
	Library/FlatFile/ffile.geo
	Library/FlatFile/ffileec.geo}
    {float.lib
	Library/Float/float.geo
	Library/Float/floatec.geo}
    {geopm.spui
	Library/GeoPM/geopm.geo
	Library/GeoPM/geopmec.geo}
    {geos.kern
	Library/Kernel/geos.geo
	Library/Kernel/geosec.geo}
    {grobj.lib
	Library/GrObj/grobj.geo
	Library/GrObj/grobjec.geo}
    {hprefcn.lib
	Library/Pref/HPrefCN/hprefcn.geo}
    {im.lib
	Library/IM/im.geo
	Library/IM/imec.geo}
    {impex.lib
	Library/Impex/impex.geo
	Library/Impex/impexec.geo}
    {int8087.lib
	Library/CoProcessor/Int8087/int8087.geo
	Library/CoProcessor/Int8087/int8087ec.geo}
    {intx87.lib
	Library/CoProcessor/Intx87/intx87.geo
	Library/CoProcessor/Intx87/intx87ec.geo}
    {lot123.lib
	Library/Translation/Text/Lotus123/lot123.geo
	Library/Translation/Text/Lotus123/lot123ec.geo}
    {math.lib
	Library/Math/math.geo
	Library/Math/mathec.geo}
    {mmate.lib
	Library/Translation/Text/MultiMate/mmate.geo
	Library/Translation/Text/MultiMate/mmateec.geo}
    {motif.spui
	Library/Motif/motif.geo
	Library/Motif/motifec.geo}
    {msmfile.lib
	Library/Translation/Text/MSMFile/msmfile.geo
	Library/Translation/Text/MSMFile/msmfileec.geo}
    {net.lib
	Library/Net/net.geo
	Library/Net/netec.geo}
    {palm.lib
	{../Palm/Installed/Library/PalmHWR/palm.geo
    	Library/PalmHWR/palm.geo}
	{../Palm/Installed/Library/PalmHWR/palmec.geo
    	Library/PalmHWR/palm.geo}}
    {palmhwr.lib
	{../Palm/Installed/Library/PalmHWR/palm.geo
    	Library/PalmHWR/palm.geo}
	{../Palm/Installed/Library/PalmHWR/palmec.geo
    	Library/PalmHWR/palm.geo}}
    {palmui.lib
	../Palm/Installed/Library/PalmUI/palmui.geo
	../Palm/Installed/Library/PalmUI/palmuiec.geo}
    {parse.lib
	Library/Parse/parse.geo
	Library/Parse/parseec.geo}
    {pcmcia.lib
	Library/PCMCIA/pcmcia.geo
	Library/PCMCIA/pcmciaec.geo}
    {pm.spui
	Library/PM/pm.geo
	Library/PM/pmec.geo}
    {pmba.spui
	Library/PMBA/pmba.geo
	Library/PMBA/pmbaec.geo}
    {prefbg.lib
	Library/Pref/Prefbg/prefbg.geo
	Library/Pref/Prefbg/prefbgec.geo}
    {prefcomp.lib
	Library/Pref/Prefcomp/prefcomp.geo
	{Library/Pref/Prefcomp/prefcompec.geo
	 Library/Pref/Prefcomp/prefcom0.geo}}
    {preffax.lib
    	Library/Pref/Preffax/preffax.geo
    	Library/Pref/Preffax/preffaxec.geo}
    {preffont.lib
	Library/Pref/Preffont/preffont.geo
	{Library/Pref/Preffont/preffontec.geo
	 Library/Pref/Preffont/preffon0.geo}}
    {prefintl.lib
	Library/Pref/Prefintl/prefintl.geo
	{Library/Pref/Prefintl/prefintlec.geo
	 Library/Pref/Prefintl/prefint0.geo}}
    {prefkbd.lib
	Library/Pref/Prefkbd/prefkbd.geo
	Library/Pref/Prefkbd/prefkbdec.geo}
    {preflf.lib
	Library/Pref/Preflf/preflf.geo
	Library/Pref/Preflf/preflfec.geo}
    {preflfi.lib
	Library/Pref/Preflfi/preflfi.geo
	Library/Pref/Preflfi/preflfiec.geo}
    {preflink.lib
	Library/Pref/Preflink/preflink.geo
	{Library/Pref/Preflink/preflinkec.geo
	 Library/Pref/Preflink/preflin0.geo}}
    {preflo.lib
	Library/Pref/Preflo/preflo.geo
	Library/Pref/Preflo/prefloec.geo}
    {prefmous.lib
	Library/Pref/Prefmous/prefmous.geo}
    {prefos.lib
	Library/Pref/Prefos/prefos.geo
	Library/Pref/Prefos/prefosec.geo}
    {prefperm.lib
	Library/Pref/Prefperm/prefperm.geo
	{Library/Pref/Prefperm/prefpermec.geo
	 Library/Pref/Prefperm/prefper0.geo}}
    {prefsnd.lib
	Library/Pref/Prefsnd/prefsnd.geo
	Library/Pref/Prefsnd/prefsndec.geo}
    {preftd.lib
	Library/Pref/Preftd/preftd.geo
	Library/Pref/Preftd/preftdec.geo}
    {prefts.lib
	Library/Pref/Prefts/prefts.geo
	Library/Pref/Prefts/preftsec.geo}
    {prefui.lib
	Library/Pref/Prefui/prefui.geo
	Library/Pref/Prefui/prefuiec.geo}
    {prefvid.lib
	Library/Pref/Prefvid/prefvid.geo}
    {ps.lib
	Library/Translation/Graphics/PS/ps.geo
	Library/Translation/Graphics/PS/psec.geo}
    {rtf.lib
	Library/Translation/Text/RTF/rtf.geo
	Library/Translation/Text/RTF/rtfec.geo}
    {ruler.lib
	Library/Ruler/ruler.geo
	Library/Ruler/rulerec.geo}
    {saver.lib
    	Library/Saver/saver.geo
    	Library/Saver/saverec.geo}
    {searchsp.lib
	Library/SearchSp/searchsp.geo
	Library/SearchSp/searchspec.geo}
    {shell.lib
	Library/Shell/shell.geo
	Library/Shell/shellec.geo}
    {sound.lib
	Library/Sound/sound.geo
	Library/Sound/soundec.geo}
    {spell.lib
	Library/Spell/spell.geo
	Library/Spell/spellec.geo}
    {spline.lib
	Library/Spline/spline.geo
	Library/Spline/splineec.geo}
    {spool.lib
	Library/Spool/spool.geo
	Library/Spool/spoolec.geo}
    {ssheet.lib
	Library/Spreadsheet/ssheet.geo
	Library/Spreadsheet/ssheetec.geo}
    {ssmeta.lib
	Library/SSMeta/ssmeta.geo
	Library/SSMeta/ssmetaec.geo}
    {styles.lib
	Library/Styles/styles.geo
	Library/Styles/stylesec.geo}
    {swap.lib
	Library/Swap/swap.geo
	Library/Swap/swapec.geo}
    {text.lib
	Library/Text/text.geo
	Library/Text/textec.geo}
    {ui.lib
	Library/User/ui.geo
	Library/User/uiec.geo}
    {winword.lib
	Library/Translation/Text/WinWord/winword.geo
	Library/Translation/Text/WinWord/winwordec.geo}
    {wperf4.lib
	Library/Translation/Text/WordPerfect4X/wperf4.geo
	Library/Translation/Text/WordPerfect4X/wperf4ec.geo}
    {wperf5.lib
	Library/Translation/Text/WordPerfect5X/wperf5.geo
	Library/Translation/Text/WordPerfect5X/wperf5ec.geo}
    {zoomconn.lib
	Library/Pref/ZoomConn/zoomconn.geo
	{Library/Pref/ZoomConn/zoomconec.geo
	 Library/Pref/ZoomConn/zoomcon0.geo}}
    {zoomdgtz.lib
	Library/Pref/ZoomDgtz/zoomdgtz.geo
	{Library/Pref/ZoomDgtz/zoomdgtzec.geo
	 Library/Pref/ZoomDgtz/zoomdgt0.geo}}
    {zoomer.lib
	Library/Zoomer/zoomer.geo
	Library/Zoomer/zoomerec.geo}
    {zoomhw.lib
	Library/Pref/ZoomHW/zoomhw.geo
	Library/Pref/ZoomHW/zoomhwec.geo}
    {zoomintl.lib
	Library/Pref/ZoomIntl/zoomintl.geo
	{Library/Pref/ZoomIntl/zoomintlec.geo
	 Library/Pref/ZoomIntl/zoomint0.geo}}
    {zoomkbd.lib
	Library/Pref/ZoomKbd/zoomkbd.geo
	Library/Pref/ZoomKbd/zoomkbdec.geo}
    {zoomprnt.lib
	Library/Pref/ZoomPrnt/zoomprnt.geo
	{Library/Pref/ZoomPrnt/zoomprntec.geo
	 Library/Pref/ZoomPrnt/zoomprn0.geo}}
    {zoomtd.lib
	Library/Pref/ZoomTD/zoomtd.geo
	Library/Pref/ZoomTD/zoomtdec.geo}
    {zoomuser.lib
	Library/Pref/ZoomUser/zoomuser.geo
	{Library/Pref/ZoomUser/zoomuserec.geo
	 Library/Pref/ZoomUser/zoomuse0.geo}
    {symfile.lib
    	Library/Symfile/symfile.geo
    	{Library/Symfile/symfileec.geo
    	Library/Symfile/symfilee.geo}
    }}
}

#############################################################################
#
#			    DEVICE DRIVERS
#
defvar file-geode-list-3 {
    {Comm.drv
	Driver/Net/Comm/comm.geo
	Driver/Net/Comm/commec.geo}
    {bpen.drvr
	Driver/Mouse/BulletPen/bpen.geo
	Driver/Mouse/BulletPen/bpenec.geo}
    {absgen.drvr
	Driver/Mouse/AbsGen/absgen.geo
	Driver/Mouse/AbsGen/absgenec.geo}
    {att6300.drvr
	Driver/Video/Dumb/ATT6300/att6300.geo
	Driver/Video/Dumb/ATT6300/att6300ec.geo}
    {bemm.drvr
	Driver/Swap/EMS/BullEMM/bemm.geo
	Driver/Swap/EMS/BullEMM/bemmec.geo}
    {bfs.ifsd
	Driver/IFS/GEOS/BullFS/bfs.geo
	Driver/IFS/GEOS/BullFS/bfsec.geo}
    {bitstrm.drvr
	Driver/Font/Bitstream/bitstrm.geo
	Driver/Font/Bitstream/bitstrmec.geo}
    {bpen.drvr
    	Driver/Mouse/BulletPen/bpen.geo
	Driver/Mouse/BulletPen/bpenec.geo}
    {bpower.drvr
	Driver/Power/Bullet/bpower.geo
	Driver/Power/Bullet/bpowerec.geo}
    {canon48.drvr
    	{Driver/Printer/DotMatrix/Canon48/canon48.geo
	 Driver/Printer/DotMatri/Canon48/canon48.geo}
	{Driver/Printer/DotMatrix/Canon48/canon48ec.geo
	 Driver/Printer/DotMatri/Canon48/canon48e.geo}}
    {casiopen.drvr
	Driver/Mouse/CasioPen/casiopen.geo
	{Driver/Mouse/CasioPen/casiopenec.geo
	 Driver/Mouse/CasioPen/casiope0.geo}}
    {casiopwr.drvr
	Driver/Power/Casio/casiopwr.geo
	{Driver/Power/Casio/casiopwrec.geo
	 Driver/Power/Casio/casiopw0.geo}}
    {casiosnd.drvr
	Driver/Sound/Casio/casio.geo
	Driver/Sound/Casio/casioec.geo}
    {casiovid.drvr
	Driver/Video/Dumb/Casio/casio.geo
	Driver/Video/Dumb/Casio/casioec.geo}
    {ccom.drvr
        Driver/Printer/Fax/CCom/ccom.geo
        Driver/Printer/Fax/CCom/ccomec.geo}
    {ccomrem.drvr
        Driver/Printer/Fax/CComRem/ccomrem.geo
        Driver/Printer/Fax/CComRem/ccomremec.geo}
    {cdrom.ifsd
	Driver/IFS/DOS/CDROM/cdrom.geo
	Driver/IFS/DOS/CDROM/cdromec.geo}
    {cga.drvr
	Driver/Video/Dumb/CGA/cga.geo
	Driver/Video/Dumb/CGA/cgaec.geo}
    {cidfs.drvr
	Driver/PCMCIA/CID/CIDFS/cidfs.geo
	Driver/PCMCIA/CID/CIDFS/cidfsec.geo}
    {cidser.drvr
	Driver/PCMCIA/CID/CIDSer/cidser.geo
	Driver/PCMCIA/CID/CIDSer/cidserec.geo}
    {class2.drvr
    	Driver/Fax/Output/Class2/class2.geo
    	Driver/Fax/Output/Class2/class2ec.geo}
    {disk.drvr
	Driver/Swap/Disk/disk.geo
	Driver/Swap/Disk/diskec.geo}
    {dri.ifsd
	Driver/IFS/DOS/DRI/dri.geo
	Driver/IFS/DOS/DRI/driec.geo}
    {dscga.drvr
	Driver/Video/Dumb/DSCGA/dscga.geo
	Driver/Video/Dumb/DSCGA/dscgaec.geo}
    {ega.drvr
	Driver/Video/VGAlike/EGA/ega.geo
	Driver/Video/VGAlike/EGA/egaec.geo}
    {emm.drvr
	Driver/Swap/EMS/EMM/emm.geo
	Driver/Swap/EMS/EMM/emmec.geo}
    {epson9.drvr
	Driver/Printer/DotMatrix/Epson9/epson9.geo
	Driver/Printer/DotMatrix/Epson9/epson9ec.geo}
    {extMem.drvr
	Driver/Swap/ExtMem/extmem.geo
	Driver/Swap/ExtMem/extmemec.geo}
    {fatfs.drvr
	Driver/PCMCIA/FATFS/fatfs.geo
	Driver/PCMCIA/FATFS/fatfsec.geo}
    {fep.drvr
    	Driver/Fep/PigLatin/fep.geo
    	Driver/Fep/PigLatin/fepec.geo}
    {filestr.drvr
    	Driver/Stream/Filestr/filestr.geo
	Driver/Stream/Filestr/filestrec.geo}
    {genmouse.drvr
	Driver/Mouse/GenMouse/genmouse.geo
	{Driver/Mouse/GenMouse/genmouseec.geo
	 Driver/Mouse/GenMouse/genmous0.geo}}
    {group3.drvr
    	Driver/Printer/Fax/Group3/group3.geo
    	Driver/Printer/Fax/Group3/group3ec.geo}
    {geosts.drvr
	Driver/Task/GeosTS/geosts.geo
	Driver/Task/GeosTS/geostsec.geo}
    {hgc.drvr
	Driver/Video/Dumb/HGC/hgc.geo
	Driver/Video/Dumb/HGC/hgcec.geo}
    {kbd.drvr
	Driver/Keyboard/kbd.geo
	Driver/Keyboard/kbdec.geo}
    {kbmouse.drvr
	Driver/Mouse/KBMouse/kbmouse.geo
	Driver/Mouse/KBMouse/kbmouseec.geo}
    {logiBus.drvr
	Driver/Mouse/LogiBus/logibus.geo
	Driver/Mouse/LogiBus/logibusec.geo}
    {logiSer.drvr
	Driver/Mouse/LogiSer/logiser.geo
	Driver/Mouse/LogiSer/logiserec.geo}
    {mSys.drvr
	Driver/Mouse/MSys/msys.geo
	Driver/Mouse/MSys/msysec.geo}
    {mcga.drvr
	Driver/Video/Dumb/MCGA/mcga.geo
	Driver/Video/Dumb/MCGA/mcgaec.geo}
    {mega.drvr
	Driver/Video/VGAlike/MEGA/mega.geo
	Driver/Video/VGAlike/MEGA/megaec.geo}
    {megafile.ifsd
	Driver/IFS/GEOS/MegaFile/megafile.geo
	{Driver/IFS/GEOS/MegaFile/megafileec.geo
	 Driver/IFS/GEOS/MegaFile/megafil0.geo}}
    {ms3.ifsd
	Driver/IFS/DOS/MS3/ms3.geo
	Driver/IFS/DOS/MS3/ms3ec.geo}
    {ms4.ifsd
	Driver/IFS/DOS/MS4/ms4.geo
	Driver/IFS/DOS/MS4/ms4ec.geo}
    {msBus.drvr
	Driver/Mouse/MSBus/msbus.geo
	Driver/Mouse/MSBus/msbusec.geo}
    {msSer.drvr
	Driver/Mouse/MSSer/msser.geo
	Driver/Mouse/MSSer/msserec.geo}
    {msnet.ifsd
	Driver/IFS/DOS/MSNet/msnet.geo
	Driver/IFS/DOS/MSNet/msnetec.geo}
    {nec24.drvr
	Driver/Printer/DotMatrix/Nec24/nec24.geo
	Driver/Printer/DotMatrix/Nec24/nec24ec.geo}
    {netware.ifsd
	Driver/IFS/DOS/NetWare/netware.geo
	Driver/IFS/DOS/NetWare/netwareec.geo}
    {nimbus.drvr
	Driver/Font/Nimbus/nimbus.geo
	Driver/Font/Nimbus/nimbusec.geo}
    {nonts.drvr
	Driver/Task/NonTS/nonts.geo
	Driver/Task/NonTS/nontsec.geo}
    {nopower.drvr
	Driver/Power/NoPower/nopower.geo
	Driver/Power/NoPower/nopowerec.geo}
    {nppcm.drvr
	Driver/Power/NoPowerPCMCIA/nppcm.geo
	Driver/Power/NoPowerPCMCIA/nppcmec.geo}
    {nw.drvr
	Driver/Net/NW/nw.geo
	Driver/Net/NW/nwec.geo}
    {os2.ifsd
	Driver/IFS/DOS/OS2/os2.geo
	Driver/IFS/DOS/OS2/os2ec.geo}
    {parallel.drvr
	Driver/Stream/Parallel/parallel.geo
	{Driver/Stream/Parallel/parallelec.geo
	 Driver/Stream/Parallel/paralle0.geo}}
    {pcl4.drvr
    	Driver/Printer/HP/Pcl4/pcl4.geo
	Driver/Printer/HP/Pcl4/pcl4ec.geo}
    {ppds24.drvr
	Driver/Printer/DotMatrix/Ppds24/ppds24.geo
	Driver/Printer/DotMatrix/Ppds24/ppds24ec.geo}
    {ps2.drvr
	Driver/Mouse/PS2/ps2.geo
	Driver/Mouse/PS2/ps2ec.geo}
    {pscript.drvr
	Driver/Printer/PScript/pscript.geo
	Driver/Printer/PScript/pscriptec.geo}
    {pserial.drvr
	Driver/PCMCIA/PSerial/Zoomer/pserial.geo
	Driver/PCMCIA/PSerial/Zoomer/pserialec.geo}
    {rfsd.ifsd
	Driver/IFS/RFSD/rfsd.geo
	Driver/IFS/RFSD/rfsdec.geo}
    {serial.drvr
	Driver/Stream/Serial/serial.geo
	Driver/Stream/Serial/serialec.geo}
    {standard.drvr
	Driver/Sound/Standard/standard.geo
	{Driver/Sound/Standard/standardec.geo
	 Driver/Sound/Standard/standar0.geo}}
    {stream.drvr
	Driver/Stream/stream.geo
	Driver/Stream/streamec.geo}
    {svga.drvr
	Driver/Video/VGAlike/SVGA/svga.geo
	Driver/Video/VGAlike/SVGA/svgaec.geo}
    {taskmax.drvr
	Driver/Task/TaskMax/taskmax.geo
	Driver/Task/TaskMax/taskmaxec.geo}
    {vga.drvr
	Driver/Video/VGAlike/VGA/vga.geo
	Driver/Video/VGAlike/VGA/vgaec.geo}
    {vga8.drvr
	Driver/Video/VGAlike/VGA8/vga8.geo
	Driver/Video/VGAlike/VGA8/vga8ec.geo}
    {vidmem.drvr
	Driver/Video/Dumb/VidMem/vidmem.geo
	Driver/Video/Dumb/VidMem/vidmemec.geo}
    {xms.drvr
	Driver/Swap/XMS/xms.geo
	Driver/Swap/XMS/xmsec.geo}
    {zoomfs.ifsd
	Driver/IFS/GEOS/ZoomFS/zoomfs.geo
	Driver/IFS/GEOS/ZoomFS/zoomfsec.geo}
}

#############################################################################
#
#			      ISV THINGS
#
defvar file-geode-list-5 {
    {pq.app
	../Intuit/Installed/Appl/PQ/pq.geo
	../Intuit/Installed/Appl/PQ/pqec.geo}
    {pqdb.lib
	../Intuit/Installed/Library/PQDB/pqdb.geo
	../Intuit/Installed/Library/PQDB/pqdbec.geo}
}

if {[string c ${file-os} unix] == 0} {
    [if {[file exists ~/.swat.files]} {
	if {[catch {source [file expand ~/.swat.files]} res] != 0} {
	    echo Warning: ~/.swat.files: $res
	}
    }]
}    
