COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	PC GEOS
MODULE:		com -- handle the serial line
FILE:		com.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name		Description
	----		-----------
	ComInit		Initializes the module
	ComRead		Returns a character from the input ring
	ComWrite	Transfers a character to the serial line
	ComReadBlock	Read a block of data to the given address.
	ComWriteBlock	Writes a block of data from the given address
	ComExit		Restores the serial line state.

	
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision
	jdm	91.02.20	Added /b:38400 support.
	jimmy	9/92		added PTTY environment variable support
	jimmy 	9/92		adding support for 15 IRQ levels

DESCRIPTION:
	This module handles all I/O for a single serial line. All the I/O
	is interrupt-driven with XON/XOFF flow control active on the input
	side and obeyed on the output side unless the line has been placed in
	raw mode. When in raw mode, we assume the function responsible will
	guarantee that the buffer won't overflow.

	The buffers are, naturally, ring buffers with no checking for
	overflow (we assume flow control will handle this...).

	The single-byte functions will not block, but the block functions will.
		
	The line may be either raw or well-done. In well-done mode, only
	7 bits are received, the high bit being stripped off before the
	character is checked for being one of XON or XOFF. In raw mode, all
	eight bits are passed.

	$Id: com.asm,v 2.21 94/01/05 17:27:07 cassie Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
COMMODULE	=	1	; Dont extrn our definitions
		include	com43.def

program		segment

;
; PSP definitions
;
;
; Buffering paramters:
;	BUFSIZE		Size of both transmit and receive buffers
;	HIGHWATER	When the number of characters in the receive buffer
;			reaches this level, an XOFF is sent to make the
;			host stop.
;	LOWWATER	Once the host has been stopped, the number of
;			characters in the buffer must fall to this level before
;			an XON is sent, allowing the host to continue.
;
BUFSIZE		equ	BUFFER_SIZE	; Number of characters to buffer
HIGHWATER	equ	BUFSIZE - 64
LOWWATER	equ	BUFSIZE / 4

XOFF		equ	19	; Control-S = Stop sending
XON		equ	17	; Control-Q = Start sending

;
; A BUFFER is used to queue incoming and outgoing data. It contains BUFSIZE
; bytes of data arranged in a ring buffer. Two indices into the buffer are
; maintained. Data are added at the tail index and removed at the head.
; The number of characters in the buffer is maintained in the numChars field
; to make the checks as efficient as possible.
;
BUFFER		STRUC
head		dw	0		; Head index
tail		dw	0		; Tail index
numChars	dw	0		; Number of characters in buffer
data		db	BUFSIZE DUP(?)	; The characters themselves
BUFFER		ENDS

;
; Update is used to update a buffer index, modulo BUFSIZE, and store it in
; its proper location. "reg" is the register in which the index is currently
; located and "dest" is where it should eventually go.
;
Update		macro	dest, reg
		local	stuffIt
		inc	reg		;; Pointers move forward
		cmp	reg, BUFSIZE	;; At end?
		jne	stuffIt		;; Nope
		clr	reg		;; Wrap to 0
stuffIt:	mov	dest, reg	;; Store it
		endm

comIn		BUFFER	<>		; Receiver buffer
comOut		BUFFER	<>		; Output buffer
		bugsym	comIn
		bugsym	comOut

;
; Various state flags and macros for manipulating them
;
;
; Flag definitions:
;	XOFFPEND	Set => XOFF couldn't be sent at interrupt time and
;			must be sent as soon as possible.
;	XONPEND		Set => XON should be sent when the input buffer has
;			drained sufficiently.
;	IRQPEND		Set => a transmitter interrupt should be coming in.
;			Tells us whether to queue a character or send it
;			directly.
;	STOP		Set => output has been stopped by the receipt of an
;			XOFF character. Output is queued until an XON char
;			is received.
;	RAW		Set => unit is in raw mode -- XON/XOFF characters are
;			ignored on input.
;
set		macro	flag
		;;
		;; Set the flag TRUE. isset? will set ZF = 0 after this
		;;
		or	comFlags, MASK flag
		endm
reset		macro	flag
		;;
		;; Reset the flag to FALSE. isset? will set ZF = 1.
		;;
		and	comFlags, NOT MASK flag
		endm
isset?		macro	flag
		;;
		;; Set the flags so jnz will take if the flag is true
		;;
		test	comFlags, MASK flag
		endm
flags		record	XOFFPEND:1,XONPEND:1,IRQPEND:1,STOP:1,RAW:1
comFlags	flags	<0,0,0,0,0>
		bugsym	comFlags
		even
;
; Place for storing old interrupt vector
;
comIntVec	dd	?
comChainVec	dd	?		; Place to which we should chain if
					; the interrupt isn't for us...
comIntVecNum	dw	74h		; Actual vector number (for reset)
;comIntMask	db	?		; Mask in 8259 (for reset)

;
; The four ports needed by the interrupt routine. These are set up by ComInit
; based on the device being used.
;
comCurDataPort	dw	COMDATAPORT
		bugsym	comCurDataPort
comCurStatPort	dw	COMSTATPORT
comCurIRQPort	dw	COMIRQPORT
comCurIENPort	dw	COMIENPORT
		bugsym	comCurStatPort
		bugsym	comCurIRQPort
		bugsym	comCurIENPort

;
; Message printed if a bogus /c switch is given.
;
comBadSwitchMsg	db	"Bad value for /c switch. Using COM2", cr, lf, 0

; default parameters, as determined from environment and other things.
comPort		byte
comBaud		word
comIRQ		byte

ComPortData	struc
    CPD_portMask	dw	?	; Mask for figuring out other
    					; ports for the channel. The number
					; being masked is 3fXh where X is
					; 8 through f
    CPD_level		dw	?	; Interrupt level (as far as
					; interrupt controller is concerned)
ComPortData	ends
ports		ComPortData	<0ffffh,4>	; COM1 (3f8, level 4)
		ComPortData	<0feffh,3>	; COM2 (2f8, level 3)
		ComPortData	<0ffefh,4>	; COM3 (3e8, level 4)
		ComPortData	<0feefh,3>	; COM4 (2e8, level 3)




COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComPrintByte
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Print out the given byte in hex

CALLED BY:	ComInterrupt

PASS:		AX	= byte to print

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Using the 'nibbles' table, translate each nibble to an ascii
	character and print it.

	Note the jmp at the start of the routine, since this is really just
	for debugging purposes.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/20/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ComPrintByte	proc 	near
	ifndef COM_DEBUG
		jmp	short CPB2
	endif
	ifdef COM_DEBUG
		push	ax
		mov	al, '@'
		call	ScrWrite
		pop	ax
		call	ScrPrintByte
		push	ax
		mov	al, '@'
		call	ScrWrite
		pop	ax
	endif
CPB2:
		ret
ComPrintByte	endp
		bugsym	ComPrintByte


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		GetPSP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get the segment adress of the PSP and put it in ds

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		ds = segment address of PSP

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/17/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

GetPSP	proc	near
	.enter
	mov	ah, DOS_GET_PSP_SEG
	int	21h
	mov	ds, bx 
	.leave
	ret
GetPSP	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		GetENVComData
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get the baud rate from the environment variable of any

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		carry set if PTTY variable not found
		else es:di = value string

		it is possible to just supply a com port,
		or just a comp port and baud rate, so zero values
		will be found in the unspecified values 


DESTROYED:	ax

PSEUDOCODE/STRATEGY: go through the environment block of the program
		     looking for a BAUDRATE variable, if we find one
		     the get the value contained else return carry

		     an environment block is a series of null ('\0') ternimated
		     strings with one final null after the last null ternimated
		     string, so the block ends with two nulls

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/17/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

startComData	label	near
comData		db	"PTTY="
endComData	label	near

ComGetENVData	proc	near
	uses	ds
	.enter
	call	GetPSP		; ds <- gets the PSP segment
	mov	es, ds:[PSP_ENV_SEG]
	push	cs
	pop	ds
	clr	ax
	mov	di, ax
doString:
	;
	; See if the variable matches what we're looking for.
	; 
	mov	si, offset comData
	mov	cx, length comData
	repe	cmpsb
	je	done			; => matches up to the =, so it's
					;  ours...
	dec	di			; deal with mismatch on null byte...
	mov	cx, -1
	repne	scasb
	
	cmp	es:[di], al		; double null (i.e. end of environment)
	jne	doString		; => no

	stc
done:
	.leave
	ret	
ComGetENVData	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComParseEnvString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set default communication parameters from the given string

CALLED BY:	Com_Init
PASS:		es:di	= string to parse
RETURN:		carry set if string invalid
		carry clear if happy:
			al	= IRQ
			dx	= baud rate divisor
			cl	= com port #
DESTROYED:	ax, cx, dx, si
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 7/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
ComParseEnvString proc	near
		uses	ds
		.enter
		push	es
		pop	ds
		mov	si, di

		call	ComDeterminePort		; al <- com port
		jc	done
		call	ComDetermineBaud		; dx <- divisor
		jc	done
		mov	cl, al
		call	ComDetermineIRQ		; al <- IRQ level
done:
		.leave
		ret
ComParseEnvString endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComDeterminePort
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Extract the port number from port description string.

CALLED BY:	Com_Init
PASS:		ds:si	= port description string
RETURN:		carry set if port number invalid
		carry clear if port number ok:
				al = com port
DESTROYED:	


PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	7/20/92		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComDeterminePort
ComDeterminePort proc near
	;
	; Fetch the first char of the descriptor. This must be a char between
	; 1 and 4, inclusive.
	; 
		xor	ax, ax
		lodsb			; al <- port number
		cmp	al, '1'
		jl	SDP_fail
		cmp	al, '4'
		jg	SDP_fail
	;
	;
	; Skip over the comma that must follow the port number.
	; 
		cmp	byte ptr ds:[si], ','
		jne	SDP_fail
		inc	si
		sub	al, '0'
SDP_done:
		ret
SDP_fail:
		stc
		jmp	SDP_done
ComDeterminePort endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComDetermineBaud
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Convert the number at ds:si into a number and see if that's
		a valid baud rate, returning the divisor for the rate if so.

CALLED BY:	Com_Init
PASS:		ds:si	= baud rate string (ending with null or comma)
RETURN:		carry set if invalid baud rate
		carry clear if baud rate is ok:
			ds:si	= null terminator, or first char of interrupt
				  level
			dx	= divisor (low 16 bits are relevant)
DESTROYED:	nothing

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	7/20/92		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComDetermineBaud
ComDetermineBaud proc near
		push	ax
		push	bx
		push	cx
		mov	bx, 10		; used as a multiplier in loop
		xor	dx, dx
		mov	ax, dx		; clear high 3 bytes of eax
						;  for the loop...
SDB_charLoop:
		lodsb
		or	al, al
		jz	SDB_hitNullTerm
		cmp	al, ','
		je	SDB_haveBaud
		sub	al, '0'
		jb	SDB_error
		cmp	al, 9
		ja	SDB_error
;		imul	dx, 10			; dx = dx * 10
		mov_tr	cx, ax			; save ax
		mov_tr  ax, dx			; ax = dx for mul
		mul	bx			; ax = ax * 10
		mov_tr	dx, ax			; dx = ax (old dx * 10)
		mov_tr	ax, cx			; restore ax
		add	dx, ax
		jmp	SDB_charLoop
SDB_hitNullTerm:
		dec	si		; point back to null
SDB_haveBaud:
	;
	; Blech. Isn't there some cleaner way to do this?
	; 
		xchg	ax, dx	; ax <- baud

	irp	baud, <300,1200,2400,4800,9600,19200,38400>
		mov	dx, BAUD_&baud
		cmp	ax, baud
		je	SDB_done	; (carry clear if branch taken)
	endm
SDB_error:
		stc
SDB_done:
		pop	cx
		pop	bx
		pop	ax
		ret
ComDetermineBaud		endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComDetermineIRQ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Determine the interrupt level to be used for the port.

CALLED BY:	Com_Init
PASS:		ds:si	= remainder of port descriptor
		cs:di	= ComPortData for selected port (contains default
			  interrupt level)
RETURN:		carry set on error
		carry clear if ok:
			al	= interrupt level for port (-1 for none)
DESTROYED:	si, cx

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	7/20/92		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComDetermineIRQ
ComDetermineIRQ proc	near
		push	dx
		push	bx
		push	cx
	;
	; Fetch the first character of the potential interrupt level.
	; 
		lodsb
		or	al, al
		jz	SDIRQ_useDefault	; => no interrupt level given,
						;  so return default for the
						;  port.
	;
	; Convert the first digit to binary.
	; 
		sub	al, '0'
		jb	SDIRQ_error	; => not numeric
		cmp	al, 9
		ja	SDIRQ_error	; => not numeric
		cwd			; clear high 3 bytes by sign-extension
	;
	; See if level is 2 digits.
	; 
		cmp	byte ptr ds:[si], 0
		je	SDIRQ_done	; => single digit; eax is level
	;
	; Multiply previous digit by 10 and add new digit to get level.
	; 
;		imul	cx, ax, 10
		mov	bx, 10
		mul	bx
		mov_tr	cx, ax
		lodsb
		sub	al, '0'
		jb	SDIRQ_error
		cmp	al, 9
		ja	SDIRQ_error
		cwd	
		add	ax, cx	; (can't carry)
SDIRQ_done:
		mov	ds:[comIRQ], al
		pop	cx
		pop	bx
		pop	dx
		ret
SDIRQ_error:
		stc
		jmp	SDIRQ_done

SDIRQ_useDefault:
		mov	al, -1
		clc
		jmp	SDIRQ_done
ComDetermineIRQ endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComFetchBaudArg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Fetch a /b argument, if there is one, and transform it
		into a baud rate divisor.

CALLED BY:	(INTERNAL) Com_Init
PASS:		es	= stubInit
		ds	= cgroup
RETURN:		cs:[comBaud] altered if /b given
DESTROYED:	ax, bx, cx, dx, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 7/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
baud		byte	'b', 0	; string for /b arg
baudCap		byte	'B', 0	; string for /b arg
baudrate	db	2 dup (?);storage for /b arg

ComFetchBaudArg	proc	near
		.enter
	;
	; Fetch the argument itself.
	; 
		mov	di, offset baud		; es:di <- arg
		mov	bx, offset baudrate	; es:bx <- buffer
		mov	dx, length baudrate	; dx <- sizeof(buffer)
		call	FetchArg
		cmp	bx, offset baudrate
		jne	gotbaud	; => no /b arg, so use default

		mov	di, offset baudCap	; es:di <- arg
		mov	bx, offset baudrate	; es:bx <- buffer
		mov	dx, length baudrate	; dx <- sizeof(buffer)
		call	FetchArg
		cmp	bx, offset baudrate
		je	done	
gotbaud:
		mov	ah, es:[baudrate]; get value provided
		mov	al, BAUD_9600	
		cmp	ah, '9'		; check for 9600 as value
		je	setNewBaud

		mov	al, BAUD_38400	; check for 38400 baud
		cmp	ah, '3'
		je	setNewBaud

		mov	al, BAUD_4800	; check for 4800 baud
		cmp	ah, '4'		; 
		je	setNewBaud	; use 4800 baud

		mov	al, BAUD_2400	; check for 2400 baud
		cmp	ah, '2'		; 
		je	setNewBaud	; use 2400 baud

		mov	al, BAUD_1200	; check for 1200 baud
		cmp	ah, '1'		; 
		jne	use19_2		; no, use 19200 baud
		cmp	es:[baudrate][1], '2'	; check for 1200, vs. 19200
		je	setNewBaud	;  yes, use 1200
use19_2:
		mov	al, BAUD_19200	; if no match, use 19200
setNewBaud:
		clr	ah
		mov	cs:[comBaud], ax; save divisor for later
done:
		.leave
		ret
ComFetchBaudArg	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComFetchPortArg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Fetch a /c argument, if there is one, and override the
		default port with its value.

CALLED BY:	(INTERNAL) Com_Init
PASS:		es	= stubInit
		ds	= cgroup
RETURN:		cs:[comPort] altered if /c given
DESTROYED:	ax, bx, cx, dx, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 7/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
devarg		char	'c', 0	; string for /c arg
devargCap	char	'C', 0	; string for /c arg
device		char		; place to store result of /c

ComFetchPortArg	proc	near
		.enter
		mov	di, offset devarg
		mov	bx, offset device
		mov	dx, length device
		call	FetchArg
		cmp	bx, offset device
		jne	gotPort

		mov	di, offset devargCap
		mov	bx, offset device
		mov	dx, length device
		call	FetchArg
		cmp	bx, offset device
		je	done
gotPort:

		mov	al, es:[device]
		cmp	al, '1'
		jl	done
		cmp	al, '4'
		jg	done
		sub	al, '0'

		mov	cs:[comPort], al
done:
		.leave
		ret
ComFetchPortArg	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComFetchIRQArg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Fetch a /I argument, if there is one, and override the
		default IRQ with its value.

CALLED BY:	(INTERNAL) Com_Init
PASS:		es	= stubInit
		ds	= cgroup
RETURN:		cs:[comIRQ] altered if /I given
DESTROYED:	ax, bx, cx, dx, di
SIDE EFFECTS:	

PSEUDO CODE/STRATEGY:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 7/92	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
irqarg		char	'I', 0	; string for /I arg
irqbuf		char	3 dup(?); room for 2-digit decimal + null-term

ComFetchIRQArg	proc	near
		uses	ds, si
		.enter
		mov	di, offset irqarg
		mov	bx, offset irqbuf
		mov	dx, length irqbuf-1
		call	FetchArg

		cmp	bx, offset irqbuf
		je	done
		
		mov	si, offset irqbuf
		call	ComDetermineIRQ
		mov	cs:[comIRQ], al
done:		
		.leave
		ret
ComFetchIRQArg	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComInitPC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	init a PC-like com port

CALLED BY:	ComInit

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/11/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
ComInitPC	proc	near
		.enter

	;
	; Now process the values we won so painfully.
	; 
		mov	bl, cs:[comPort]
		clr	bh
		dec	bx		; change to 0-origin from 1-origin
		shl	bx, 1		; *4 for indexing into ports
		shl	bx, 1

		;
		; Fetch the port mask and interrupt level now so we have no
		; further need for the port data.
		;
		mov	al, cs:[comIRQ]
		cmp	al, -1
		jne	haveIRQ
		mov	ax, es:ports[bx].CPD_level
haveIRQ:
		mov	bx, es:ports[bx].CPD_portMask

		; Adjust level by interrupt base for the controller to
		; get the actual vector number to use. Invert the mask since
		; we want to enable the interrupt.
		;
		clr	dx
		mov	dl, al		; dx = IRQ level
		mov	cl, al		; save IRQ level for later
		add	dx, IRQBASE	; adjusted irq level for SetInterrupt
		cmp	dx, IRQBASE+8
		jb	setIOPorts
		add	dx, IRQBASE2-(IRQBASE+8)
setIOPorts:
		;
		; Set up comCurDataPort, comCurStatPort and comCurIRQPort
		; for ComInterrupt
		;
		mov	ax, COMDATAPORT
		and	ax, bx
		mov	ds:[comCurDataPort], ax

		mov	ax, COMSTATPORT
		and	ax, bx
		mov	ds:[comCurStatPort], ax

		mov	ax, COMIRQPORT
		and	ax, bx
		mov	ds:[comCurIRQPort], ax

		;
		; Fetch original vector and stuff in ours in its place.
		;
		push	bx			; Still need port mask
		mov_tr	ax, dx			; Pass vector number in AX
		mov	ds:[comIntVecNum], ax	; Store vector number for
		mov	bx, offset comIntVec	; Address of storage in BX
		mov	dx, offset ComInterrupt	; Our routine in DX
		call	SetInterrupt		; Do it. (uses old function
						; if stub relocated)
		pop	bx			; Get mask back

		;
		; Set the baud rate of the line. The baud rate is set by
		; dividing an 18.432 mhz clock by something. The divisors
		; for various baud rates are:
		;	38400		 3
		;	19200		 6
		;	 9600		12
		;	 7200		16
		;	 4800		24
		;	 3600		32
		;	 2400		48
		;	 2000		58
		;	 1800		64
		;	 1200		96
		;
		; Note that the spec claims the thing shouldn't run at more
		; than 9600, but it appears to run just fine at 19.2
		;
		mov	dx, COMLINEPORT
		and	dx, bx
		mov	al, 80h
		out	dx, al

		;
		; Actually install the baud rate divisor
		; 
		mov	ax, cs:[comBaud]
		mov	dx, COMDLLPORT
		and	dx, bx
		out	dx, al

		jmp	$+2		; I/O delay...

		inc	dx
		mov	al, ah
		out	dx, al

		;
		; Make sure the interrupt enable port and data port are
		; actually at the appropriate addresses by clearing the
		; Divisor Latch Access Bit in the line control register.
		; Note the and'ing of COM1's port with the mask in bx
		; Also make sure the line is in the format we want:
		;	8 data bits
		;	1 stop bit
		;	no parity
		; qv. Options and Adapters vol. 2 for more info
		;
		mov	dx, COMLINEPORT
		and	dx, bx
		mov	al, 3
		out	dx, al

		;
		; Turn on DTR, RTS and OUT2. The first 2 are on because that's
		; what terminals normally expect. OUT2 is on b/c it must be on
		; for interrupts to come through.
		;
		mov	dx, COMMODEMPORT
		and	dx, bx
		mov	al, COMDTR OR COMRTS OR COMOUT2
		out	dx, al

		;
		;
		; Make sure the input buffer is clear and the interrupt-
		; pending flag is clear by reading the buffer now. If we
		; don't and for some reason an interrupt is already pending
		; from the device, we're hosed.
		;
		mov	dx, ds:[comCurDataPort]
		in	al, dx

		;
		; Enable the receiver interrupts. We've nothing to transmit
		; yet, so no point in enabling transmitter interrupts.
		;
		mov	dx, COMIENPORT
		and	dx, bx
		mov	ds:[comCurIENPort], dx
		mov	al, COMRECVENABLE
		out	dx, al

		;
		; Enable interrupts from the device in the interrupt controller
		;

		; see if we are dealing with irq 0-7 or 8-15
		; 0-7 are on PIC1, and 8-15 are on PIC2
		mov	dx, ICMASKPORT		; assume 0-7
		cmp	cl, 8
		jl	gotPIC

		mov	dx, ICMASKPORT2
		sub	cl, 8
gotPIC:
		mov	al, 1
		shl	al, cl			; set up mask
		not	al
		mov	ah, al
		in	al, dx		; al <- current mask
		and	al, ah		; clear the proper bit
		out	dx, al
		.leave
		ret
ComInitPC	endp


BadComInitString db   "Error: check PTTY environment variable",'\n','\r',0
COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComInit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize the appropriate communications port

CALLED BY:	Main

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX, BX, DX

PSEUDO CODE/STRATEGY:
	Figures out which device to use by looking for the /c flag in the
	command tail stored in the PSP that MS-DOS gives us. Usage is
		com43 /c[12]
	Defaults to COM1 if nothing specified.

	Once the device is determined, we initialize it to provide interrupts
	and install our own interrupt handler via SetInterrupt. The old
	vector is saved in comIntVec.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision
	jdm	91.02.20	Added 38400 baud support.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComInit

defaultComString char	"2,19200", 0

ComInit		proc	near
		push	es

		call	ComGetENVData
		jnc	haveDefaultString
useDefault:
		mov	di, offset defaultComString
		push	ds
		pop	es
haveDefaultString:
		call	ComParseEnvString
		jc	useDefault

		mov	cs:[comBaud], dx
		mov	cs:[comIRQ], al
		mov	cs:[comPort], cl

		pop	es
		
		;
		; First see if any /b flag given.  If not, use 19.2Kbaud
		;
		
		call	ComFetchBaudArg
		call	ComFetchPortArg

		call	ComFetchIRQArg
		call	ComInitPC
done:
		ret
error:
		mov	si, offset BadComInitString		
		call	PCTPrintString
		stc
		jmp	done
ComInit		endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComExit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Restore the state of the serial port world

CALLED BY:	Exit

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Restore the original interrupt vector and turn off interrupts for
	the device.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	What if XON still pending?

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComExit
ComExit		proc	near
		push	ax
		push	bx
		push	dx
		dsi
		mov	dx, ICMASKPORT
		mov	al, ds:[comIRQ]
		cmp	al, 8
		jl	gotPIC
		sub	al, 8
		mov	dx, 0a1h
gotPIC:
		mov	cl, al
		in	al, dx
		mov	ah, 1
		shl	ah, cl
		or	al, ah
		out	dx, al

		mov	dx, comCurIENPort
		clr	al
		out	dx, al

		mov	ax, comIntVecNum
		lea	bx, comIntVec
		call	ResetInterrupt
		eni

		pop	dx
		pop	bx
		pop	ax
		ret
ComExit		endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComInterrupt
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle an interrupt from the selected serial device

CALLED BY:	INTERNAL (and SPONTANEOUS)

PASS:		CS set to proper segment

RETURN:		Nothing

DESTROYED:	Nothing (I Hope)

PSEUDO CODE/STRATEGY:
	While interrupt pending
		read status
		if data available, read and queue. send xoff if too much.
		if can send and have data to send, send it.
	loop

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	It should probably check for buffer overrun and do something about
	it...

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision
	adam	6/20/88		Made use of OUT2 to make sure the IRQ line
				goes all the way down before coming back up
				again if more than one interrupt is pending.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		bugsym	ComInterrupt
ComInterrupt	proc	far
		push	ax
		push	bx
		push	dx
		push	ds

		mov	ax, cs
		mov	ds, ax			; set up ds for our own ease

	ifdef COM_DEBUG
		mov	al, '*'
		call	ScrWrite
	endif
		;
		; Make sure the interrupt's actually ours -- a bus mouse
		; shares the same interrupt level as COM2...
		;
		mov	dx, comCurIRQPort
		in	al, dx
		test	al, 1
		jz	CIIsOurs
		;
		; Nope -- restore registers and pass control to old
		; handler.
		;
		pop	ds
		pop	dx
		pop	bx
		pop	ax
		jmp	cs:comChainVec
CIIsOurs:
		; Force the interrupt output of the chip low by turning
		; off all interrupts. This makes sure the IRQ line makes it

		; all the way down before coming back up if an interrupt comes
		; in while we're in here
		;
		mov	dx, comCurIENPort
		clr	al
		out	dx, al

		;
		; We've turned off interrupts from this device, so it's ok to
		; turn interrupts on now.
		;
		eni

		;
		; Fetch current status bits
		;
		mov	dx, comCurStatPort
		in	al, dx
		push	ax			; Save for ComCheckTrans

		test	al, COMDAVAIL
		jz	ComCheckTrans

		;
		; Data waiting. Fetch character and stow it. Send
		; XOFF if hit high water mark
		;
		mov	dx, comCurDataPort
		in	al, dx			; Fetch the character
		call	ComPrintByte

		isset?	RAW
		jnz	ComQueueIn
		;
		; If not in raw mode, strip off any parity bit and see
		; if the character is XOFF or XON and set the STOP flag
		; appropriately if so.
		;
		and	al, 7fh
		cmp	al, XON
		je	ComRestart
		cmp	al, XOFF
		je	ComStop
ComQueueIn:
		mov	bx, comIn.tail
		mov	comIn.data[bx],al	; Stuff it

		Update	comIn.tail, bx

		;
		; See if the buffer is too full and go send XOFF if so.
		;
		inc	comIn.numChars
		cmp	comIn.numChars, HIGHWATER
		je	ComSendXOFF

ComCheckTrans:
		;
		; See if we can send data. Original status stored on the
		; stack, so use that.
		;
		pop	ax
		test	al, COMTHRE
		jnz	ComIntSend

ComIntReturn:
		;
		; Signal end-of-interrupt to interrupt controller, restore
		; state and return.
		;
		mov	al, ICEOI		; All done
		out	ICEOIPORT, al

		; if the IRQ >= 8 then we must send out the EOI to the
		; other PIC as well
		cmp	ds:[comIRQ], 8
		jb	sentEOI
		out	ICEOIPORT2, al
sentEOI:
		;
		; Reenable interrupts from the chip. If a character came in
		; or the transmitter became ready while we were in here, this
		; will generate an interrupt right away...
		; Note that we only enable the transmitter interrupt if we
		; actually expect an interrupt to come in. Otherwise, we will
		; loop infinitely getting transmitter interrupts each time we
		; re-enable interrupts for the device.
		;
		mov	dx, comCurIENPort
		mov	al, COMRECVENABLE
		isset?	IRQPEND
		jz	CI2
		or	al, COMTRANSENABLE
CI2: 		out	dx, al

		pop	ds
		pop	dx
		pop	bx
		pop	ax
		iret

ComRestart:
		;
		; Received XON so can start sending again.
		;
		reset	STOP
		jmp	ComCheckTrans
ComStop:
		;
		; Received XOFF so may not send.
		;
		set	STOP
		jmp	ComCheckTrans

ComIntSend:
		;
		; Transmitter ready for more. If output is disabled,
		; don't send anything (pretty funky, wot?). If no characters
		; queued, clear the IRQPEND flag so we know we have to prime
		; the serial line again.
		;
		isset?	STOP
		jnz	ComNoTrans

		test	comOut.numChars, 0ffffh
		jz	ComNoTrans

		mov	bx, comOut.head		; Fetch char at head
		mov	al, comOut.data[bx]
		mov	dx, comCurDataPort

		out	dx, al			; and send it
		Update	comOut.head, bx
		dec	comOut.numChars
		set	IRQPEND			; s/b set, but better safe...
		jmp	ComIntReturn

ComNoTrans:
		;
		; Nothing transmittable -- clear the CharSent flag, so we
		; know no interrupt is coming, and return.
		;
		reset	IRQPEND
	ifdef COM_DEBUG
		mov	al, 'Q'
		call	ScrWrite
	endif
		jmp	ComIntReturn

ComSendXOFF:
		;
		; Input buffer dangerously full -- send XOFF ourselves if
		; possible, else queue it for sending.
		;
		set	XONPEND		; Tell ComRead to send an
					; XON when the buffer empties
					; enough
		pop	ax		; Fetch line status again. We won't
					; be going to ComCheckTrans since we'll
					; know we can't send anything more. So
					; we pop the status for our own use.
		;
		; See if the transmitter is ready now. If not, try and queue
		; the XOFF for sending. One of these two should always happen,
		; if my logic is still working. As a fallback case, though,
		; we set the XOFFPEND flag to tell ComRead to send the XOFF
		; when it gets a chance.
		;
		test	al, COMTHRE
		jz	ComQueueXOFF?
		mov	dx, comCurDataPort
		mov	al, XOFF
		out	dx, al
		set	IRQPEND		; An interrupt will come in...
		jmp	ComIntReturn

ComQueueXOFF?:
		isset?	IRQPEND 	; See if we'll get an interrupt
		jz	ComPendXOFF	; Nope -- go tell ComRead to do it.
		;
		; Place an XOFF at the head of the output queue so it gets
		; sent ASAP.
		;
		mov	bx, comOut.head
		dec	bx
		jge	ComStuffXOFF
		mov	bx, BUFSIZE - 1		; Wrap
ComStuffXOFF:
		mov	comOut.data[bx], XOFF
		mov	comOut.head, bx
		inc	comOut.numChars
		jmp	ComIntReturn

ComPendXOFF:
		;
		; XXX: Should never get here
		;
		set	XOFFPEND	; Set flag so ComRead knows
					; to send an XOFF when the
					; serial line is ready.
		jmp	ComIntReturn

ComInterrupt	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComReadWithWait
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	read until we actually find something

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@


ComReadWithWait	proc	near
time	local	dword
	uses	ds, bx
	.enter
	GetTimeLocation	ds, bx
	mov	ax, ds:[bx]
	mov	time.low, ax
	mov	ax, ds:[bx+2]
	mov	time.high, ax
readLoop:
	push	cs
	pop	ds
	call	ComRead
	jnz	gotIt

	; if the high words are off by 2 or more, then its been a long time
	; so timeout
	GetTimeLocation	ds, bx		; ds:[bx] = pointer to time location
	mov	ax, ds:[bx+2]	
	sub	ax, time.high
	cmp	ax, 2
	jae	timeOut	

	; otherwise the diffeence is just the subtraction of the low words
	mov	ax, ds:[bx]
	sub	ax, time.low
	cmp	ax, 182		; 182 = 10 seconds
	jae	timeOut
	jmp	readLoop
gotIt:
	clc
done:
	.leave
	ret
timeOut:
	stc
	jmp	done
ComReadWithWait	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComDrainQueue
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	loop until the queue is empty

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	7/31/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
	public ComDrainQueue
ComDrainQueue	proc	near
	.enter
drainLoop:
	cmp	comOut.numChars, 0
	jg	drainLoop
	.leave
	ret
ComDrainQueue	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComRead
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Read a byte from the serial device, if any there.

CALLED BY:	Main

PASS:		Nothing

RETURN:		Z = 1 if nothing there, else
		Z = 0 and AL contains byte.

DESTROYED:	AL

PSEUDO CODE/STRATEGY:
	If XOFFPEND is set, we're supposed to send an XOFF, so do it.

	See if the char count for the input buffer is non-zero.
	If so, there's a byte available, so return it, stupid.
		If this reduces the number of characters to the low water
		mark, and we're supposed to send an XON, send it.
	Else leave Z set and return.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComRead
ComRead		proc	near
		push	bx

		isset?	XOFFPEND		; See if we should send
						; an XOFF
		jnz	ComReadSendXOFF
ComRead1:
		;
		; Because the LogiTech Bus Mouse Driver doesn't chain the
		; interrupt, make sure we still own the vector and replace
		; it if we don't...
		;
		push	dx		; S.O.R.
		mov	ax, comIntVecNum
		mov	bx, offset comChainVec
		mov	dx, offset ComInterrupt
		call	SetInterrupt
		pop	dx

		test	comIn.numChars, 0ffffh	; Anything waiting?
		jz	ComReadRet		; Nope

		mov	bx, comIn.head
		mov	al, comIn.data[bx]
		Update	comIn.head, bx
		dec	comIn.numChars
		isset?	XONPEND
		jnz	ComReadCheckLowWat
ComReadRetNZ:
		or	bx, 1			; Make sure Z is clear
ComReadRet:
		pop	bx
		ret

ComReadCheckLowWat:
		;
		; See if we've reached the low-water mark yet
		;
		cmp	comIn.numChars, LOWWATER
		jne	ComReadRet

		;
		; Reached low water. Clear the flag and send an XON
		;
		reset	XONPEND
		push	ax
		mov	al, XON
		call	ComRawWrite
		pop	ax
		jmp	ComReadRetNZ

ComReadSendXOFF:
		;
		; An XOFF is pending, so send it now. We can afford to
		; wait for the transmitter to become ready now, something
		; we couldn't do at interrupt time.
		;
		reset	XOFFPEND
		mov	al, XOFF
		call	ComRawWrite
		jmp	ComRead1
ComRead		endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComRawWrite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Write the character to the serial port w/o regard for
		flow control.

CALLED BY:	ComRead, ComWrite

PASS:		AL	= character to send

RETURN:		Nothing

DESTROYED:	AX, DX

PSEUDO CODE/STRATEGY:
	Wait for transmitter to become ready
	Send AL

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	IRQPEND is set true

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/21/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ComRawWrite	proc	near
		push	ax

		mov	dx, comCurStatPort
CRWLoop:	in	al, dx
		test	al, COMTHRE
		jz	CRWLoop

		pop	ax
		mov	dx, comCurDataPort
		out	dx, al

		set	IRQPEND
		mov	dx, comCurIENPort
		mov	al, COMRECVENABLE OR COMTRANSENABLE
		out	dx, al

		ret
ComRawWrite	endp
		bugsym	ComRawWrite


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComWrite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Write a byte to the serial port.

CALLED BY:	Main

PASS:		AL = the byte to write

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Wait for the port to become ready.
	Write the data byte.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	This should also be interrupt driven, eventually, methinks.

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComWrite
ComWrite	proc	near
		push	ax
		push	bx
		push	dx		; S.O.R.

ComWriteLoop:
		;
		; If output is disabled, spin until it's enabled again.
		;
		isset?	STOP
		jnz	ComWriteLoop

		dsi			; So we don't miss the interrupt...

		isset?	IRQPEND
		jz	ComWaitAndSend	; No interrupt will come -- we must
					; wait and send the character
					; by hand.
ComWriteQueue:
		mov	bx, comOut.tail
		mov	comOut.data[bx], al
		Update	comOut.tail, bx
		inc	comOut.numChars
		eni
		;
		; If output buffer over the high water mark, allow it to
		; drain before we return. Note we enable interrupts before
		; this....
		;
		cmp	comOut.numChars, HIGHWATER
		jge	ComWriteEmpty
ComWriteReturn:
		pop	dx
		pop	bx
		pop	ax
		ret

ComWriteEmpty:
		;
		; Wait for the output buffer to drain
		;
		cmp	comOut.numChars, LOWWATER
		jge	ComWriteEmpty
		jmp	ComWriteReturn

ComWaitAndSend:
		;
		; No transmitter interrupt will be coming in, so we must
		; send a character by hand. Call ComRawWrite to do this.
		;
		eni
		call	ComRawWrite
		jmp	ComWriteReturn
ComWrite	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComSetRawMode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the serial line into or out of raw mode

CALLED BY:	FileReceive, FileSend

PASS:		CX	= 1 if raw mode
			= 0 if cooked mode
		AX	= bit 0 = 1 if should flush input
			  bit 1 = 1 if should flush output

RETURN:		Nothing

DESTROYED:	CX

PSEUDO CODE/STRATEGY:
	Set the RAW flag and flush the input and output buffers

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/15/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComSetRawMode
ComSetRawMode 	proc	near
		dsi
		jcxz	ComSRMReset
		set	RAW
ComSRMFlush:
		clr	cx
		test	ax, 1
		jz	ComSRM2
		mov	comIn.head, cx
		mov	comIn.tail, cx
		mov	comIn.numChars, cx
ComSRM2:
		test	ax, 10b
		jz	ComSRM3
		mov	comOut.head, cx
		mov	comOut.tail, cx
		mov	comOut.numChars, cx
ComSRM3:
		eni
		ret
ComSRMReset:
		reset	RAW
		jmp	ComSRMFlush
ComSetRawMode	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComReadBlock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Read a block of bytes from the serial line

CALLED BY:	FileReceive, FileSend

PASS:		DI = Offset of buffer
		ES = Segment of buffer
		CX = Number of bytes to read

RETURN:		Nothing

DESTROYED:	DI, CX, AX, DF = 0

PSEUDO CODE/STRATEGY:
	Call ComRead until CX goes to 0

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComReadBlock
ComReadBlock	proc	near
		cld
		jcxz	ComReadBlockRet
ComReadBlock1:
		call	ComRead
		jz	ComReadBlock1
		stosb
		loop	ComReadBlock1
ComReadBlockRet:
		ret
ComReadBlock	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ComWriteBlock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Write a block of data from a buffer

CALLED BY:	FileSend, FileReceive

PASS:		SI = offset of buffer
		DS = segment of buffer
		CX = number of bytes to write

RETURN:		Nothing

DESTROYED:	SI, CX, AX, DF = 0

PSEUDO CODE/STRATEGY:
	Call ComWrite repeatedly until CX is 0

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ComWriteBlock
ComWriteBlock	proc	near
		cld
		jcxz	ComWriteBlockRet
ComWriteBlock1:
		lodsb
		call	ComWrite
		loop	ComWriteBlock1
ComWriteBlockRet:
		ret
ComWriteBlock	endp

program		ends
		end

