COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

PROJECT:	PC GEOS
MODULE:		Scr -- Screen routines for com43.
FILE:		scr.asm

AUTHOR:		Adam de Boor

ROUTINES:
	Name			Description
	----			-----------
	ScrInit			Initialize the screen
	ScrExit			Restore the screen
	ScrWrite		Write a character to the screen
	ScrWriteString		Write a string of characters to the screen
	ScrPrintByte		Print a byte in hex to the screen
	ScrEnable		Re-enable the screen

	
REVISION HISTORY:
	Name	Dat		Description
	----	----		-----------
	adam	6/7/88		Initial Revision
	adam	9/27/88		Added screen-saver
	jdm	91.01.25	Cleaned up screen restoration
	jdm	91.01.30	Added /v switch to force conservative
				screen setting.
	jdm	91.02.20	Added default pallette restoration.


DESCRIPTION:
	This module deals with the screen for com43.

	The screen is placed in 80x[43|50] mode after the EGA/VGA is
	initialized.
	Escape codes are parsed (including those for file transfer) and
	acted upon.
		
	$Id: scr.asm,v 2.17 93/07/30 16:14:52 jimmy Exp $

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
SCRMODULE	=	1
		include	com43.def

program		segment
NUMCOLS		= 80	; number of columns (currently always 80).

;
; Id numbers for the different types of graphic cards.
;
TYPE_UNKNOWN	= 0
TYPE_MDA	= 1	; Unused
TYPE_CGA	= 2	; 
TYPE_EGA	= 3
TYPE_MCGA	= 4	; Unused
TYPE_VGA	= 5
TYPE_HGC	= 80h
TYPE_HGCPLUS	= 81h	
TYPE_INCOLOR	= 82h	; Unused

BLACK		= 00h
BLUE		= 01h
GREEN		= 02h
CYAN		= 03h
RED		= 04h
MAGENTA		= 05h
BROWN		= 14h
WHITE		= 07h
GRAY		= 38h
LTBLUE		= 39h
LTGREEN		= 3ah
LTCYAN		= 3bh
SALMON		= 3ch
LTMAGENTA	= 3dh
YELLOW		= 3eh
INTWHITE	= 3fh

;
; Color palette assignments for the various pieces
;
FG		= YELLOW
BG		= GRAY
MOUSE		= LTBLUE
BOLD		= MAGENTA
UNDERLINE	= RED


defColors	db	BG, FG, BOLD, UNDERLINE, MOUSE
defColorStr	db	";;;;;",0
scrColorVar	db	"COM43COLORS=",0

;
; Pallette strings used to re-set the color pallette on exit.
; Since the EGA pallette registers are read-only, the EGA string
; is just the standard default color settings.
; For the VGA, we just use the EGA stuff.
;
egaPalletteStr 	db 	"0;1;2;3;4;5;14;7;38;39;3a;3b;3c;3d;3e;3f;0;", 0
		even

;
; Scr attribute byte definitions. The normal foreground is color 1,
; bold is color 2, underline is color 3, the mouse is color 4.
;
HGCNORMAL	=	07h
CGANORMAL	=	07h
EGANORMAL	=	01h
SCRBOLD		=	02h
SCRUNDERLINED 	=	03h
SCRREVERSE	=	10h

;
; Function definitions
;
VIDSETMODE	=	0	; Set display mode
				; al contains mode. We use 3 to produce
				; 640x350 color alpha mode. When
				; combined with the 8x8 character set and
				; an enhanced color display, this
				; gives a resolution of 80x43

VIDSETCURSORTYPE =	1h	; Set the cursor type. cl contains scan line
				; for bottom of cursor, while ch contains
				; scan line (- 1) for top of cursor.
VIDSETCURSOR	=	02h	; Set cursor position
VIDGETCURSOR	=	03h	; Get cursor position

VIDSETPAGE	=	05h	; Set the active screen page

VIDWRITECHAR	=	09h	; Write character and attribute at cursor

VIDGETMODE	=	0fh	; Fetch video mode

VIDSETCHARGEN 	=	11h	; Set character generator mode
				; al contains mode. CHARGEN8x8 says to
				; use the 8x8 font, char_gen_mono says use
				; the regular monochrome font. There are
				; lots of other choices, but most of them
				; are user-defined and, thus, unimportant
				; for us.
CHARGEN8x8	=	12h	; use 8x8 double-dot font
CHARGENMONO	=	11h	; use monochrome font

VIDMISC		=	12h	; Miscellaneous functions. bl selects what
				; is changed: 10h alters color/mono, mem size,
				; feature bits and switch setting.
				; 20h means to use alternate routine to print
				; the scr, when the PrtSc key is pressed.
ALTPRTSCR	=	20h

;
; Video BIOS definitions -- absolute references (i.e. segment 0)
;
VIDNUMCOLS	=	44ah	; Number of columns on screen (word)
VIDBUFFERLEN	=	44ch	; Length of buffer in bytes (word)
VIDSTARTADDR	= 	44eh	; Offset into video segment of frame buffer
VIDNUMROWS	=	484h	; Number of rows-1 (byte)
;
; Definitions for the hardware itself, rather than its BIOS
;
HGCBASEADDR	=	0b000h	; Base address of HGC mode 7 buffer.
CGABASEADDR	=	0b800h	; Base address of EGA mode 7 buffer.
EGABASEADDR	=	0b800h	; Base address of EGA mode 7 buffer. Buffer is
				; formatted as words: ascii code in low byte,
				; attribute in high byte.
HGCADDRPORT	=	03b4h	; Index register for mono adapters (e.g. HGC)
CGAADDRPORT	=	03d4h	; Index register 
EGAADDRPORT	=	03d4h	; CRTC Index register (See ScrSetCursor)

EGAATTRINIT	=	03dah	; Port to read to init attribute chip
EGAATTRPORT	=	03c0h	; Address/Data port for attribute chip. It is
				; initialized to be the address port by reading
				; port EGAATTRINIT. It then alternates.
EGACURSHIGH	=	0eh	; High part of cursor address
EGACURSLOW	=	0fh	; Low part of cursor address
CRTCMODE	=	17h	; CRTC mode
REGULARMODE	=	0a3h	;	Standard mode
BLANKMODE	=	0b3h	;	Standard mode with video disabled

MAXPAGE		=	4	; Four pages will fit in the 32K we have

;
; vidcall is used to invoke the VIDEO BIOS. It places its argument in
; ah and traps to the BIOS via software interrupt level 10h
;
vidcall		macro	num
		mov	ah, num
		int	10h
		endm

Mul160		macro	reg
		push	cx
		sal	reg, 1		; Multiply by 32 first
		sal	reg, 1
		sal	reg, 1
		sal	reg, 1
		sal	reg, 1
		mov	cx, reg		; save reg * 32 in cx
		sal	reg, 1		; make reg * 64
		sal	reg, 1		; reg * 128
		add	reg, cx		; Add in reg * 32
		pop	cx
		endm

Mul80		macro	reg, save
	ifnb <save>
		push	save
	endif
		shl	reg, 1
		shl	reg, 1
		shl	reg, 1
		shl	reg, 1
	ifnb <save>
		mov	save, reg
	else
		mov	cx, reg		; Y * 16 to CX
	endif
		shl	reg, 1
		shl	reg, 1

	ifnb <save>
		add	reg, save
		pop	save
	else
		add	reg, cx		; Y * 64 + y * 16 = Y * 80 in reg
	endif
		endm


		bugsym	scrX
		bugsym	scrY
		bugsym	scrSavedX
		bugsym	scrSavedY
		bugsym	scrTop
		bugsym	scrBot
		bugsym	scrAttr
		bugsym	scrSeq
		bugsym	scrFlags
		bugsym	scrArg1
		bugsym	scrArg2
		bugsym	scrHandler
		
		bugsym	scrType
		bugsym	scrSegment
		public	scrLines
		bugsym	scrAddrPort
		bugsym	scrNormal
		public	scrEtcFlags
		bugsym	scrMode
		bugsym	scrCols

;
; Adapter-specific variables
;
scrSegment	dw	?	; Segment (base address) of the video memory.
scrLines	dw	?	; Maximum lines for current display
scrAddrPort	dw	?	; CRTC address port
scrType		db	?	; Type of video card.
scrNormal	db	?	; Attribute byte that is considered normal.
scrMode		db	?	; Previous video mode number.
scrCols		db	?	; Previous number of columns.

;
; Command line switch for forcing conservative screen restoration.
;
scrCmdLineArgFlag	dw	0
scrCmdLineArg		db	'v', 0

;
; Miscellaneous state
;
		even
scrEtcFlags	db	0	; Flags for external use.
scrPage		db	0
scrDisPage	db	0
		even

scrX		dw	0		; Current X coordinate
scrY		dw	0		; Current Y coordinate
scrSavedX	dw	0		; Saved X coordinate
scrSavedY	dw	0		; Saved Y coordinate
scrTop		dw	0		; Current top of scrolling region
scrBot		dw	?		; Current bottom of scrolling region

scrAttr		db	EGANORMAL	; Current attribute for characters
scrHighBit  	db  	0   	    	; 80h if should set the high bit of
					; all characters.
		
scrVb		db	RED		; Visible bell color

scrStart	dw	0		; Current screen start (offset from
					; scrSegment)
;
; Area for recording the state of various pages when they aren't being
; displayed. All are initialized to reasonable values so things to get
; completely hosed should someone switch to a page without initializing
; it.
; 
scrStartSave	dw	MAXPAGE DUP(0)	; Saved start of all pages
scrXSave	dw	MAXPAGE DUP(0)	; Per-page scrX
scrYSave	dw	MAXPAGE DUP(0)	; Per-page scrY
scrSavedXSave	dw	MAXPAGE	DUP(0)	; Per-page scrSavedX
scrSavedYSave	dw	MAXPAGE DUP(0)	; Per-page scrSavedY
scrTopSave	dw	MAXPAGE DUP(0)	; Per-page scrTop
scrBotSave	dw	MAXPAGE DUP(?)	; Per-page scrBot
scrAttrSave	db	MAXPAGE DUP(EGANORMAL)	; Per-page scrAttr
scrHighBitSave 	db  	MAXPAGE DUP(0)	; Per-page scrHighBit

scrSeq		dw	?		; Place to store escape sequence
scrFlags	db	0		; Parser state flags
SCRGETFIRST	equ	00000001b	; Get first char of esc sequence
SCRGETSECOND	equ	00000010b	; Get second char of esc sequence
SCR1ARG		equ	00000100b	; Sequence takes one argument
SCR2ARG		equ	00001000b	; Sequence takes two arguments

scrArg1		db	?		; First argument read
scrArg2		db	?		; Second argument read
		even
scrHandler	dw	?		; Function to call when have args

;
; scrCtlHandlers are jump vectors for handling control characters. The
; byte is translated using scrControls to yield an index into scrCtlHandlers
; from which the address to which to jump is extracted and the jump performed
;
scrCtlHandlers	label	word
DISCARDSLOT	= 0
		dw	offset ScrDiscard
BSSLOT		= 1
		dw	offset ScrBackspace
TABSLOT		= 2
		dw	offset ScrTab
NEWLINESLOT	= 3
		dw	offset ScrNewline
FORMFEEDSLOT	= 4
		dw	offset ScrFormFeed
RETURNSLOT	= 5
		dw	offset ScrReturn
ESCSLOT		= 6
		dw	offset ScrEscape
BACKTABSLOT	= 7
		dw	offset ScrBackTab
BELLSLOT	= 8
		dw	offset ScrBeep

scrControls	db	0, 0, 0, 0, 0, 0, 0, BELLSLOT, BSSLOT
		db	TABSLOT, NEWLINESLOT, BACKTABSLOT
		db	FORMFEEDSLOT, RETURNSLOT
		db	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		db	ESCSLOT, 0, 0, 0, 0

;
; Screen-saver data
TIMER_VEC	= 1ch			; Vector number for timer 0. This is
					; the software interrupt MS-DOS uses
					; every tick to give control to the
					; user.
SCR_TIME	= 10920			; This is 10 minutes at 18.2 interrupts
					; per second. The keyboard and mouse
					; must have been idle for this long
					; before the screen is turned off.

scrDoSave	db	1		; non-zero if should save the screen
scrTime		dw	SCR_TIME	; Ticks remaining before screen
					; disabled.
scrTimerVec	dd	?		; Chain of timer interrupt handlers

;
; Constants for ScrMoveCursor. Loaded into CX to tell what needs adjusting
;
SCRADJUSTX	= 0
SCRADJUSTY	= 1


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrNukeMouse
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Remove the cursor if displaying the page we're about to
		modify.

CALLED BY:	Many things. ScrClear, ScrScrollForw, ScrScrollRev.
PASS:		Nothing
RETURN:		Nothing
DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
       If scrPage and scrDisPage are the same, then remove the cursor.
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 6/88	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrNukeMouse
ScrNukeMouse	proc	near
		push	ax			; Save ax (really?)
		mov	al, scrPage		; Fetch about-to-be-modified
						; page.
		cmp	al, scrDisPage		; Same as displayed page?
		jne	SNM1			; Nope. Do nothing.
		call	MouseRemoveCursor	; Yes, remove the cursor
SNM1:
		pop	ax
		ret
ScrNukeMouse	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSetColor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the next color value

CALLED BY:	ScrInit
PASS:		[ES:DI]	= first character of the color value. Value ends
			  in a semi-colon.
		AX	= color being set -- 0 - 4
		DX	= palette port.
RETURN:		Nothing
DESTROYED:	AX, BX

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	DI is advanced to the first character of the next color value.
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/29/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrSetColor
ScrSetColor	proc	near
		out	dx, al			; Tell the ega what color we're
						; setting.
		mov	bx, ax			; Need the color in bx as an
						; index into defColors

		mov	al, es:[di]		; See if default color wanted
		cmp	al, ';'
		jne	SSC0_5			; No
		mov	al, defColors[bx]	; Yes -- fetch default
		jmp	short SSCSet		; Go set the thing
SSC0_5:
		cmp	al, 0
		jne	SSC1
		mov	al, defColors[bx]
		jmp	short SSCSetNoInc
SSC1:
		clr	ah
		cmp	byte ptr es:1[di], ';'	; single digit?
		je	SSC2			; yes
		sub	al, '0'			; high digit must be 0-3.
		ror	ax, 1			; Shift to high nibble of ah
		ror	ax, 1
		ror	ax, 1
		ror	ax, 1
		inc	di			; Advance to next char
		mov	al, es:[di]		; Fetch it
SSC2:
		cmp	al, '9'			; 0-9?
		jg	SSC3			; no
		sub	al, '0'			; Convert to binary
		jmp	short SSC5		; Merge with high nibble
SSC3:
		cmp	al, 'F'			; A-F?
		jg	SSC4			; no
		sub	al, 'A' - 10		; Convert to binary
		jmp	short SSC5		; Merge
SSC4:
		sub	al, 'a' - 10		; a-f
SSC5:
		inc	di			; Advance to semi-colon
		or	al, ah			; Merge the nibbles
SSCSet:
		inc	di			; Advance past semi
SSCSetNoInc:
		out	dx, al			; Set the color
		ret
ScrSetColor	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrResetColors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Reset the color pallette to the EGA defaults for those
		colors manipulated during initialization.

CALLED BY:	ScrExit
PASS:		Void.
RETURN:		Nothing
DESTROYED:	???

PSEUDO CODE/STRATEGY:
	Call ScrSetEVGAMode with the standard default color pallette to
	put the screen into 80x43/50 line mode.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	Save/restore registers?
		
REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	JDM	91.02.20	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrResetColors
ScrResetColors	proc	near	uses	es
		.enter

		;
		; Reset the E/VGA to the default pallette.
		;
		push	ds				; ES:DI = default
		pop	es				; color pallette
		mov	di, offset egaPalletteStr	; string.
		call	ScrSetEVGAMode

		.leave
		ret
ScrResetColors	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSetEVGAMode
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Reset the color pallette to the EGA defaults for those
		colors manipulated during initialization.

CALLED BY:	ScrResetColors, ScrInitEGA, ScrInitVGA

PASS:		ES:DI = pointer to pallette string to use for color
			setting.

RETURN:		Nothing
DESTROYED:	???
	Trashs a lot of stuff :-).

PSEUDO CODE/STRATEGY:
	Set the globally accessed adapter dependent identifiers.
	Put the adapter into 80x43/50 line mode.
	Tell the adapter to use intensity rather than blinking.
	Check the environment for the user's color preferences.  If they
	exist use them, otherwise use the passed in color pallette string.
	Make the cursor a blinking single line.
	Tell the adapter to use the alternate printscreen routine.

KNOWN BUGS/SIDE EFFECTS/IDEAS:	???
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	JDM	91.02.20	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrSetEVGAMode
ScrSetEVGAMode	proc	near
		;
		; Save the pallette string pointer for later use.
		;
		push	es
		push	di

		;
		; Set global identifiers.
		;
		mov	scrNormal,EGANORMAL		; 'Normal' attribute.
		mov	scrSegment, EGABASEADDR		; Video start address.
		mov	scrAddrPort, EGAADDRPORT	; Video control port.

		;
		; First set to 80x43(50) mode (using an 8x8 font).
		; Video Mode three is either 640x350 for EGA or
		; 640x400 for VGA.
		;
		mov	al, 3
		vidcall	VIDSETMODE
		clr	bl
		mov	al, CHARGEN8x8
		vidcall	VIDSETCHARGEN

		;
		; Turn off blinking mode and give it the color display
		; attributes...
		;
		mov	dx, EGAATTRINIT		; Set attribute flip-flop.
		in	al, dx

		mov	al, 3			; Set intensity on.
		mov	bl, 0			; I.e. blinking off.
		vidcall	10h

		in	al, dx			; Clear attrib flip-flop.

		;
		; DX = E/VGA attribute controller port.
		;
		mov	dx, EGAATTRPORT

		;
		; Check for user defined colors (in environment).
		;
		mov	si, offset scrColorVar
		call	GetEnv
		jnc	SSEVGAM2		; Found variable

		;
		; ScrSetColor expects:
		; AX == [0-4] being the background, foreground, bold
		;	underline, and mouse respectively.
		; DX == Pallette port to write to.
		; ES:DI == pointer to the pallette color string to use.
		;
		pop	di			; Restore save pallette
		pop	es			; pointer.
		push	ax			; Stupid hack.  Didn't feel
		push	ax			; like restructing the logic.
SSEVGAM2:
		pop	ax			; Get rid of the passed
		pop	ax			; in stuff (if necessary).

		mov	ax, 0			; background
		call	ScrSetColor		;
						;
		mov	ax, 1			; normal foreground
		call	ScrSetColor		;
						;
		mov	ax, 2			; bold foreground
		call	ScrSetColor		;
						;
		mov	ax, 3			; underline foreground
		call	ScrSetColor		;
						;
		mov	ax, 4			; mouse background
		call	ScrSetColor		;

		;
		; Turn the palette back on again
		;
		mov	al, 20h
		out	dx, al

		;
		; Make the cursor into a 7-line blinking block
		;
		mov	cl, 7			; end cursor at 7
		clr	ch			; start cursor at 0
		vidcall	VIDSETCURSORTYPE	;

		;
		; Use the new and improved (or not :-) print screen routine.
		;
		mov	bl, ALTPRTSCR
		vidcall	VIDMISC

		ret
ScrSetEVGAMode	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize the screen

CALLED BY:	Main

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX, BX

PSEUDO CODE/STRATEGY:
	Figure out type of and initialize for the current display
	adapter.
	Clear the screen and and install our timer interrupt handler for
	the screen-saver function

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	Some VGA adapters incorrectly return codes that are supposed to
	tell us that the adapter supports certain functions when they
	obviously don't.

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision
	adam	9/27/88		Added screen saver stuff
	jcw	1/89		Added extra card support
	jdm	1/25/91		Added vga support
	jdm	1/30/91		Added /v command line switch

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	ScrInit
ScrInit		proc	near
		call	ScrFindCard		; Get type of the video card
						; and initialize the various
						; adapter-specific parameters.
		jnc	SIUnsupported		;
		;
		; Check for /v command line switch.
		; If found force ScrExit to set the screen to
		; something safe.  This leaves some the stupid
		; (i.e. lying) VGA clone adapters in a clean state.
		;
		mov	[scrCmdLineArgFlag], 0	; Assume okay.
		mov	di, offset scrCmdLineArg
		clr	bx
		call	FetchArg
		tst	bx
		jz	SI_2			; no switch given
		mov	[scrCmdLineArgFlag], 1	; /v flag given
		
SI_2:
		call	ScrClear		;
						;
		mov	ax, TIMER_VEC		;
		mov	bx, offset scrTimerVec	;
		mov	dx, offset ScrTick	;
		call	SetInterrupt		;
						;
		mov	scrDoSave, 1		; Re-enable screen saver by
						;  default
		ret				;

SIUnsupported:
		mov	dx, offset notSuppMsg	;
		mov	ah, 9			; Print string
		int	21h			;
		jmp	Exit			;
notSuppMsg	db	"To use com43 you must have either a VGA, EGA "
		db	"or Hercules card.", cr, lf, "$"
ScrInit		endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInitHercules
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Init the hercules graphic card for 25 line mode.

CALLED BY:	ScrFindCard
PASS:		nothing
RETURN:		nothing
DESTROYED:	

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	1/12/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrInitHercules
ScrInitHercules	proc	near
		mov	scrNormal,HGCNORMAL	; Define "normal" for 
						; ScrInitPage.

		mov	scrSegment, HGCBASEADDR
		mov	scrAddrPort, HGCADDRPORT

		or	scrEtcFlags, SCR_ISMONO
		
		;
		; Make the cursor into a 13-line blinking block
		;
;		mov	cl, 13			; end cursor at 13
;		clr	ch			; start cursor at 0
;		vidcall	VIDSETCURSORTYPE	;

		ret				;
ScrInitHercules	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInitCGA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Init the CGA card

CALLED BY:	ScrFindCard
PASS:		nothing
RETURN:		nothing
DESTROYED:	

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	mav	1/26/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrInitCGA
ScrInitCGA	proc	near
		mov	scrNormal,CGANORMAL	; Define "normal" for 
						; ScrInitPage.

		mov	scrSegment, CGABASEADDR
		mov	scrAddrPort, CGAADDRPORT

		or	scrEtcFlags, SCR_ISMONO
		
		;
		; Make the cursor into a 13-line blinking block
		;
		mov	cl, 13			; end cursor at 13
		clr	ch			; start cursor at 0
		vidcall	VIDSETCURSORTYPE	;

		push	es
		clr	ax
		mov	es, ax
		mov 	ax, 25
		mov	es:VIDNUMROWS, ax	; 
		pop	es

		ret				;


ScrInitCGA	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInitHGCPlus
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize an HGC+ adapter

CALLED BY:	ScrFindCard
PASS:		Nothing
RETURN:		Nothing
DESTROYED:	...

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	1/24/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrInitHGCPlus
ScrInitHGCPlus	proc	near
		mov	scrNormal,HGCNORMAL	; Define "normal" for 
						; ScrInitPage.

		mov	scrSegment, HGCBASEADDR
		mov	scrAddrPort, HGCADDRPORT

		or	scrEtcFlags, SCR_ISMONO OR SCR_ISWEIRD
		;
		; Turn off blinking mode
		;
		mov	dx, 3b5h
		mov	al, 8			; No blink, video on, a/n
		out	dx, al

		;
		; Switch to 44-line mode
		;
		mov	dx,3bfh			; configuration switch port
		mov	al,1			; 1 = set mem size to 32K.
		out	dx,al			;
						;
		mov	dx,3bdh			; configuration switch port
		mov	al,4			; set to 132 column char set.
		out	dx,al			;
						;
		mov	dx,3b4h			; crt address port
						;
		mov	si,offset CRTCParams	; vertical and horizontal
		cld				; parameters.
		mov	cx,10			; (see table below).
		clr	al			;
SIHGCP_paramLoop:				;
		out	dx,al			;Set register #
		inc	al			;Increment for next pass
		mov	ah,al			;Preserve it
		lodsb				;Get parameter
		inc	dx			;Data register...
		out	dx,al			; send it
		mov	al,ah			;Recover register #
		dec	dx			;Back to address register.
		loop	SIHGCP_paramLoop	;
						;
		push	es			;
		clr	ax			;
		mov	es, ax			; BIOS r Us.
		mov	byte ptr es:VIDNUMROWS, 43
		pop	es			;

		;
		; Make the cursor into a 7-line blinking block... Well,
		; for no readily apparent this needs to be 13 in order to
		; make the cursor show up anywhere but the top line...
		;
		mov	cl, 13			; end cursor at 7
		clr	ch			; start cursor at 0
		vidcall	VIDSETCURSORTYPE	;
		ret				;
ScrInitHGCPlus	endp

CRTCParams	db	9ah	; horizontal total (character clocks)
		db	50h	; display end (character clocks)
		db	6dh	; start h-blank
		db	28h	; end h-blank
		db	2dh	; vertical total
		db	02h	; total adjust.
		db	2ch	; vertical display
		db	2ch	; start vert retrace.
		db	02h	; interlace mode.
		db	07h	; max scan lines.
;		db	00h	; cursor start.
;		db	07h	; cursor end.


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInitEGA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize the EGA card for 43 line mode.

CALLED BY:	ScrInit
PASS:		Void.
RETURN:		Void.
DESTROYED:	???

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	1/12/89		Initial version
	JDM	91.02.20	Abstracted functionality to ScrSetEVGAMode.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrInitEGA
ScrInitEGA	proc	near	uses	es
		.enter

		;
		; Initialize the EGA to our defaults.
		;
		push	ds				; ES:DI = default
		pop	es				; color pallette
		mov	di, offset defColorStr		; string.
		call	ScrSetEVGAMode

		.leave
		ret
ScrInitEGA	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInitVGA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize the VGA card for 50 line mode.

CALLED BY:	ScrInit
PASS:		Void.
RETURN:		Void.
DESTROYED:	???

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jdm	1/25/91		Initial version
	JDM	91.02.20	Abstracted functionality to ScrSetEVGAMode.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrInitVGA
ScrInitVGA	proc	near
		;
		; Basically we do the same thing as the EGA initializer
		; with some extras so...
		;
		call	ScrInitEGA

		;
		; Enable the Video Refresh.
		;
		mov	ax, 1200h		; Func. 0x12, enable
		mov	bl, 36h			; Sub. Func.
		int	10h
		cmp	al, 12h
		je	SIV_DOKY
		mov	scrMode, 3
SIV_DOKY:
		ret				;
ScrInitVGA	endp




COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrExit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Return the screen to its former condition

CALLED BY:	Exit

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX, CX, BX

PSEUDO CODE/STRATEGY:
	If the screen restoration flag was set then reset the video mode
	to 80x25.  If using a VGA adapter then leave the adapter in
	80x50 mode.  Otherwise, reset the old video mode.  Get rid of the
	Timer.  Enable the screen and reset the cursor.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	I wish there were a way to get the cursor start and end (could look
	at the CRTC registers, I suppose...) so I could restore it.

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision
	adam	9/27/88		Added screen saver stuff
	jdm	1/25/91		Restores old video mode.
	jdm	1/30/91		Handle forced restoration flag.
	jdm	2/18/91		Added pallette restoration.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	ScrExit
ScrExit		proc	near
		;
		; Clear the screen
		;
		call	ScrClear

		;
		; Check for the safe screen restoration flag.
		;
		mov	ax, [scrCmdLineArgFlag]
		tst	ax
		jz	SE1
		mov	scrMode, 3
		jmp	short SE2
SE1:
		;
		; If on a VGA do NOT reset the video mode.
		;
		cmp	[scrType], TYPE_VGA
		jne	SE2

		;
		; Reset the colors that we trashed during initialization.
		;
		call	ScrResetColors

		; Skip the video mode reset.
		jmp	short	SE3

SE2:
		;
		; Have we been using an EGA?
		;
		cmp	[scrType], TYPE_EGA
		jne	SESetMode		; Nope.

		;
		; Reset the colors that we trashed during initialization.
		;
		call	ScrResetColors

SESetMode:	
		;
		; Reset the old video mode
		;
		xor	ah, ah			; Set Video Mode
		mov	al, [scrMode]		; Old mode #.
		int	10h
SE3:
		;
		; Detach from the timer chain and re-enable the screen.
		;
		mov	ax, TIMER_VEC
		mov	bx, offset scrTimerVec
		call	ResetInterrupt

		call	ScrEnable

		;
		; Move BIOS's cursor to the home position.
		;
		clr	dx
		clr	bh
		vidcall	VIDSETCURSOR

		ret
ScrExit		endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrScrollUp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Scroll the current scrolling region up

CALLED BY:	INTERNAL

PASS:		CX	= number of lines to scroll
		scrTop	= top of region to scroll
		scrBot	= bottom of region to scroll
		scrStart= offset of start of screen
		scrAttr	= attribute for spaces in emptied lines

RETURN:		Nothing

DESTROYED:	AX, BX, CX, SI, DI

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	8/23/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrScrollUp	proc	near
		push	es
		push	ds

		;
		; Fetch start of scrolling region into di (it's where the
		; text will go)
		;
		mov	ax, scrTop
		Mul160	ax
		add	ax, scrStart
		mov	di, ax

		;
		; Multiply the line count by 160 to get the number of bytes
		; to shift
		;
		mov	bx, cx
		Mul160	bx

		add	ax, bx	; Add the number of bytes to get the first
				; line that stays
		mov	si, ax

		;
		; Figure the number of bytes to shift by subtracting the
		; number of bytes in the lines we'll be clearing (CX*160)
		; from the total number of bytes in the region (bot-top)*160
		;
		mov	ax, scrBot
		inc	ax
		sub	ax, scrTop
		Mul160	ax
		sub	ax, bx
		sar	ax, 1	; moving words
		mov	cx, ax

		;
		; Load es and ds with the screen segment for the move and
		; make sure the direction flag is clear
		;
		mov	ax, scrSegment
		mov	es, ax
		mov	ds, ax
		cld

		rep	movsw	; scroll

		;
		; Fill in the trailing lines with blanks of the current
		; attribute
		;
		mov	ah, cs:scrAttr
		mov	al, ' '
		sar	bx, 1	; We're moving words, so divide by 2
		mov	cx, bx

		rep	stosw

		pop	ds
		pop	es
		ret
ScrScrollUp	endp
		bugsym	ScrScrollUp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrScrollDown
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Scroll the current region down

CALLED BY:	INTERNAL

PASS:		CX	= number of lines to scroll
		scrTop	= top of region to scroll
		scrBot	= bottom of region to scroll
		scrStart= offset to start of page
		scrAttr	= attribute for spaces to fill vacated region

RETURN:		Nothing

DESTROYED:	AX, BX, CX, SI, DI

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	8/23/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrScrollDown	proc	near
		push	es
		push	ds

		;
		; Find the address of the last word in the region and
		; stuff it in di
		;
		mov	ax, scrBot
		inc	ax		; Need to start at end
		Mul160	ax
		sub	ax, 2		; Ditto
		add	ax, scrStart
		mov	di, ax

		;
		; Figure the number of bytes to shift
		;
		mov	bx, cx
		Mul160	bx

		;
		; Subtract that from the end ptr to get the last word of
		; the first line to stay on the screen
		;
		sub	ax, bx
		mov	si, ax

		mov	ax, scrBot
		inc	ax
		sub	ax, scrTop
		Mul160	ax
		sub	ax, bx
		sar	ax, 1
		mov	cx, ax

		mov	ax, scrSegment	; Load screen segment into es and ds
		mov	es, ax
		mov	ds, ax
		std			; Move down, please

		rep	movsw		; scroll

		;
		; Store spaces of the current attribute in the new lines
		;
		mov	ah, cs:scrAttr
		mov	al, ' '
		sar	bx, 1		; Shifting words, so /= 2
		mov	cx, bx

		rep	stosw

		pop	ds
		pop	es
		ret
ScrScrollDown	endp
		bugsym	ScrScrollDown



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSetCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the position of the EGA's cursor

CALLED BY:	ScrMoveCursor, ScrSetHorizPos, ScrMotion, ScrSetVertPos,
		ScrRestoreCursor

PASS:		AX	= Y coordinate of cursor
		DX	= X coordinate of cursor

RETURN:		Nothing

DESTROYED:	AX, DX

PSEUDO CODE/STRATEGY:
	The CRTC's view of memory is skewed by 1 bit (the hardware adds an
	extra zero so the CRTC can access the whole buffer in words), so
	rather than multiplying Y by 160 and X by 2, we only need to multiply
	Y by 80 and add X to it to form the word address in the refresh
	buffer. This skewing also applies to the base address, which we shift
	right one bit. We now have the correct address for the cursor in
	cx.

	The addressing of the CRTC's registers is done through two ports,
	one of which (EGAADDRPORT) specifies which register is to be
	referenced when EGADATAPORT is used.


KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/9/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrSetCursor	proc	near
		push	bx
		;
		; See if the current page is the one actually being displayed.
		; If not, don't change the on-screen cursor
		; 
		mov	bl, scrPage
		cmp	bl, scrDisPage
		jne	SSCRet
		
		push	cx

		Mul80	ax
		add	ax, dx		; AX = Y * 80 + X
		;
		; Add in offset to start of screen...
		;
		mov	bx, scrStart
		shr	bx, 1
		add	ax, bx
		mov	cx, ax

		;
		; Direct data port to CursorHigh and stuff the high byte in
		;
		mov	dx, scrAddrPort
		mov	al, EGACURSHIGH
		out	dx, al
		
		inc	dx
		mov	al, ch
		out	dx, al
		;
		; Direct data port to CursorLow and put it there
		;
		mov	dx, scrAddrPort
		mov	al, EGACURSLOW
		out	dx, al
		inc	dx

		mov	al, cl
		out	dx, al

		pop	cx
SSCRet:
		pop	bx
		ret
ScrSetCursor	endp
		bugsym	ScrSetCursor


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrMoveCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor in the given direction

CALLED BY:	Various places

PASS:		CX	= 0 if adjusting X, 1 if adjusting Y
		AX	= amount to adjust

RETURN:		Nothing

DESTROYED:	scrX and scrY changed

PSEUDO CODE/STRATEGY:
	Figure out what's being adjusted, do it and scroll as necessary.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrMoveCursor	proc	near
		jcxz	ScrMovX
ScrMovY:
		;
		; Adjust the Y coordinate and store the new value.
		;
		add	ax, scrY
		mov	scrY, ax
		;
		; See if need to scroll by comparing with scrBot. If greater
		; than scrBot, then it's outside the scrolling region and
		; must be adjusted.
		;
		cmp	ax, scrBot
		jg	SMCScrollUp
		;
		; See if going off the top. Going off the top DOES NOT result
		; in a reverse scroll -- the cursor just hits the ceiling.
		;
		cmp	ax, scrTop
		jge	ScrMoveCursorRet
		mov	ax, scrTop
		mov	scrY, ax
		jmp	ScrMoveCursorRet

SMCScrollUp:
		call	ScrNukeMouse
		;
		; Calculate number of lines to scroll up (new scrY - scrBot)
		;
		sub	ax, scrBot
		mov	cx, ax
		call	ScrScrollUp

		;
		; Set scrY at bottom of window
		;
		mov	ax, scrBot
		mov	scrY, ax
		jmp	ScrMoveCursorRet
ScrMovX:
		;
		; Adjust X and store new value away.
		;
		add	ax, scrX
		mov	scrX, ax
		;
		; If off to the right, shift the cursor down a line
		;
		cmp	ax, NUMCOLS
		jge	ScrShiftDown
		;
		; If off to the left, shift the cursor up a line
		;
		tst	ax
		jl	ScrShiftUp
ScrMoveCursorRet:
		;
		; Install the new cursor coordinates with the EGA
		;
		mov	ax, scrY
		mov	dx, scrX
		call	ScrSetCursor
		ret
ScrShiftUp:
		;
		; Form correct X coordinate w/o munging any other register,
		; then set AX to be -1 (go back one line) and jump to the
		; code to adjust Y
		;
		add	ax, NUMCOLS
		mov	scrX, ax
		mov	ax, -1
		jmp	ScrMovY
ScrShiftDown:
		;
		; Form correct X position by shifting it over 80 positions,
		; then set AX to be 1 (go forward one line) and jump to the
		; Y adjustment code
		;
		sub	ax, NUMCOLS
		mov	scrX, ax
		mov	ax, 1
		jmp	ScrMovY
ScrMoveCursor	endp
		bugsym	ScrMoveCursor

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteXferComplete
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	write a feil transfer complete message to the screen

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/11/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
FileXferCompleteString	db	"File transfer complete. ",'\r','\n',0

ScrWriteXferComplete	proc	near
	.enter
	mov	bx, offset FileXferCompleteString
	call	ScrWriteString
	.leave
	ret
ScrWriteXferComplete	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteAbortString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	tell the user that pccom has aborted

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
abortString	db	"Transfer aborted",'\r','\n',0

ScrWriteAbortString	proc	near
	.enter
	mov	bx, offset abortString
	call	ScrWriteString
	.leave
	ret
ScrWriteAbortString	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteUserString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	tell the user that pccom is running

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	8/ 3/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
userString	db	"waiting for command from remote PC: ",0

ScrWriteUserString	proc	near
	.enter
	mov	bx, offset userString
	call	ScrWriteString
	.leave
	ret
ScrWriteUserString	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteStringWithEcho
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	

CALLED BY:	GLOBAL

PASS:		BX	= start of null-terminated string

RETURN:		Nothing

DESTROYED:	nothing

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/15/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrWriteStringWithEcho	proc	far
		uses	ax, bx
		.enter
ScrWriteLoop:
		mov	al, [bx]
		inc	bx
		call	ComWrite	; write to com port
		tst	al
		jz	ScrWriteStringRet
		call	ScrWrite
		jmp	ScrWriteLoop
ScrWriteStringRet:
		.leave
		ret
ScrWriteStringWithEcho	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteString
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Write a null-terminated string to the screen

CALLED BY:	ComInit,...

PASS:		BX	= start of null-terminated string

RETURN:		Nothing

DESTROYED:	AX, BX

PSEUDO CODE/STRATEGY:
	Write the characters one at a time through ScrWrite until a null
	byte is found.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	ScrWriteString
ScrWriteString	proc	near
ScrWriteLoop:
		mov	al, [bx]
		inc	bx
		tst	al
		jz	ScrWriteStringRet
		call	ScrWrite
		jmp	ScrWriteLoop

ScrWriteStringRet:
		ret
ScrWriteString	endp
		bugsym	ScrWriteString


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrPrintByte
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Print a byte in HEX on the screen

CALLED BY:	Debugging things

PASS:		AL	= byte to print

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/27/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	ScrPrintByte
nibbles		db	"0123456789ABCDEF"
ScrPrintByte	proc	near
		push	bx
		push	ax
		mov	bx, offset nibbles
		shr	al, 1
		shr	al, 1
		shr	al, 1
		shr	al, 1
		and	al, 0fh
		xlatb
		call	ScrWrite
		pop	ax
		push	ax
		and	al, 0fh
		xlatb
		call	ScrWrite
		pop	ax
		pop	bx
		ret
ScrPrintByte	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteChar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Place the passed character on the screen

CALLED BY:	ScrWrite

PASS:		AL	= character to write
		scrAttr	= attribute for character
		scrX	= X position of character
		scrY	= Y position of character

RETURN:		Nothing

DESTROYED:	AX, BX, CX

PSEUDO CODE/STRATEGY:
	Calculate the correct offset for the character and stuff it.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/9/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
echoBack	db	0

ScrWriteChar	proc	near
		push	es		; Preserve es

		;
		; Figure its offset and store the word there (going through
		; BIOS is too slow for me)
		;
		mov	bx, scrSegment	; Place vid buffer address in es
		mov	es, bx

		mov	bx, scrY	; Figure correct offset

		;
		; Make sure the mouse cursor is out of the way. Only check
		; if we're writing to the same page being displayed.
		;
		mov	ah, scrPage
		cmp	ah, scrDisPage
		jne	SWC1
		mov	cx, scrX
		call	MouseCheckCursor
SWC1:
		Mul80	bx		; offset = 2 * (80 * Y + X)
		add	bx, scrX
		shl	bx, 1
		add	bx, scrStart	; Add in current screen offset

    	    	or  	al, scrHighBit	; Merge in the desired high bit
					; (This is used to get line graphics
					; characters)
		mov	ah, scrAttr	; Attribute goes in high byte
		mov	es:[bx], ax	; Store the character

		pop	es		; Restore es
		cmp	echoBack, 0
		jz	done
		call	ComWrite		
done:
		ret
ScrWriteChar	endp
		bugsym	ScrWriteChar

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDumpHex
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	dump a hex value to the screen using DOS bios int 10h

CALLED BY:	GLOBAL

PASS:		al = byte to dump

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	10/23/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
	public ScrDumpHex
ScrDumpHex	proc	near
		push	bx
		mov	bx, ax
		shr	bl
		shr	bl
		shr	bl
		shr	bl
		call	ScrWriteHexNibble
		mov	bx, ax
		call	ScrWriteHexNibble
		pop	bx		
		ret
ScrDumpHex	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDivide
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	common code to do an unsigned divide of two 32 bit numbers

CALLED BY:	GLOBAL

PASS:		dx:cx = dividend	
		bx:ax = divisor

RETURN:		dx:ax = quotient
		si:bx = remainder

DESTROYED:	Nada.

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	5/11/92		Initial version.
	ardeb	6/15/92		Changed to do everything in registers

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrDivide	proc	near
		uses	cx, di, bp
		.enter
		clrdw	dibp		; clear partial dividend
		mov	si, 32		; bit counter (loop count)

		; loop through all bits, doing that funky divide thing
GUD_loop:
	;	shift another bit from the dividend into the partial

		saldw	dxcx
		rcl	bp
		rcl	di
	;
	; if partial dividend is >= divisor, must do some work.
	; 
		cmp	di, bx		; pdiv.high > divisor.high
		ja	GUD_work	; yes
		jne	GUD_next	; no
		cmp	bp, ax		; is equal; pdiv.low >= divisor.high?
		jb	GUD_next	; no

		; divisor <= partial dividend, do some work
GUD_work:
		inc	cx			; can only be in there once,
						;  and b0 must be 0 (it was
						;  shifted into b0 by the
						;  SALDW up there...)

		subdw	dibp, bxax		; partial dividend -= divisor
GUD_next:
		dec	si
		jg	GUD_loop		; continue with next iteration

		; set up results

		xchg	ax, cx		; dx:ax = quotient
		movdw	sibx, dibp	; si:bx = remainder (partial dividend)

		.leave
		ret			; all done
ScrDivide	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDumpDecimal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	dump a hex value to the screen using DOS bios int 10h

CALLED BY:	GLOBAL

PASS:		dx:ax = dword
		ds:bx = buffer to put charaters
		
RETURN:		

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	10/23/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
ScrDumpDecimal	proc	near
		uses	ax, bx, cx, dx, si, es, di
divisor	local	dword
num_buf	local	fptr	
		.enter
		clr	di			; leading zero flag
		mov	num_buf.high, ds
		mov	num_buf.low, bx
		mov	divisor.high, 3b9ah
		mov	divisor.low, 0ca00h	; divisor = 10^9
		mov	cx, ax		; dx:cx = dividend
divLoop:
		mov	bx, divisor.high
		mov	ax, divisor.low
		call	ScrDivide	; dx:ax = quot, si:bx = remainder
		push	si		
		push	bx
		tst	di
		jz	leadingZero
doDigit:
		; ok, al = digit to display
		add	al, '0'
		stosb
		jmp	loopEnd
leadingZero:
		tst	ax
		jz	loopEnd
		mov	di, num_buf.low
		jmp	doDigit
loopEnd:
		; now divide the divisor by 10
		mov	dx, divisor.high
		mov	cx, divisor.low
		clr	bx
		mov	ax, 10
		call	ScrDivide
		mov	divisor.high, dx
		mov	divisor.low, ax
		pop	cx
		pop	dx	; old remainder becomes dividend dx:cx
		test	ax, 1	; divisors are 10^x so if low bit is lit
				; we must be at 10^0 so we are done
		jz	divLoop
		tst	di
		jnz	lastDigit
		mov	di, num_buf.low
lastDigit:
		mov	al, cl	; last digit
		add	al, '0'
		stosb
		clr	al
		stosb
done:
		.leave
		ret
ScrDumpDecimal	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWriteHexNibble
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	dump a hex value on the screen using DOS int 10h

CALLED BY:	GLOBAL

PASS:		bl (low 4 bits) = value to dump

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	10/23/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrWriteHexNibble	proc	near
		push	ax
		push	bx
		and	bl, 0fh	; get bottom nibble
		cmp	bl, 10
		jge	doHexLetter
		add	bl, '0'
		jmp	printNibble
doHexLetter:
		sub	bl, 10
		add	bl, 'A'
printNibble:
		mov	al,  bl
		call	ScrWrite
		pop	bx
		pop	ax
		ret
ScrWriteHexNibble	endp

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWrite
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Write a character to the scr, handling escape codes.

CALLED BY:	Main, ScrWriteString

PASS:		AL = character to write

RETURN:		Nothing.

DESTROYED:	...

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	ScrWrite
ScrWrite	proc	near

		;call	ScrEnable

		push	bx
		push	cx
		push	dx

		and	al, 7fh		; Strip off parity bit now.
		mov	bl,scrFlags
		tst	bl
		jnz	ScrInSeq	; Already parsing a sequence

		cmp	al, 20h		; Is it a control code?
		jl	ScrCheckCtl	; Go handle it

		call	ScrWriteChar
		mov	cx, SCRADJUSTX	; Shift cursor one place
		mov	ax, 1
		call	ScrMoveCursor
ScrDiscard	label	near		; Place for discarding control
					; characters.
		nop
		bugsym	ScrDiscard
ScrWriteRet:
		pop	dx
		pop	cx
		pop	bx
		ret

ScrInSeq:
		;
		; Already parsing a sequence. Figure out what we want this
		; character for and, if the sequence is finished, Do The Right
		; Thing
		;
		test	bl, SCR1ARG OR SCR2ARG
		jnz	ScrIsArg

		test	bl, SCRGETFIRST
		jz	ScrNotFirst
		mov	byte ptr scrSeq, al
		;
		; Switch to getting the second character and return
		;
		xor	bl, SCRGETFIRST OR SCRGETSECOND
		mov	scrFlags, bl
		jmp	ScrWriteRet
ScrNotFirst:
		test	bl, SCRGETSECOND
		jz	ScrNotSecond
		mov	byte ptr scrSeq + 1, al
		;
		; Figure out what to do. Sets scrFlags appropriately.
		;
		call	ScrParseSeq
		jmp	ScrWriteRet
ScrNotSecond:
		;
		; Should never get here. But if we do, clear out the flags
		;
		clr	bl
		mov	scrFlags, bl
		jmp	ScrDiscard	; Discard character

ScrIsArg:
		;
		; Character is an argument.
		;
		sub	al, ' '		; All arguments are offset by a space
		test	bl, SCRGETFIRST
		jz	ScrNotFirstArg
		mov	scrArg1, al
		test	bl, SCR1ARG
		jnz	ScrGotArgs
		xor	bl, SCRGETFIRST OR SCRGETSECOND
		mov	scrFlags, bl
		jmp	ScrWriteRet
ScrNotFirstArg:
		cbw
		mov	dx, ax		; Shift second argument to dx

ScrGotArgs:
		mov	al, scrArg1	; Load first argument into cx
		cbw			; after sign-extending it.
		mov	cx, ax

		mov	ax,scrHandler	; Load handler's address
		call	ax		; handle that puppy

		clr	bl
		mov	scrFlags, bl	; Reset flags now sequence is parsed

		jmp	ScrWriteRet

ScrCheckCtl:
		lea	bx, scrControls
		xlatb
		cbw
		shl	ax, 1
		mov	bx, ax
		mov	ax, scrCtlHandlers[bx]
		jmp	ax
ScrBackspace	label	near
		;
		; Back up one character
		;
		mov	cx, SCRADJUSTX
		mov	ax, -1
		call	ScrMoveCursor
		jmp	ScrWriteRet

ScrEscape	label	near
		;
		; Note arrival of first character of escape sequence
		;
		or	scrFlags, SCRGETFIRST
		jmp	ScrWriteRet
ScrNewline	label	near
		;
		; Go down one line. Done by restoring the X coordinate
		; to AL and going to the cursor adjustment code used when
		; the output wraps
		;
		mov	cx, SCRADJUSTY
		mov	ax, 1
		call	ScrMoveCursor
		jmp	ScrWriteRet
ScrFormFeed	label	near
		;
		; What to do?
		;
		jmp	ScrDiscard
ScrReturn	label	near
		;
		; Return to start of line. Just clear al and jump back to
		; the cursor code.
		;
		mov	cx, SCRADJUSTX
		mov	ax, scrX
		neg	ax
		call	ScrMoveCursor
		jmp	ScrWriteRet

ScrTab		label	near
		;
		; Go to next tab stop (8 characters apart)
		; Figure out how much to move:
		;	dX = 8 - (X & 7) = -((X & 7) - 8)
		;
		mov	ax, scrX
		and	ax, 7
		sub	ax, 8
		neg	ax
		mov	cx, SCRADJUSTX
		call	ScrMoveCursor
		jmp	ScrWriteRet

ScrBackTab	label	near
		;
		; Back to previous tab stop (8 characters apart)
		;	dX = -(X & 7) if X MOD 8 != 0
		;	dX = -8 if X MOD 8 = 0
		;
		mov	ax, scrX
		and	ax, 7
		jnz	ScrBackTab1
		mov	ax, 8
ScrBackTab1:
		neg	ax
		mov	cx, SCRADJUSTX
		call	ScrMoveCursor
		jmp	ScrWriteRet

ScrBeep		label	near
		call	ScrVisibleBell
		jmp	ScrWriteRet
ScrWrite	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrParseSeq
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Figure out what to do with the current escape sequence

CALLED BY:	ScrWrite (INTERNAL)

PASS:		The two-character sequence in scrSeq

RETURN:		Nothing.

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Use the two characters to search through a string of possible
	sequences.

	When the correct one is found, figure its offset from the start
	of the string and use that to index into a jump table.

	Call the routine to do its thing.

	If not a valid sequence, reset the flags and return.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	To make life easier for the termcap entries that do things multiple
	times, we load cx with 1 before calling the sequence handler. Since
	the first arg is placed in cx before the handler for a sequence-with-
	arguments is called, this allows these multiple things to simply install
	the regular sequence-without-arguments handler after setting the
	SCR1ARG flag.

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
scrSequences	db 	"ALalascdcechclcmcscvDCdcDLdl"
		db	"DOdoleLEndRISPupUP"
		db	"hoICickeksll"
		db	"rcscSFsfSRsrvbXFMAITCCSNIPSHWMSARARS"
		db	"LSCDMDRDRFMVEXCP"

scrNumSeqs	= $-scrSequences
scrSeqJumps	dw	offset ScrAddManyLines
		dw	offset ScrAddLines
		dw	offset ScrAttrSet
		dw	offset ScrClearToEOD
		dw	offset ScrClearToEOL
		dw	offset ScrSetHorizPos
		dw	offset ScrClear
		dw	offset ScrMotion
		dw	offset ScrChangeScroll
		dw	offset ScrSetVertPos
		dw	offset ScrDelManyChars
		dw	offset ScrDelChars
		dw	offset ScrDelManyLines
		dw	offset ScrDelLines
		dw	offset ScrDownManyLines
		dw	offset ScrDownLines
		dw	offset ScrLeftChars
		dw	offset ScrLeftManyChars
		dw	offset ScrRightChars
		dw	offset ScrRightManyChars
		dw	offset ScrSwitchPage
		dw	offset ScrUpLines
		dw	offset ScrUpManyLines
		dw	offset ScrHome
		dw	offset ScrInsManyChars
		dw	offset ScrInsertChars
		dw	offset KbdKeypadEnd
		dw	offset KbdKeypadStart
		dw	offset ScrLowerLeft
		dw	offset ScrRestoreCursor
		dw	offset ScrSaveCursor
		dw	offset ScrScrollManyForw
		dw	offset ScrScrollForw
		dw	offset ScrScrollManyRev
		dw	offset ScrScrollRev
		dw	offset ScrVisibleBell
		dw	offset FileStartXfer
		dw	offset MouseAcknowledge
		dw	offset ScrInvertText
		dw	offset ScrChangeColor
		dw	offset ScrSwitchPageNoDisp
		dw	offset ScrInitPage
		dw	offset ScrSetHigh
		dw	offset ScrWarpMouse
		dw	offset ScrSaveAttr
		dw	offset ScrRestoreAttr
		dw	offset ScrRunSwat
		dw	offset ScrLs
		dw	offset ScrCd
		dw	offset ScrMkDir
		dw	offset ScrRmDir
		dw	offset ScrRmFile
		dw	offset ScrRenameFile
		dw	offset ScrExitToDos
		dw	offset ScrCopyFile

ScrParseSeq	proc	near
		push	di
		push	cx
		push	bx

		mov	scrFlags, 0
		mov	ax, scrSeq	; Fetch sequence into ax
		lea	di, scrSequences
		mov	cx, scrNumSeqs
		cld
		repne scasw
		jne	ScrParseDiscard
		;
		; Find the correct handler function and place its address
		; in ax. Note that di points just beyond the one matched, so
		; we subtract 2 from scrSeqJumps to get the right function
		;
		mov	bx, di
		sub	bx, offset scrSequences
		mov	ax, [scrSeqJumps - 2][bx]
		mov	cx, 1		; Make life easier by setting cx
					; to one.
		call	ax
		jmp	ScrParseRet
ScrParseDiscard	label	near
		bugsym	ScrParseDiscard
		;
		; Sequence was bogus, discard it
		;
ScrParseRet:
		pop	bx
		pop	cx
		pop	di
		ret
ScrParseSeq	endp
		bugsym	ScrParseSeq


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrAddManyLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to parse an add-many-lines sequence.

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Sets the SCRGETFIRST and SCR1ARG flags in scrFlags while turning off
	the SCRGETSECOND flag that is on, then installs ScrAddLines as the
	handler for when the sequence finishes.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrAddManyLines	proc	near
		mov	scrFlags, SCRGETFIRST OR SCR1ARG
		mov	scrHandler, offset ScrAddLines
		ret
ScrAddManyLines	endp
		bugsym	ScrAddManyLines


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrAddLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Insert blank lines before the current position

CALLED BY:	ScrWrite (when add-many-lines sequence done), ScrParseSeq

PASS:		CX 	= number of lines to scroll
		scrY	= current Y position

RETURN:		Nothing.

DESTROYED:	scrFlags set to 0

PSEUDO CODE/STRATEGY:
	Scroll the window from scrY to scrBot down CX lines.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrAddLines	proc	near
		push	ax
		push	bx
		push	dx

		call	ScrNukeMouse
		push	scrTop
		mov	ax, scrY
		mov	scrTop, ax
		call	ScrScrollDown
		pop	ax
		mov	scrTop, ax

		pop	dx
		pop	bx
		pop	ax
		ret
ScrAddLines	endp
		bugsym	ScrAddLines


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrClearToEOD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Clear from the current position to the end of the display

CALLED BY:	ScrParseSeq

PASS:		scrX	= current X
		scrY	= current Y

RETURN:		Nothing

DESTROYED:

PSEUDO CODE/STRATEGY:
	Form byte to store (scrAttr << 8 | ' ') in AX, figure proper address
	in EGA display buffer, number of bytes that need clearing in CX, then
	do a REP STOS

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrClearToEOD	proc	near
		push	di
		push	es
		push	ax
		push	cx

		call	ScrNukeMouse
		mov	ax, scrSegment
		mov	es, ax		; Fetch base into ES for stos

		;
		; Since each line is 80 words long, the offset is
		;	160 * y + 2 * x
		; Leave the offset in both DI and AX
		;
		mov	ax, scrY
		Mul160	ax
		mov	cx, scrX
		shl	cx, 1
		add	cx, ax
		mov	di, cx

		;
		; Must now figure out how many characters to blank. Do this
		; by calculating the offset of the line below the window
		; (160 * scrBot) and subtracting the starting offset from that.
		; Since this value is in bytes, we must shift cx right one to
		; get it in words.
		; 
		mov	ax, scrBot
		Mul160	ax
		sub	ax, cx
		shr	ax, 1
		mov	cx, ax

		;
		; In the EGA, the low byte of the word contains the ASCII
		; code, while the high byte contains the attribute, so
		; form the same thing in AX
		;
		add	di, scrStart	; Add in offset to start of page
		mov	ah, byte ptr scrAttr
		mov	al, ' '

		rep stosw

		pop	cx
		pop	ax
		pop	es
		pop	di
		ret
ScrClearToEOD	endp
		bugsym	ScrClearToEOD


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrClearToEOL
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Clear from the current position to the end of the line

CALLED BY:	ScrParseSeq

PASS:		scrX	= current X
		scrY	= current Y

RETURN:		Nothing

DESTROYED:

PSEUDO CODE/STRATEGY:
	Form byte to store (scrAttr << 8 | ' ') in AX, figure proper address
	in EGA display buffer, number of bytes that need clearing in CX, then
	do a REP STOS

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrClearToEOL	proc	near
		push	di
		push	es
		push	ax
		push	cx

		call	ScrNukeMouse
		mov	ax, scrSegment
		mov	es, ax		; Fetch base into ES for stos

		;
		; Since each line is 80 words long, the offset is
		;	160 * y + 2 * x
		; Leave the offset in both DI and AX
		;
		mov	ax, scrY
		Mul160	ax
		mov	cx, scrX
		shl	cx, 1
		add	ax, cx
		mov	di, ax

		;
		; Must now figure out how many characters to blank. Subtract
		; the current X from 80...
		; 
		mov	cx, NUMCOLS
		sub	cx, scrX

		;
		; In the EGA, the low byte of the word contains the ASCII
		; code, while the high byte contains the attribute, so
		; form the same thing in AX
		;
		add	di, scrStart	; Add offset to page start
		mov	ah, byte ptr scrAttr
		mov	al, ' '

		rep stosw

		pop	cx
		pop	ax
		pop	es
		pop	di
		ret
ScrClearToEOL	endp
		bugsym	ScrClearToEOL


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSetHorizPos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Change the position of the cursor in the current line.

CALLED BY:	ScrParseSeq and ScrWrite

PASS:		CX	= new position, if scrFlags & SCR1ARG

RETURN:		Nothing

DESTROYED:	scrFlags set to 0, if SCR1ARG was set, else they are set
		to SCR1ARG | SCRGETFIRST

PSEUDO CODE/STRATEGY:
	If we're being called to set up for fetching the argument to
	this thing (scrFlags & SCR1ARG == 0), do so.

	Else set the current cursor position.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrSetHorizPos	proc	near
		test	scrFlags, SCR1ARG
		jz	ScrHorizSetup
		cmp	cx, NUMCOLS
		jae	ScrHorizAdjust
ScrHorizSet:
		mov	scrX, cx
		mov	dx, cx
		mov	ax, scrY
		call	ScrSetCursor
		ret
ScrHorizAdjust:
		jl	ScrHorizMakeZero
		mov	cx, NUMCOLS-1
		jmp	ScrHorizSet
ScrHorizMakeZero:
		clr	cx
		jmp	ScrHorizSet

ScrHorizSetup:
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrSetHorizPos
		ret
ScrSetHorizPos	endp
		bugsym	ScrSetHorizPos



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrMotion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up or implement cursor motion

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX 	= new X, if scrFlags & SCR2ARG
		DX	= new Y, if scrFlags & SCR2ARG

RETURN:		Nothing

DESTROYED:	scrFlags set to SCR2ARG | SCRGETFIRST if not already so,
		else set to 0

PSEUDO CODE/STRATEGY:
	If sequence not complete (scrFlags & SCR2ARG == 0), set up to
	parse the rest of the sequence by setting the SCR2ARG and SCRGETFIRST
	flags while clearing the SCRGETSECOND flag. Install ourselves as the
	handler for sequence completion and return.

	If sequence actually complete, make sure the cursor is within range
	of the display (NOT the screen -- cursor escapes beyond the
	scrolling region are allowed), then put the cursor there.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrMotion	proc	near
		test	scrFlags, SCR2ARG
		jz	ScrMotionSetup

		cmp	cx, NUMCOLS
		jae	ScrMotionAdjustX	; NOTE unsigned comparison to
						; catch X < 0, too
ScrMotionSetY:
		cmp	dx, scrLines
		jae	ScrMotionAdjustY
ScrMotionSetXY:
		mov	scrX, cx
		mov	scrY, dx
		mov	ax, dx
		mov	dx, cx
		call	ScrSetCursor
		ret
ScrMotionAdjustX:
		jl	ScrResetX		; Signed branch catches < 0
		mov	cx, NUMCOLS-1		; Set to left edge
		jmp	ScrMotionSetY
ScrResetX:
		clr	cx			; Set to right edge
		jmp	ScrMotionSetY
ScrMotionAdjustY:
		jl	ScrResetY		; Signed branch catches < 0
		mov	dx, scrLines		; Set to bottom edge
		dec	dx
		jmp	ScrMotionSetXY
ScrResetY:
		clr	dx			; Set to top edge
		jmp	ScrMotionSetXY

ScrMotionSetup:
		;
		; Set up to parse rest of sequence by turning SCR2ARG and
		; SCRGETFIRST on, while turning SCRGETSECOND off (it must
		; have been on when we were called).
		;
		mov	scrFlags, SCR2ARG OR SCRGETFIRST 
		mov	scrHandler, offset ScrMotion
		ret
ScrMotion	endp
		bugsym	ScrMotion


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrChangeScroll
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up or implement cursor motion

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX 	= new X, if scrFlags & SCR2ARG
		DX	= new Y, if scrFlags & SCR2ARG

RETURN:		Nothing

DESTROYED:	scrFlags set to SCR2ARG | SCRGETFIRST if not already so,
		else set to 0

PSEUDO CODE/STRATEGY:
	If sequence not complete (scrFlags & SCR2ARG == 0), set up to
	parse the rest of the sequence by setting the SCR2ARG and SCRGETFIRST
	flags while clearing the SCRGETSECOND flag. Install ourselves as the
	handler for sequence completion and return.

	If sequence actually complete, make sure the cursor is within range
	of the display (NOT the screen -- cursor escapes beyond the
	scrolling region are allowed), then put the cursor there.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrChangeScroll	proc	near
		test	scrFlags, SCR2ARG
		jz	ScrCSSetup

		cmp	cx, scrLines
		jae	ScrCSAdjustTop		; NOTE unsigned comparison to
						; catch top < 0, too
ScrCSSetBot:
		cmp	dx, scrLines
		jae	ScrCSAdjustBot
ScrCSSet:
		cmp	cx, dx
		jge	ScrCSRet
		mov	scrTop, cx
		mov	scrBot, dx
ScrCSRet:
		ret
ScrCSAdjustTop:
		jl	ScrResetTop		; Signed branch catches < 0
		mov	cx, scrLines		; Set to bottom edge
		dec	cx
		jmp	ScrCSSetBot
ScrResetTop:
		clr	cx			; Set to top edge
		jmp	ScrCSSetBot
ScrCSAdjustBot:
		jl	ScrCSResetBot		; Signed branch catches < 0
		mov	dx, scrLines		; Set to bottom edge
		dec	dx
		jmp	ScrCSSet
ScrCSResetBot:
		clr	dx			; Set to top edge
		jmp	ScrCSSet

ScrCSSetup:
		;
		; Set up to parse rest of sequence by turning SCR2ARG and
		; SCRGETFIRST on, while turning SCRGETSECOND off (it must
		; have been on when we were called).
		;
		mov	scrFlags, SCR2ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrChangeScroll
		ret
ScrChangeScroll	endp
		bugsym	ScrChangeScroll


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSetVertPos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Change the row of the cursor.

CALLED BY:	ScrParseSeq and ScrWrite

PASS:		CX	= new position, if scrFlags & SCR1ARG

RETURN:		Nothing

DESTROYED:	scrFlags set to 0, if SCR1ARG was set, else they are set
		to SCR1ARG | SCRGETFIRST

PSEUDO CODE/STRATEGY:
	If we're being called to set up for fetching the argument to
	this thing (scrFlags & SCR1ARG == 0), do so.

	Else set the current cursor position.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrSetVertPos	proc	near
		test	scrFlags, SCR1ARG
		jz	ScrVertSetup
		cmp	cx, scrLines
		jae	ScrVertAdjust
ScrVertSet:
		mov	scrY, cx
		mov	ax, cx
		mov	dx, scrX
		call	ScrSetCursor
		ret
ScrVertAdjust:
		jl	ScrVertMakeZero
		mov	cx, scrLines
		dec	cx
		jmp	ScrVertSet
ScrVertMakeZero:
		clr	cx
		jmp	ScrVertSet

ScrVertSetup:
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrSetVertPos
		ret
ScrSetVertPos	endp
		bugsym	ScrSetVertPos



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDelManyChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to delete many characters

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Set the SCR1ARG and SCRGETFIRST flags in scrFlags, while resetting
	the SCRGETSECOND flag, then install ScrDelChars in the scrHandler
	vector.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrDelManyChars	proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrDelChars
		ret
ScrDelManyChars	endp
		bugsym	ScrDelManyChars


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDelChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Delete character(s) at the current cursor position.

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX	= number of characters to delete

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Copy characters to right down and fill in the gaps with spaces in
	the current attribute.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrDelChars	proc	near
		push	es
		push	ds
		push	si
		push	di
		push	ax
		push	bx

		call	ScrNukeMouse
		mov	ax, scrSegment
		mov	es, ax
		mov	ds, ax

		;
		; Figure starting address from
		;	offset = (160 * y) + (x * 2)
		; then multiply the count of characters to delete by two to
		; get the offset from that to the first character to be moved
		; down.
		;
		; Ends up with
		;	di = offset of first character to be deleted
		;	si = offset of first character to be preserved
		;
		mov	bx, cs:scrY
		Mul160	bx
		mov	ax, cs:scrX
		shl	ax, 1
		add	ax, bx
		mov	di, ax

		mov	bx, cx
		shl	bx, 1
		add	ax, bx
		mov	si, ax
		add	di, cs:scrStart
		add	si, cs:scrStart
		mov	ax, cs:scrX

		add	ax, cx
		sub	ax, NUMCOLS
		jge	ScrBlankRest	; Trying to delete beyond end of
					; line -- just blank the rest of it

		;
		; Figure the count of characters to copy down.
		;
		neg	ax
		push	cx
		mov	cx, ax

		cld			; copy forward

		rep movsw		; Copy the characters down

		;
		; Form space with current attribute in AX and fetch the count
		; off the stack, then store the space in the buffer. Note that
		; DI points to the first character to blank.
		;
ScrBlankChars:
		mov	ah, byte ptr cs:scrAttr
		mov	al, ' '

		pop	cx
		rep stosw

ScrDelCharsRet:
		pop	bx
		pop	ax
		pop	di
		pop	si
		pop	ds
		pop	es
		ret
ScrBlankRest:
		; Adjust CX to be the number of characters left on the
		; line and push it on the stack, then go back to ScrBlankChars
		; to actually wipe out those characters.
		mov	cx, NUMCOLS
		sub	cx, cs:scrX
		push	cx
		jmp	ScrBlankChars
ScrDelChars	endp
		bugsym	ScrDelChars


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDelManyLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to parse a sequence to delete many lines

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG | SCRGETFIRST

PSEUDO CODE/STRATEGY:
	Set scrFlags and install ScrDelLines as the handler

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrDelManyLines	proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrDelLines
		ret
ScrDelManyLines	endp
		bugsym	ScrDelManyLines


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDelLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Delete lines on the screen.

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX	= number of lines to delete
		scrY	= first line to nuke
		scrBot	= bottom of region

RETURN:		Nothing

DESTROYED:	?

PSEUDO CODE/STRATEGY:
	Scroll the window from the current line to the bottom up CX lines.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrDelLines	proc	near
		push	ax
		push	bx
		push	dx

		call	ScrNukeMouse
		push	scrTop
		mov	ax, scrY
		mov	scrTop, ax
		call	ScrScrollUp
		pop	ax
		mov	scrTop, ax

		pop	dx
		pop	bx
		pop	ax
		ret
ScrDelLines	endp
		bugsym	ScrDelLines


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDownManyLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to parse the rest of a down-many-lines sequence

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG | SCRGETFIRST

PSEUDO CODE/STRATEGY:
	Install things as appropriate

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrDownManyLines proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrDownLines
		ret
ScrDownManyLines endp
		bugsym	ScrDownManyLines

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrDownLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor down.

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX 	= number of lines to go down

RETURN:		Nothing

DESTROYED:	AX, CX

PSEUDO CODE/STRATEGY:
	Shift CX to AX, load CX with SCRADJUSTY and let ScrMoveCursor handle
	it.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrDownLines	proc	near
		mov	ax, cx
		mov	cx, SCRADJUSTY
		call	ScrMoveCursor
		ret
ScrDownLines	endp
		bugsym	ScrDownLines

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrHome
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor to its home position.

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX, BX, DX

PSEUDO CODE/STRATEGY:
	Move the cursor...

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrHome		proc	near
		mov	ax, scrTop
		clr	dx
		mov	scrY, ax
		mov	scrX, dx
		call	ScrSetCursor
		ret
ScrHome		endp
		bugsym	ScrHome


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInsManyChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to delete many characters

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Set the SCR1ARG and SCRGETFIRST flags in scrFlags, while resetting
	the SCRGETSECOND flag, then install ScrInsertChars in the scrHandler
	vector.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrInsManyChars	proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrInsertChars
		ret
ScrInsManyChars	endp
		bugsym	ScrInsManyChars


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInsertChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Insert a series of spaces at the current position.

CALLED BY:	ScrParseSeq

PASS:		CX	= number of characters to insert

RETURN:		

DESTROYED:

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrInsertChars	proc	near
		push	es		; Preserve registers
		push	ds
		push	si
		push	di

		call	ScrNukeMouse
		;
		; Find the last word of the current line and place it in
		; di.
		;
		mov	ax, scrY
		inc	ax
		Mul160	ax
		sub	ax, 2
		mov	di, ax
		add	di, scrStart

		;
		; Find the first character to move by subtracting the number
		; of characters to insert from 80 and multiplying that by
		; two (since each character is a word).
		;
		mov	bx, cx
		shl	bx, 1
		sub	ax, bx
		mov	si, ax
		add	si, scrStart

		push	cx		; Save count for later
		;
		; Figure out how many characters to shift. This is the diff
		; between the first character being moved (80 - cx) and the
		; current X position (scrX).
		;
		sub	cx, NUMCOLS
		neg	cx
		mov	ax, scrX
		sub	cx, ax
		jl	ScrInsBlankRest	; If moving negative characters,
					; just blank the rest of the line.

		std			; Move it backwards
		mov	ax, scrSegment
		mov	es, ax
		mov	ds, ax
		rep movsw

		;
		; Restore the count as the number of characters to blank out,
		; store the blank in AX and shove the bytes in.
		;
ScrInsBlankChars:
		pop	cx
		mov	ah, byte ptr cs:scrAttr
		mov	al, ' '
		rep stosw

ScrInsRet:
		pop	di
		pop	si
		pop	ds
		pop	es
		ret
ScrInsBlankRest:
		;
		; Discard CX already on stack then call the clear-to-eol
		; function to clear the rest of the line -- things are too
		; complex to be worth doing anything else.
		;
		pop	cx
		call	ScrClearToEOL
		jmp	ScrInsRet
ScrInsertChars	endp
		bugsym	ScrInsertChars


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrLeftManyChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to move left more than one character

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG OR SCRGETFIRST
		scrHandler = ScrLeftChars

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrLeftManyChars proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrLeftChars
		ret
ScrLeftManyChars endp
		bugsym	ScrLeftManyChars

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrLeftChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor left some.

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX	= number of characters to move

RETURN:		Nothing

DESTROYED:	CX, AX

PSEUDO CODE/STRATEGY:
	Call ScrMoveCursor to deal with it

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrLeftChars	proc	near
		mov	ax, cx
		neg	ax
		mov	cx,SCRADJUSTX
		call	ScrMoveCursor
		ret
ScrLeftChars	endp
		bugsym	ScrLeftChars

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrLowerLeft
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor to the lower-left corner of the screen

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX, BX, DX, scrX and scrY modified

PSEUDO CODE/STRATEGY:
	Move scrBot and 0 to scrY and scrX, respectively, and call the EGA
	to shift the cursor.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrLowerLeft	proc	near
		mov	ax, scrBot
		mov	scrY, ax
		clr	dx
		mov	scrX, dx
		call	ScrSetCursor
		ret
ScrLowerLeft	endp
		bugsym	ScrLowerLeft


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRightChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor to the right a given number of characters

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX	= number of characters to move

RETURN:		Nothing

DESTROYED:	AX

PSEUDO CODE/STRATEGY:
	Make ScrMoveCursor do all the work

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrRightChars	proc	near
		mov	ax, cx
		mov	cx, SCRADJUSTX
		call	ScrMoveCursor
		ret
ScrRightChars	endp
		bugsym	ScrRightChars

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRestoreCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Restore the cursor to the position saved by ScrSaveCursor

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	DX, BX, AX

PSEUDO CODE/STRATEGY:
	Copy the scrSaved[XY] variables to the scr[XY] variables

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrRestoreCursor proc	near
		mov	ax, scrSavedY
		mov	scrY, ax
		mov	dx, scrSavedX
		mov	scrX, dx
		call	ScrSetCursor
		ret
ScrRestoreCursor endp
		bugsym	ScrRestoreCursor


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRightManyChars
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to move right a given number of characters

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG | SCRGETFIRST

PSEUDO CODE/STRATEGY:
	Change the flags and install the handler

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrRightManyChars proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrRightChars
		ret
ScrRightManyChars endp
		bugsym	ScrRightManyChars

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSaveCursor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Save the current cursor position.

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	AX

PSEUDO CODE/STRATEGY:
	Copy scr[XY] to scrSaved[XY]

KNOWN BUGS/SIDE EFFECTS/IDEAS:
	Might be good to have a flag saying this was done so ScrRestoreCursor
	won't restore a bogus position...

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrSaveCursor	proc	near
		mov	ax, scrX
		mov	scrSavedX, ax
		mov	ax, scrY
		mov	scrSavedY, ax
		ret
ScrSaveCursor	endp
		bugsym	ScrSaveCursor


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrScrollManyForw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to scroll forward more than one line

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG | SCRGETFIRST
		scrHandler = ScrScrollForw

PSEUDO CODE/STRATEGY:
	Install flags and handler.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrScrollManyForw proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrScrollForw
		ret
ScrScrollManyForw endp
		bugsym	ScrScrollManyForw


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrScrollForw
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Scroll the current window a given number of lines up

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX 	= number of lines to scroll

RETURN:		Nothing

DESTROYED:	AX, BX, CX, DX

PSEUDO CODE/STRATEGY:
	Set up the window parameters and call the EGA to scroll the thing

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrScrollForw	proc	near
		call	ScrNukeMouse
		call	ScrScrollUp
		ret
ScrScrollForw	endp
		bugsym	ScrScrollForw


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrScrollManyRev
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to scroll backward more than one line

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG | SCRGETFIRST
		scrHandler = ScrScrollRev

PSEUDO CODE/STRATEGY:
	Install flags and handler.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrScrollManyRev proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrScrollRev
		ret
ScrScrollManyRev endp
		bugsym	ScrScrollManyRev


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrScrollRev
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Scroll the current window a given number of lines down

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX	= number of lines to scroll

RETURN:		Nothing

DESTROYED:	AX, BX, CX, DX

PSEUDO CODE/STRATEGY:
	Set up window parameters and trap to EGA to do it.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrScrollRev	proc	near
		call	ScrNukeMouse
		call	ScrScrollDown
		ret
ScrScrollRev	endp
		bugsym	ScrScrollRev


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrUpManyLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set up to move the cursor up more than one line

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	scrFlags = SCR1ARG | SCRGETFIRST

PSEUDO CODE/STRATEGY:
	install handler and set flags

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrUpManyLines	proc	near
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrUpLines
		ret
ScrUpManyLines	endp
		bugsym	ScrUpManyLines

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrUpLines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Move the cursor up a given number of lines.

CALLED BY:	ScrParseSeq, ScrWrite

PASS:		CX	= number of lines to go up

RETURN:		Nothing

DESTROYED:	AX, CX

PSEUDO CODE/STRATEGY:
	Call ScrMoveCursor to do all the work.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrUpLines	proc	near
		mov	ax, cx
		neg	ax
		mov	cx, SCRADJUSTY
		call	ScrMoveCursor
		ret
ScrUpLines	endp
		bugsym	ScrUpLines

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrVisibleBell
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Inverts the screen for a moment

CALLED BY:	ScrParseSeq

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Go through all characters on the screen and invert their attribute
	bytes.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/8/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrVisibleBell	proc	near
		;
		; Init attribute chip flip-flop
		;
		mov	dx, EGAATTRINIT
		in	al, dx

		;
		; Change overscan register while leaving the palette enabled
		;
		mov	dx, EGAATTRPORT
		mov	al, 31h		; Overscan register w/palette on
		out	dx, al
		mov	al, scrVb
		out	dx, al

		;
		; Pause for the flash to be seen
		;
		mov	cx, 0
SVB1:		loop	SVB1

		;
		; Restore normal blank border
		;
		mov	al, 31h
		out	dx, al
		mov	al, 0
		out	dx, al
		ret
ScrVisibleBell	endp
		bugsym	ScrVisibleBell


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrClear
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Clear the screen with the current attribute

CALLED BY:	ScrInit, ScrWrite

PASS:		Nothing in registers.
		scrAttr		Attribute to use
		scrTop		Top of region to clear
		scrBot		Bottom of region to clear

RETURN:		Nothing

DESTROYED:	AX, BX, CX, DX

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	6/7/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrClear	proc	near
		;
		; Turn off the mouse
		; 
		call	ScrNukeMouse
		
		;
		; Load ax with a space of the current attribute
		; 
		mov	ah, scrAttr
		mov	al, ' '
		;
		; Figure the number of lines to clear in cx.
		; 
		mov	cx, scrBot
		inc	cx
		mov	bx, scrTop
		sub	cx, bx
		;
		; Multiply by 80 to give the number of words to clear
		; 
		Mul80	cx, dx

		;
		; Multiply the top line # by 160 to get the start offset in
		; the page. Then add the offset to the start of the whole page
		; and shove the result in di for STOSW to use.
		;
		Mul160	bx
		add	bx, scrStart
		mov	di, bx

		;
		; Get the segment of the frame buffer in es
		; 
		push	es
		mov	dx, scrSegment
		mov	es, dx
		rep	stosw
		pop	es
		;
		; Move the cursor to the upper left corner
		; 
		call	ScrHome
		ret
ScrClear	endp
		bugsym	ScrClear



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInvertText
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Invert text in current row between columns

CALLED BY:	ScrParseSeq

PASS:		CX	= First column to invert
		DX	= Last column to invert

RETURN:
		Nothing

DESTROYED:
		AX, BX, CX, DX

PSEUDO CODE/STRATEGY:
	Fetch each attribute from the columns and nibble swap it by rotating
	it four bits left.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	7/29/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrInvertText	proc	near
		test	scrFlags, SCR2ARG
		jz	SITSetHandler
		push	es

		call	ScrNukeMouse
		mov	ax, scrSegment	; Point ES at EGA buffer
		mov	es, ax

		mov	ax, scrY	; Form address of Y * startX
		Mul80	ax, bx
		add	ax, cx
		sal	ax, 1		; Each char is a word, so *= 2
		mov	bx, ax		; Need it in bx for dereference
		add	bx, scrStart	; Offset to page start
		sub	dx, cx		; Form # chars in dx
		mov	cl, 4		; nibble swap requires 4
		inc	bx		; mangling attributes
SIT2:
		mov	al, es:[bx]	; fetch next attr
		rol	al, cl		; nibble swap
		mov	es:[bx],al	; store inverse
		inc	bx		; advance to next
		inc	bx
		dec	dx		; one down, how many to go?
		jns	SIT2		; Inversion is inclusive, so go one
					; more beyond column diff.

		pop	es
		ret

SITSetHandler:
		mov	scrFlags, SCR2ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrInvertText
		ret
ScrInvertText	endp
		bugsym	ScrInvertText


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrChangeColor
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Change the definition of a color

CALLED BY:	ScrWrite and ScrParseSeq

PASS:		CL	= color # to change
		DL	= new value

RETURN:		Nothing

DESTROYED:	AX,DX

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	7/29/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrChangeColor	proc	near
		test	scrFlags, SCR2ARG
		jz	SCC1

		mov	ch, dl		; Put color value into ch 

		;
		; Init attribute chip flip-flop
		;
		mov	dx, EGAATTRINIT
		in	al, dx

		mov	dx, EGAATTRPORT
		mov	ax, cx
		and	al, 0fh		; Make sure it's a color register
		out	dx, al		; Set address
		xchg	ah, al		; get value
		out	dx, al		; Store it

		mov	al, 20h		; Re-enable palette
		out	dx, al
		ret
SCC1:
		mov	scrFlags, SCR2ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrChangeColor
		ret
ScrChangeColor	endp
		bugsym	ScrChangeColor



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrAttrSet
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the current screen attribute

CALLED BY:	ScrParseSeq

PASS:		CX	= fg color
		DX	= bg color

RETURN:		Nothing

DESTROYED:	CX, DX

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	8/23/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrAttrSet	proc	near
		test	scrFlags, SCR2ARG
		jnz	SAS1
		mov	scrFlags, SCR2ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrAttrSet
		ret
SAS1:
		and	cx, 0fh		; Color is in low nibble
		sal	dx, 1		; Need background in high nibble
		sal	dx, 1
		sal	dx, 1
		sal	dx, 1
		or	cx, dx		; merge to form attribute
		mov	scrAttr, cl	; Store new attribute
		ret
ScrAttrSet	endp
		bugsym	ScrAttrSet


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSwitchPage
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Start displaying a different page

CALLED BY:	ScrParseSeq, ScrInitPage

PASS:		CX	= page to display

RETURN:		Nothing

DESTROYED:	AX, CX

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	8/19/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
ScrSwitchPage	proc	near
		test	scrFlags, SCR1ARG
		jz	SSP99

		;
		; Make ScrSwitchPageNoDisp do most of the work
		;
		call	ScrSwitchPageNoDisp

		;
		; Tell the BIOS to switch pages
		;
		mov	al, scrPage
		mov	scrDisPage, al	; Set the page being displayed, too.
		vidcall	5

		;
		; Fetch the actual start address from where the BIOS stored
		; it.
		;
		push	es

		clr	ax
		mov	es, ax
		mov	ax, es:VIDSTARTADDR
		mov	scrStart, ax

		pop	es

		;
		; Set the correct cursor by moving it 0 pixels horizontally
		;
		mov	cx, SCRADJUSTX
		clr	ax
		call	ScrMoveCursor

		ret
SSP99:
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrSwitchPage
ScrSwitchPage	endp
		bugsym	ScrSwitchPage

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSwitchPageNoDisp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Change state to be displaying a different screen page

CALLED BY:	ScrSwitchPage, ScrParseSeq

PASS:		CX	= page to display

RETURN:		Nothing

DESTROYED:	scrPage... set up for new page.

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	8/19/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrSwitchPageNoDisp	proc	near
		test	scrFlags, SCR1ARG
		jnz	SSPND1
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrSwitchPageNoDisp
		ret
SSPND1:
		mov	al, scrPage	; Fetch current page
		cbw			; -> word
		mov	bx, ax		; -> bx for indexing
		;
		; Save current state.
		;
		mov	al, scrAttr
		mov	scrAttrSave[bx], al
		mov	al, scrHighBit
		mov	scrHighBitSave[bx], al
		sal	bx, 1		; *2 to index words
		mov	ax, scrX
		mov	scrXSave[bx],ax
		mov	ax, scrY
		mov	scrYSave[bx],ax
		mov	ax, scrSavedX
		mov	scrSavedXSave[bx],ax
		mov	ax, scrSavedY
		mov	scrSavedYSave[bx],ax
		mov	ax, scrTop
		mov	scrTopSave[bx], ax
		mov	ax, scrBot
		mov	scrBotSave[bx], ax
		mov	ax, scrStart
		mov	scrStartSave[bx], ax

		mov	scrPage, cl
		mov	bx, cx		; Copy for indexing

		;
		; Recover page's state
		;
		mov	al, scrAttrSave[bx]
		mov	scrAttr, al
    	    	mov 	al, scrHighBitSave[bx]
		mov	scrHighBit, al
		sal	bx, 1
		mov	ax, scrXSave[bx]
		mov	scrX, ax
		mov	ax, scrYSave[bx]
		mov	scrY, ax
		mov	ax, scrSavedXSave[bx]
		mov	scrSavedX, ax
		mov	ax, scrSavedYSave[bx]
		mov	scrSavedY, ax
		mov	ax, scrTopSave[bx]
		mov	scrTop, ax
		mov	ax, scrBotSave[bx]
		mov	scrBot, ax
		mov	ax, scrStartSave[bx]
		mov	scrStart, ax

		ret
ScrSwitchPageNoDisp	endp
		bugsym	ScrSwitchPageNoDisp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrInitPage
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Initialize a new page

CALLED BY:	ScrParseSeq

PASS:		CX	= page to initialize and display

RETURN:		Nothing

DESTROYED:	AX, BX, CX, DX

PSEUDO CODE/STRATEGY:
	Call ScrSwitchPage to change to the new page, then call
	ScrClearScreen to clear the new page after installing initial
	values for scrAttr, scrTop and scrBot.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	8/19/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrInitPage	proc	near
		test	scrFlags, SCR1ARG
		jz	SIP99
		;
		; Switch to the new page
		;
		call	ScrSwitchPage

		;
		; Initialize the various parameters
		;
		mov	al,scrNormal
		mov	scrAttr,al
		mov	scrTop, 0
		mov	ax,scrLines
		dec	ax
		mov	scrBot, ax
		;
		; Clear the screen and home the cursor
		;
		call	ScrClear

		ret
SIP99:
		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrInitPage
		ret

ScrInitPage	endp
		bugsym	ScrInitPage



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrTick
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Handle a tick of the system clock

CALLED BY:	MS-DOS

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	9/27/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrTick		proc	far
		;
		; See if the screen is already off. If so, do nothing
		;
		test	cs:scrEtcFlags, SCR_ISSAVED
		jnz	ScrTick2
		;
		; Count down one tick. If not at zero, the screen may stay
		; on.
		;
		dec	cs:scrTime
		jnz	ScrTick2

		;
		; Time has run out -- turn off the screen. We need to turn
		; the mouse off for the duration to avoid a loop in the
		; mouse driver (with interrupts off) while it waits for
		; display memory to become unavailable [why it does that
		; is anyone's guess]. The loop is entered if the mouse needs
		; to be removed while the screen is disabled.
		;
		push 	ax
		push 	dx

		push	ds	; Save DS
		push	cs	; load DS with CS for MouseRemoveCursor's
		pop	ds	; benefit.
		call	MouseRemoveCursor

		test	scrType, TYPE_HGC
		jnz	ScrTickDisHGC
		mov	dx, scrAddrPort
		mov	al, CRTCMODE
		out	dx, al
		inc	dx
		mov	al, BLANKMODE
		out	dx, al
ScrTick1_5:
		or	scrEtcFlags, SCR_ISSAVED
		pop	ds

		pop	dx
		pop	ax
ScrTick2:
		jmp	cs:scrTimerVec

ScrTickDisHGC:
		mov	dx, 3b8h
		clr	al		; No video, no block, non-graphic
		out	dx, al
		jmp	short ScrTick1_5

ScrTick		endp
		bugsym	ScrTick



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrEnable
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Make sure the screen is enabled

CALLED BY:	MouseEvent, ScrWrite, KbdInterrupt

PASS:		Nothing

RETURN:		Nothing

DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
	Turn off interrupts (so the screen doesn't get turned off in the
	middle of things).

	If the screen is off, re-enable it and reset the SCR_ISSAVED flag.

	In any case, reset the timeout to the full amount.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	9/27/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		public	ScrEnable
ScrEnable	proc	near
		pushf
		dsi	
		test	scrEtcFlags, SCR_ISSAVED
		jz	ScrEnable2

		push	ax
		push	dx

		test	scrType, TYPE_HGC
		jnz	ScrEnableHGC
		mov	dx, scrAddrPort
		mov	al, CRTCMODE
		out	dx, al
		inc	dx
		mov	al, REGULARMODE
		out	dx, al
ScrEnable1_5:
		and	scrEtcFlags, NOT SCR_ISSAVED

		pop	dx
		pop	ax
ScrEnable2:
		mov	scrTime, SCR_TIME
		popf
		ret
ScrEnableHGC:
		mov	dx, 3b8h
		mov	al, 8		; No blink, video on, a/n
		out	dx, al
		jmp	short ScrEnable1_5
ScrEnable	endp



COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrToggleSave
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Toggle the state of the screen saver.

CALLED BY:	KbdHandleSys when SysReq-s seen

PASS:	    	Nothing

RETURN:	    	Nothing

DESTROYED:  	AX, BX, DX

PSEUDO CODE/STRATEGY:
       See if the screen-saver is enabled.
       If so, disable it, else enable it.

KNOWN BUGS/SIDE EFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	adam	9/27/88		Initial Revision

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}

ScrToggleSave	proc	near
		test	scrDoSave, 1
		jnz	STSOff

		mov	scrTime, SCR_TIME

		mov	ax, TIMER_VEC
		mov	bx, offset scrTimerVec
		mov	dx, offset ScrTick
		call	SetInterrupt

		inc	scrDoSave

		jmp	short STSExit

STSOff		label	near
		dsi

		mov	ax, TIMER_VEC
		mov	bx, offset scrTimerVec
		call	ResetInterrupt

		dec	scrDoSave
		call	ScrEnable

		eni
STSExit:
		;mov	al, scrDoSave
		;call	ScrPrintByte

		ret
ScrToggleSave	endp
		bugsym	ScrToggleSave




COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSaverOff
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Turn the screen saver off unconditionally

CALLED BY:	RunSwat in main.asm
PASS:		Nothing
RETURN:		Nothing
DESTROYED:	?

PSEUDO CODE/STRATEGY:
		If the screen saver is off, jump to STSOff to turn it off.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	1/30/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrSaverOff
ScrSaverOff	proc	near
		test	scrDoSave, 1
		jnz	STSOff
		ret
ScrSaverOff	endp

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSetHigh
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Set the state of succeeding characters's high bit

CALLED BY:	ScrParseSeq
PASS:		CX  	= new high bit in b0
RETURN:		Nothing
DESTROYED:	CX

PSEUDO CODE/STRATEGY:
	Rotate CL right once to put the bit into b7, the store the result in
	scrHighBit

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/27/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrSetHigh
ScrSetHigh	proc	near
		test	scrFlags, SCR1ARG
		jz	SSH1

    	    	ror 	cl, 1
		mov 	scrHighBit, cl
		ret
SSH1:		mov	scrFlags, SCR1ARG OR SCRGETFIRST
		mov	scrHandler, offset ScrSetHigh
		ret
ScrSetHigh	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWarpMouse
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Front for the MouseWarp function to gather the two args

CALLED BY:	ScrParseSeq
PASS:		Nothing
RETURN:		Nothing
DESTROYED:	Nothing

PSEUDO CODE/STRATEGY:
       Simply sets the state machine to read two args and pass them to
       MouseWarp.
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	9/30/88		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrWarpMouse
ScrWarpMouse	proc	near
		mov	scrFlags, SCR2ARG OR SCRGETFIRST
		mov	scrHandler, offset MouseWarp
		ret
ScrWarpMouse	endp

scrSavedAttr	db	?	; Attribute saved by ScrSaveAttr
		even

COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrSaveAttr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Save the current attribute byte.

CALLED BY:	ScrWrite
PASS:		Nothing
RETURN:		Nothing
DESTROYED:	AL

PSEUDO CODE/STRATEGY:
		

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 4/88	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrSaveAttr
ScrSaveAttr	proc	near
		mov	al, scrAttr
		mov	scrSavedAttr, al
		ret
ScrSaveAttr	endp


COMMENT }%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRestoreAttr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Restore the previous attribute byte.

CALLED BY:	ScrWrite
PASS:		Nothing
RETURN:		Nothing
DESTROYED:	AL

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	10/ 4/88	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%}
		bugsym	ScrRestoreAttr
ScrRestoreAttr	proc	near
		mov	al, scrSavedAttr
		mov	scrAttr, al
		ret
ScrRestoreAttr	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRunSwat
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Start Swat running.

CALLED BY:	\eRS<argtype>
PASS:		CX	= type of args to provide swat:
				0	= run ec version
				1	= load ec version, but don't let it run
				2	= run non-ec version
				3	= load non-ec version, no run

RETURN:		nothing
DESTROYED:	CX

PSEUDO CODE/STRATEGY:
		Set the SF_SWAT flag in sysFlags so when we get back to
		Main, we'll start Swat running.

		Then figure out which additional arguments we're to pass.

KNOWN BUGS/SIDE EFFECTS/IDEAS:
		

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	ardeb	11/10/89	Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrRunSwat
ScrRunSwat	proc	near
		test	scrFlags, SCR1ARG
		jz	SRS2
	;
	; Always set the run-swat flag
	;
		or	ds:sysFlags, mask SF_SWAT
	;
	; Point swatArgs at the proper extra argument string
	;
		shl	cx
		cmp	cx, SwatArgs
		jb	ok
		mov	cx, SA_STOP_EC		; default to ec/s
ok:
		mov	bx, ds:[swatArgPtrs]
		add	bx, cx
		mov	bx, ds:[bx]
		mov	ds:[swatArgs], bx
		ret
SRS2:
		mov	scrFlags, SCR1ARG or SCRGETFIRST
		mov	scrHandler, offset ScrRunSwat
		ret
ScrRunSwat	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrLs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	to an LS (dos' DIR) of the current directory

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	11/ 4/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ExtFileControlBlock	struct
	EFCB_extFlag		byte
	EFCB_reserved1		byte 5 dup(?)
	EFCB_attributes		byte
	EFCB_drive		byte
	EFCB_filename		byte 8 dup(?)
	EFCB_extension		byte 3 dup(?)
	EFCB_curBlockNum	word
	EFCB_recordSize		word
	EFCB_fileSize		dword
	EFCB_date		word
	EFCB_time		word
	EFCB_reserved2		byte 8 dup (?)
	EFCB_curRecordNum 	byte
	EFCB_randomRecordNum 	byte 4 dup (?)
ExtFileControlBlock	end

FileAttributes		record
	RESERVED:2
	FA_ARCHIVE:1
	FA_DIRECTORY:1
	FA_VOLUME:1
	FA_SYSTEM:1
	FA_HIDDEN:1
	FA_READ_ONLY:1
FileAttributes		end

DateRecord	record
	DR_YEAR:7
	DR_MONTH:4
	DR_DAY:5
DateRecord	end

TimeRecord	record
	TR_HOURS:5
	TR_MINUTES:6
	TR_SECONDS:5
TimeRecord	end

FindFirstNext		struct
	FFN_reserved	byte 21 dup(?)
	FFN_attributes	FileAttributes
	FFN_time	word
	FFN_date	DateRecord
	FFN_fileSize	dword
	FFN_filename	byte 13 dup(?)
FindFirstNext		end

ffn	FindFirstNext	<>

fileSpec 	db	'*.*', 0
dirString 	db	'<DIR> ', 0
spaceString	db	'    ', 0

if 0	
fcb	ExtFileControlBlock <  0ffh, 		; ext flag must be ff
			       <0,0,0,0,0>,	; reserved must be 0
			       0ffh,		; all attributes on
			       0,		; drive 0 = current drive
			       <?,?,?,?,?,?,?,?>, ; wildcards for filename
			       <?,?,?>,		; wildcards for extension
			       0,
			       0,
			       <0,0>,
			       0,
			       0,
			       <0,0,0,0,0,0,0,0>,
			       0,
			       <0,0,0,0>
			    >

fcb	db	0ffh
	db	5 dup (0)
	db	0ffh
	db	0
	db	'????????'
	db	'???'
	db	25 dup(0)
fcb_buf ExtFileControlBlock <>
endif

	public	ScrLs
ScrLs	proc	near
	uses	ax, dx, cx, di, si, ds, es
	.enter
	
	; this will cause all output to get sent to the remote PC as
	; well as the screen
	mov	cs:[echoBack], 1
	mov	ax, cs
	mov	ds, ax
	mov	es, ax
	; first set up the DTA to our FCB structure
	mov	ah, 1ah
	mov	dx, offset ffn
	int	21h

	mov	ah, 4eh
	mov	cx, 0ffh
	mov	dx, offset fileSpec
	int	21h		; find first file in directory
findFileLoop:
	; first do a carridge return
	pushf
	cmp	cs:[echoBack], 0
	jz	afterReturn
	mov	al, ';'
	call	ComWrite
afterReturn:		
	mov	al, '\n'
	call	ScrWrite
	mov	al, '\r'
	call	ScrWrite
	popf
	jc	done		; else go home...nothing found

	; get the length of the filename
	mov	cx, size ffn.FFN_filename
	mov	bx, offset ffn.FFN_filename
	mov	al, ' '				; pad character
	call	ScrWritePaddedStringLJ
	mov	al, ffn.FFN_attributes	
	test	al, mask FA_DIRECTORY
	jnz	doDirectory
	; write out the file size
	mov	dx, ffn.FFN_fileSize.high
	mov	ax, ffn.FFN_fileSize.low
	mov	bx, offset general_buf
	call	ScrDumpDecimal
	mov	cx, 9
	mov	al, ' '
	call	ScrWritePaddedStringRJ
	mov	al, ' '
	call	ScrWrite
doDate:
	; date:  low 5 bits = day, middle 4 bits = month. rest = year
	mov	ax, ffn.FFN_date
;month
	and	ax, mask DR_MONTH
	mov	cl, offset DR_MONTH
monthLoop:
	shr	ax
	loop	monthLoop
	clr	dx
	mov	bx, offset general_buf
	call	ScrDumpDecimal
	mov	cx, 2
	mov	al, '0'
	call	ScrWritePaddedStringRJ
	mov	al, '-'
	call	ScrWrite
;day
	mov	ax, ffn.FFN_date
	and	ax, mask DR_DAY
	clr	dx
	mov	bx, offset general_buf
	call	ScrDumpDecimal
	mov	cx, 2
	mov	al, '0'
	call	ScrWritePaddedStringRJ
	mov	al, '-'
	call	ScrWrite
;year
	mov	ax, ffn.FFN_date
	and	ax, mask DR_YEAR
	mov	cl, offset DR_YEAR
yearLoop:
	shr	ax
	loop	yearLoop
	add	ax, 80		; based on 1980
	clr	dx
	mov	bx, offset general_buf
	call	ScrDumpDecimal
	mov	al, '0'
	mov	cx, 2
	call	ScrWritePaddedStringRJ
	mov	al, ' '
	call	ScrWrite

	mov	ax, ffn.FFN_time
	and	ax, mask TR_HOURS
	mov	cl, offset TR_HOURS
hourLoop:
	shr	ax
	loop	hourLoop
	mov	bx, 'a'
	cmp	ax, 12
	jl	printHour
	sub	ax, 12
	mov	bx, 'p'
printHour:
	tst	ax
	jnz	printHour2
	mov	ax, 12
printHour2:
	push	bx		; same am/pm marker for end of time string
	clr	dx
	mov	bx, offset general_buf
	call	ScrDumpDecimal
	mov	al, ' '
	mov	cx, 2
	call	ScrWritePaddedStringRJ
	mov	al, ':'
	call	ScrWrite
;minutes
	mov	ax, ffn.FFN_time
	and	ax, mask TR_MINUTES
	mov	cl, offset TR_MINUTES
minuteLoop:
	shr	ax
	loop	minuteLoop
	mov	bx, offset general_buf
	clr	dx
	call	ScrDumpDecimal
	mov	al, '0'
	mov	cx, 2
	call	ScrWritePaddedStringRJ
	pop	ax			; get am/pm char
	call	ScrWrite
findNext:
	; now find next file
	mov	ah, 4fh
	int	21h
	jmp	findFileLoop
done:
	call	ScrCmdEndCommon
	.leave
	ret
doDirectory:
	mov	bx, offset dirString
	call	ScrWriteString
	mov	bx, offset spaceString
	call	ScrWriteString
	jmp	doDate
ScrLs	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrCmdEndCommon
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	common end stuff

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrCmdEndCommon	proc	far
	.enter
	cmp	cs:[echoBack], 0
	je	noEcho
	; normally we don't want to echo back so reset this variable
	mov	cs:[echoBack], 0
	; a semi-colon will indicate an end of data
	mov	al, ';'
	call	ComWrite
noEcho:
	mov	al, '\r'
	call	ScrWrite
	mov	al, '\n'
	call	ScrWrite
	call	ScrWriteUserString
	call	ComDrainQueue

	.leave
	ret
ScrCmdEndCommon	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWritePaddedStringLJ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	write a string with padding (left justifeid)

CALLED BY:	GLOBAL

PASS:		cx = total length of space
		[bx] = string
		al = pad character

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/15/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrWritePaddedStringLJ	proc	near
	uses	si, bx
	.enter
	mov	si, bx
	push	ax
findLength:
	lodsb
	cmp	al, ' '
	je	gotLength
	cmp	al, '\0'
	je	gotLength
	loop	findLength
gotLength:
	; now write out the filename
	push	cx
	call	ScrWriteString
	pop	cx
	pop	bx
	jcxz	done
padLoop:
	mov	al, bl			; get padding character
	push	cx
	call	ScrWrite
	pop	cx
	loop	padLoop
done:
	.leave
	ret
ScrWritePaddedStringLJ	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrWritePaddedStringRJ
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	write a string with padding (right justifeid)

CALLED BY:	GLOBAL

PASS:		cx = total length of space
		[bx] = string
		al = pad character

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/15/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrWritePaddedStringRJ	proc	near
	uses	si, bx
	.enter
	mov	si, bx
	push	ax
findLength:
	lodsb
	cmp	al, ' '
	je	gotLength
	cmp	al, '\0'
	je	gotLength
	loop	findLength
gotLength:
	pop	ax
	push	bx
	mov	bl, al
	jcxz	writeString
padLoop:
	mov	al, bl			; get padding character
	push	cx
	call	ScrWrite
	pop	cx
	loop	padLoop
writeString:
	pop	bx
	; now write out the filename
	call	ScrWriteString
done:
	.leave
	ret
ScrWritePaddedStringRJ	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrGetTwoArgs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get two args back to back in general_buf

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		general_buf contains two arguments
		the first one starts at cs:general_buf
		the second one at cs:[dx]

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/13/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrGetTwoArgs	proc	far
	.enter
	mov	di, offset general_buf
	mov	si, size general_buf
	call	ScrGetArg		; get old name
	mov	dx, di			; save start of new name in ds:dx
	mov	si, offset general_buf
	add	si, size general_buf
	sub	si, di
	call	ScrGetArg
	.leave
	ret
ScrGetTwoArgs	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrGetArg
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	get an arument from the com port

CALLED BY:	GLOBAL

PASS:		es:di = destination for argument
		si = max size of argument

RETURN:		carry set on error
		else es:di = one character past end of arg

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrGetArg	proc	far
	.enter
readLoop:
	cmp	si, di
	je	done
	call	ComReadWithWait
	jc	done
	cmp	al, '!'
	je	gotit
	stosb
	jmp	readLoop
gotit:
	clr	al		; store a zero for the end-of-arg '!'
	stosb
	clc
done:
	.leave
	ret
ScrGetArg	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrCd
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	change the current directory

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	read in the directory name and do it

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	11/ 4/92		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

;dirname	db 130	dup(0) ; big enough for two 64 byte arguments plus nulls
driveError db "Invalid Drive.", 0

ScrCd	proc	near
	.enter
	mov	cs:[echoBack], 1
	mov	di, offset general_buf
	mov	si, size general_buf
	call	ScrGetArg
	dec	di
	cmp	di, offset general_buf
	je	getCurrentDir
	mov	si, offset general_buf
	inc	si			; es:si = second character in name
	cmp	{byte}es:[si], ':'
	je	changeDir
	mov	{byte}es:[di], 0
	mov	ah, 3bh
	mov	dx, offset general_buf
	int	21h
	jc	fileError
getCurrentDir:
	mov	ah, 19h			; get defualt disk drive
	int	21h
	add	al, 'A'
	mov	di, offset general_buf
	stosb
	mov	al, ':'
	stosb
	mov	al, '\\'
	stosb
	mov	si, di 
	mov	ah, 47h
	clr	dl
	int	21h			; get current direrctory
; now print the current directory
	mov	al, '\r'
	call	ScrWrite
	mov	al, '\n'
	call	ScrWrite
	mov	bx, offset general_buf
	call	ScrWriteString
	mov	al, '>'
	call	ScrWrite
	jmp	done
changeDir:
	inc	si
	mov	{byte}es:[si], 0
	sub	si, 2
	mov	dl, es:[si]
	cmp	dl, 'a'
	jl	haveCap
	sub	dl, 'a'

	add	dl, 'A'
haveCap:
	sub	dl, 'A'
	mov	ah, 0eh			; set current drive
	int	21h		
	cmp	al, dl
	jge	getCurrentDir
	mov	bx, offset driveError
	call	ScrWriteString
	jmp	getCurrentDir
done:
	call	ScrCmdEndCommon
	.leave
	ret
fileError:
	call	ScrFileError
	jmp	done
ScrCd	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrFileError
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	

CALLED BY:	GLOBAL

PASS:		ax = file error
			2 = file not found
			3 = path not found
			5 = access denied
		if the writeOnly flag is set us that string rather than
		the value in ax

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/16/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
CannotWriteString	db	'\r', "Access denied, drive read only.",'\r','\n',0
writeError	byte	

ScrFileError	proc	far
	cmp	cs:[writeError], 0
	je	doNormal
	mov	cs:[writeError], 0
	mov	bx, offset CannotWriteString
	jmp	printError
doNormal:
	mov	bx, offset FileNotFoundString
	cmp	ax, 2
	je	printError
	mov	bx, offset PathNotFoundString
	cmp	ax, 3
	je	printError
	mov	bx, offset AccessDeniedString
	cmp	ax, 5
	je	printError
printError:
	call	ScrWriteString
	.leave
	ret
ScrFileError	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrMkDir
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	make a directory

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
PathNotFoundString	db	"Path not found.", 0
FileNotFoundString	db	"File not found.", 0
AccessDeniedString	db	"Access denied.", 0
CurrentDirectoryString	db	"Current Directory.", 0
DirectoryMadeString	db	"Directory Made", 0
DirectoryRemovedString	db	"Directory Removed", 0
FileRemovedString	db	"File Removed", 0
FileRenamedString	db	"File Renamed", 0
FileCopiedString	db	"File Copied", 0

ScrMkDir	proc	near
	.enter
	mov	cs:[echoBack], 1
	mov	di, offset general_buf
	mov	si, size general_buf
	call	ScrGetArg
	mov	ah, 39h
	mov	dx, offset general_buf
	int	21h
	jc	mkDirError
	mov	bx, offset DirectoryMadeString
	call	ScrWriteString
done:
	call	ScrCmdEndCommon
	.leave
	ret
mkDirError:
	call	ScrFileError
	jmp	done
ScrMkDir	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRmDir
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	delete a directory

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrRmDir	proc	near
	.enter
	mov	cs:[echoBack], 1
	mov	di, offset general_buf
	mov	si, size general_buf
	call	ScrGetArg
	mov	ah, 3ah
	mov	dx, offset general_buf
	int	21h
	jc	rmDirError
	mov	bx, offset DirectoryRemovedString
	call	ScrWriteString
done:
	call	ScrCmdEndCommon
	.leave
	ret
rmDirError:
	call	ScrFileError
	jmp	done
ScrRmDir	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRmFile
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	delete a file

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrRmFile	proc	near
	.enter
	mov	cs:[echoBack], 1
	mov	di, offset general_buf
	mov	si, size general_buf
	call	ScrGetArg
	mov	ah, 41h
	mov	dx, offset general_buf
	int	21h
	jc	rmFileError
	mov	bx, offset FileRemovedString
	call	ScrWriteString
done:
	call	ScrCmdEndCommon
	.leave
	ret
rmFileError:
	call	ScrFileError
	jmp	done
ScrRmFile	endp

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrRenameFile
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Rename a file

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrRenameFile	proc	near
	.enter
	mov	cs:[echoBack], 1
	call	ScrGetTwoArgs		; es:dx = old name
	mov	ah, 56h
	mov	di, offset general_buf	; es:di = new name
	xchg	di, dx			; es:dx = new name, es:di = old name
	int	21h
	jc	rnError
	mov	bx, offset FileRenamedString
	call	ScrWriteString
done:
	call	ScrCmdEndCommon
	.leave
	ret
rnError:
	call	ScrFileError
	jmp	done
ScrRenameFile	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrCopyFile
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	copy a file

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/13/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
general_buf	db	BUFFER_SIZE dup(0)
ScrCopyFile	proc	near
src 	local	word
dest	local	word
	.enter
	mov	cs:[echoBack], 1
	call	ScrGetTwoArgs		; es:[dx] = destination file/path
	mov	ah, 3ch			; create/truncate FILE
	clr	cx
	int	21h
	jc	doFileError
	mov	dest, ax			; save file handle
	mov	dx, offset general_buf	; es:[di] = source file
	mov	al, 10100000b	; read/read access/no inheritence
	mov	ah, 3dh			; open file
	int	21h
	jc	doFileError
	mov	src, ax
	mov	dx, offset general_buf
copyloop:
	mov	bx, src
	mov	ah, 3fh			; file read
	mov	cx, size general_buf
	int	21h
	jc	doFileError
	mov	cx, ax		; number of bytes actually read
	mov	ah, 40h
	mov	bx, dest
	int	21h
	jc	doFileError
	cmp	ax, size general_buf
	je	copyloop
	mov	ah, 3eh		; close file
	mov	bx, dest
	int	21h		; close dest
	jc	doFileError
	mov	ah, 3eh
	mov	bx, src
	int	21h		; close src
	jc	doFileError
	mov	bx, offset FileCopiedString
	call	ScrWriteString
done:
	call	ScrCmdEndCommon
	.leave
	ret
doFileError:
	call	ScrFileError
	jmp	done
ScrCopyFile	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrExitToDos
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	exit to Dos

CALLED BY:	GLOBAL

PASS:		nothing

RETURN:		Void.

DESTROYED:	Nada.

PSEUDOCODE/STRATEGY:	

KNOWN BUGS/SIDEFFECTS/IDEAS:

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jimmy	4/12/93		Initial version.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@

ScrExitToDos	proc	near
	call	ScrExit
	call	ComExit
	call	KbdExit
	call	MouseEnd
	mov	al, 0		; exit(0)
	mov	ah, 4ch
	int	21h
	.unreached
ScrExitToDos	endp
	public ScrExitToDos

COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrGetCardType
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Get the type of the graphics card that is attached to this
		system so that we can call the correct routines for screen
		manipulation.
CALLED BY:	ScrInit
PASS:		nothing
RETURN:		Carry set if found a supported card.
		scrType = the type of card.
			TYPE_UNKNOWN	0
			TYPE_MDA	1	; Unused
			TYPE_CGA	2	; 
			TYPE_EGA	3
			TYPE_MCGA	4	; Unused
			TYPE_VGA	5
			TYPE_HGC	80h
			TYPE_HGCPLUS	81h	; Unused
			TYPE_INCOLOR	82h	; Unused
		scrLines = lines of characters on the screen
		scrAttr = initial attribute
		scrBot = scrLines-1
		scrMode = previous video mode
		scrCols = previous number of columns
DESTROYED:	nothing

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	1/12/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
hgcPlusFlag	db	'h',0		; flag for 'do hgc plus' init.

		public	ScrFindCard
ScrFindCard	proc	near
		mov	scrEtcFlags, 0
		;
		; Check for VGA.
		;
		call	ScrFindVGA
		jnc	SFCE
		mov	scrType, TYPE_VGA
		call	ScrInitVGA
		jmp	short SFCCommon
SFCE:		
		;
		; Get the current video mode and save it for later
		; restoration.
		;
		mov	ax, 0f00h
		int	10h
		mov	scrCols, ah		; Save old column #.
		mov	scrMode, al		; Save old video mode.

		;
		; Check for EGA present.
		;
		call	ScrFindEGA
		jnc	SFC2
		mov	scrType, TYPE_EGA
		call	ScrInitEGA
		jmp	SFCCommon
SFC2:
		;
		; Check for Hercules card present.
		;
		call	ScrFindHerc	;
		jnc	SFC3
		;
		; Assume doing regular hercules initialization
		;
		mov	scrType, TYPE_HGC
		push	bp
		mov	bp, offset ScrInitHercules
		;
		; Look for /h flag
		;
		mov	di, offset hgcPlusFlag
		clr	bx
		call	FetchArg
		tst	bx
		jz	SFC2_5
		;
		; /h present -- init as HGC+
		;
		mov	bp, offset ScrInitHGCPlus
		mov	scrType, TYPE_HGCPLUS
SFC2_5:
		call	bp
		pop	bp
		jmp	short SFCCommon

SFC3:
		; 
		; Check for CGA card
		;
		call	ScrFindCGA		;
		jnc	SFCUnknown	 	; if not, we can't id the card
		mov	scrType, TYPE_CGA
		call	ScrInitCGA
		;
		; Fall through to common stuff
		;
SFCCommon:
		;
		; Set the initial attribute to be the normal one for the screen
		;
		mov	al, scrNormal
		mov	scrAttr, al
		
		;
		; Figure out how many lines are being displayed...
		;
		push	es
		clr	ax
		mov	es, ax
		mov	al, es:VIDNUMROWS
		cbw
		pop	es
		;
		; Sometimes, the rows gets zeroed in the BIOS. This only
		; happens on CGA and HGC, however, so if it's < 5, we assume
		; it's 25 lines total (KLUDGE).
		;
		cmp	ax, 5
		jge	SFI4
		mov	ax, 24
SFI4:
		mov	scrBot, ax		; That's what we need...
		inc	ax			; Plus 1 for whole screen
		mov	scrLines, ax		; Save that
		
		stc				; Signal card found
		ret

SFCUnknown:
		mov	ax, TYPE_UNKNOWN
		ret
ScrFindCard	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrFindHerc
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check for the existence of a hercules card.

CALLED BY:	ScrFindCard
PASS:		nothing
RETURN:		carry set if a Hercules card is present.
DESTROYED:	AX, CX, DX

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	1/12/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrFindHerc
ScrFindHerc	proc	near
		mov	dx,HGCADDRPORT	; dx <- crtc address of hercules
		call	ScrFind6845	; Try to find the crtc.
		jnc	SFH_NotFound	; 6845 exists on a hercules.
		;
		; We know it is either an MDA or an HGC.
		; Check for the sync bit of the status port changing.
		; If it does change then this is an HGS (hercules) card.
		;
		mov	dx,3bah		; dx <- status port address
		in	al,dx		; al <- value of status byte
		and	al,80h		; only interested in the sync bit.
		mov	ah,al		; ah <- bit 7 (corresponds to vertical
					;    sync bit on hercules card).
		mov	cx,8000h	; loop for a long time.
SFH_loop:			; Loop, waiting for change in sync bit.
		in	al,dx		; get status byte again.
		and	al,80h		; check status bit
		cmp	ah,al		; check for difference.
		loope	SFH_loop	; loop until bit changes or cx = 0.
		je	SFH_NotFound	; if bit hasn't changed then this is
					; not a Hercules card
		; else fall thru to signal we have found a hercules card.
SFH_Found:			; Hercules found
		stc			; Signal found
		ret			;
SFH_NotFound:			; Hercules not found
		clc			; Signal not found
		ret			;
ScrFindHerc	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrFindCGA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check for the existence of a CGA card.

CALLED BY:	ScrFindCard
PASS:		nothing
RETURN:		carry set if a CGA card is present.
DESTROYED:	AX, CX, DX

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	mav	1/26/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrFindCGA
ScrFindCGA	proc	near
		mov	dx,EGAADDRPORT	; dx <- crtc address of hercules
		call	ScrFind6845	; Try to find the crtc.
		;
		; this returns the carry correctly so just return
		;
		ret
ScrFindCGA	endp



COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrFind6845
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Checks for the existence of the crtc that is in the Hercules
		card.

CALLED BY:	ScrFindHerc
PASS:		nothing
RETURN:		carry set if the 6845 is found
DESTROYED:	

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	1/12/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrFind6845
ScrFind6845	proc	near
		mov	al,0fh		;
		out	dx,al		; select 6845 register f (crsr low)
		inc	dx		;
		in	al,dx		; al <- current crsr low
		mov	ah,al		; save it.
		mov	al,66h		; trash value
		out	dx,al		; try to write it
		mov	cx,100h		; loop value
SF6_loop:				;
		loop	SF6_loop	; Spin wheels waiting for bit to change
		in	al,dx		;
		xchg	ah,al		; ah <- new value, al <- old value.
		out	dx,al		; restore original value.
		cmp	ah,66h		; Check for register change.
		je	SF6_Found	;
		clc			; Signal : not found.
		ret			;
SF6_Found:				;
		stc			; Signal : found.
		ret			;
ScrFind6845	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrFindEGA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check for the existence of an EGA card.

CALLED BY:	ScrGetCardType
PASS:		nothing
RETURN:		carry set if EGA card is found.
DESTROYED:	BL, AH, ...

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jcw	1/12/89		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrFindEGA
ScrFindEGA	proc	near
		mov	bl,10h		; 10h == return ega info
		vidcall	VIDMISC		; if bl returns unchanged then there
					; is no EGA present.
		cmp	bl,10h		;
		jne	SFE_Found	;
		; Not found if bl has not changed. (carry already clear)
		ret			;
SFE_Found:				;
		stc			; Found if bl has changed.
		ret			;
ScrFindEGA	endp


COMMENT @%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ScrFindVGA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SYNOPSIS:	Check for the existence of an VGA card.

CALLED BY:	ScrGetCardType
PASS:		nothing
RETURN:		carry set if VGA card is found.
DESTROYED:	BL, AH, ...

REVISION HISTORY:
	Name	Date		Description
	----	----		-----------
	jdm	1/25/91		Initial version

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%@
		public	ScrFindVGA
ScrFindVGA	proc	near
		mov	ax, 1A00h	; Read Display Combo. Code
		int	10h
		cmp	al, 1Ah		; == iff VGA
		je	SFV_Found	;
		clc
		ret			;
SFV_Found:				;
		stc			;
		ret			;
ScrFindVGA	endp
program		ends

		end


